-- Vital 2.1
--
import SS
--
-- LOCN 650 75 true
-- STYLE 
{-
Actions
-}
-- LOCN 50 75 true
-- STYLE title
{-
Demo of actions and the ActionLib functions
-}
-- LOCN 50 -50 true
-- STYLE abstract
--
import ActionLib
--
-- LOCN 50 150 true
-- STYLE 
{-
An action is a value of type Action (similar to the IO () type of standard Haskell).
The function
    display :: String -> Action
yields an action that, when executed, displays the given string in a pop-up dialog window.
-}
-- LOCN 50 200 true
-- STYLE comment
--
act1, act2 :: Action
--
-- LOCN 50 325 true
-- STYLE 
--
act1 = display "Hello"
--
-- LOCN 50 375 true
-- STYLE 
--
act2 = display "Goodbye"
--
-- LOCN 400 375 true
-- STYLE 
{-
An action can be executed by associating it with a PicAction value (which displays as
a rectangle) and then double-clicking the rectangle.
-}
-- LOCN 50 475 true
-- STYLE comment
{-
A PicAction value is created using the constructor
        PicAction :: Int -> Int -> Color -> Color -> Action -> Pic
-}
-- LOCN 50 550 true
-- STYLE comment
--
pic =
  let {width = 80;
       height = 20;
       bodyColor = lightGray;
       edgeColor = red
  } in PicAction width height bodyColor edgeColor act1
--
-- LOCN 50 625 true
-- STYLE 
{-
Try double-clicking on this rectangle
to cause a dialog window to appear.
-}
-- LOCN 575 700 true
-- STYLE alert2
{-
The ActionLib module defines a function
        button :: String -> Action -> Pic
that uses the PicAction constructor to form a button-like picture for executing an action:
-}
-- LOCN 50 800 true
-- STYLE comment
--
button "Hello" act1
--
-- LOCN 50 900 true
-- STYLE 
--
button "Goodbye" act2
--
-- LOCN 325 900 true
-- STYLE 
{-
Try double-clicking on these buttons
to execute their associated actions.
-}
-- LOCN 575 900 true
-- STYLE alert2
{-
Sequencing and delaying actions
-}
-- LOCN 50 1075 true
-- STYLE heading1
{-
Compound actions may be formed using the sequencing operator
        (>>) :: Action -> Action -> Action 
-}
-- LOCN 50 1125 true
-- STYLE comment
{-
For example:
-}
-- LOCN 50 1200 true
-- STYLE comment
--
act12 = act1 >> act2
--
-- LOCN 50 1275 true
-- STYLE 
--
act21 = act2 >> act1
--
-- LOCN 575 1275 true
-- STYLE 
--
button "Hello-goodbye" act12
--
-- LOCN 50 1350 true
-- STYLE 
--
button "Goodbye-hello" act21
--
-- LOCN 575 1350 true
-- STYLE 
{-
The execution of an action may be delayed using the function
        delay :: Time -> Action -> Action     
(with the delay time is expressed in ms).
-}
-- LOCN 50 1500 true
-- STYLE comment
{-
For example:
-}
-- LOCN 50 1600 true
-- STYLE comment
--
act1d = delay 2000 act1
--
-- LOCN 51 1648 true
-- STYLE 
--
button "Delayed hello" act1d
--
-- LOCN 50 1725 true
-- STYLE 
{-
(By default, an action value is 
 displayed as a string describing
 the effect of the action.)
-}
-- LOCN 700 375 true
-- STYLE comment
--
let {act1 = display "One";
     act2 = display "Two";
     act3 = display "Three";
     act = act1 >> delay 1000 (act2 >> delay 1000 act3)
  } in button "Counter" act
--
-- LOCN 50 1825 true
-- STYLE 
{-
Back to previous document
-}
-- LOCN 75 2725 true
-- STYLE open1 ""
{-
Actions for controlling the Midi synthesiser
-}
-- LOCN 50 2025 true
-- STYLE heading1
{-
(Note that not all environments support Midi synthesis.)
-}
-- LOCN 75 2075 true
-- STYLE commentB
{-
Actions can be used to control the playing of Midi sequences. For
example:
-}
-- LOCN 75 2150 true
-- STYLE comment
--
import Midi
--
-- LOCN 75 2225 true
-- STYLE 
--
play :: String -> Pic
--
-- LOCN 75 2275 true
-- STYLE 
--
play name =
  let {seq = midiFile (name ++ ".mid");
       act = playMidi seq
  } in button ("Play " ++ name) act
--
-- LOCN 75 2300 true
-- STYLE 
--
play "enyafair"
--
-- LOCN 75 2400 true
-- STYLE 
--
button "Stop" stopMidi
--
-- LOCN 450 2400 true
-- STYLE 
--
play "bach"
--
-- LOCN 75 2500 true
-- STYLE 
--
play "furElise"
--
-- LOCN 75 2600 true
-- STYLE 
{-
Back to previous document
-}
-- LOCN 650 125 true
-- STYLE open1 ""
{-
With acknowledgements to
unknown Midi arrangers
for these three compositions.
-}
-- LOCN 450 2525 true
-- STYLE commentB
