-- Vital 2.1
--
import SS
--
-- LOCN 625 50 true
-- STYLE 
{-
Principles of Animation
-}
-- LOCN 50 50 true
-- STYLE title
--
box w = let x = (w `div` 10) `mod` 500
        in Rect x 20 yellow red 
--
-- LOCN 50 525 true
-- STYLE Hide
--
box :: Int -> Pic
--
-- LOCN 50 500 true
-- STYLE 
{-
Picture values constructed using the "Animation" constructor of the Pic datatype:

        data Pic = 
                   NoPic
                 | Super Pic Pic
                  . . .
                 | Animation (Int -> Pic)

are rendered as animated pictures.
-}
-- LOCN 50 125 true
-- STYLE commentM
{-
When the Vital interpreter encounters a value of the form
        Animation f
in conjunction with the "Picture" style,  the function  f  is repetitively applied to the
current real time, yielding a sequence of Pic values which are displayed in real time.
-}
-- LOCN 50 325 true
-- STYLE comment
{-
For example, here is a Pic-valued function on the integers:
-}
-- LOCN 50 450 true
-- STYLE comment
{-
This function takes its argument, divides it by 10 and reduces it modulo 500, and then
constructs a rectangle of that width. For example:
-}
-- LOCN 50 600 true
-- STYLE comment
--
box 1000
--
-- LOCN 47 666 true
-- STYLE Picture
--
box 8000
--
-- LOCN 300 665 true
-- STYLE Picture
--
box 5100
--
-- LOCN 697 664 true
-- STYLE Picture
{-
The same function, wrapped in the Animation constructor, is instead applied to the
current value of real time (by convention measured in milli-seconds since 1970) and hence
the width of the rectangle it draws will depend on the time at which it is rendered. For
example:
-}
-- LOCN 50 775 true
-- STYLE comment
--
Animation box
--
-- LOCN 48 882 true
-- STYLE Picture
{-
Try clicking or dragging the mouse. Since this caues the document to be re-rendered, it causes
the "box" function to be re-evaluated with the current time.
-}
-- LOCN 50 975 true
-- STYLE comment
{-
Now press the Animate button. This causes the document to be re-rendered at regular intervals
(at about the frame rate of the computer display) and so yields the impression of a smoothly
changing animation.
-}
-- LOCN 50 1050 true
-- STYLE comment
{-
Changing the rate of an animation
-}
-- LOCN 50 1175 true
-- STYLE heading1
{-
The rate at which an animation evolves may be modified by pre-composing the function with
an integer-scaling function. For example:
-}
-- LOCN 50 1225 true
-- STYLE comment
--
slowDown, speedUp :: Int -> Int
--
-- LOCN 43 1307 true
-- STYLE 
--
slowDown = (`div` 3)
--
-- LOCN 43 1342 true
-- STYLE Hide
--
speedUp = (* 2)
--
-- LOCN 285 1340 true
-- STYLE Hide
--
Animation (box . slowDown)
--
-- LOCN 43 1460 true
-- STYLE Picture
--
Animation (box . speedUp)
--
-- LOCN 42 1541 true
-- STYLE Picture
--
Animation box
--
-- LOCN 43 1386 true
-- STYLE Picture
{-
(Notice that scaling the time function in this way is completely independent of, and does not
affect, the refresh rate of the animation.)
-}
-- LOCN 50 1650 true
-- STYLE comment
{-
Animated textual information
-}
-- LOCN 50 1775 true
-- STYLE heading1
--
fmt = Format SansSerif 24 red False False
--
-- LOCN 593 1959 true
-- STYLE Hide
--
Animation (\i -> let s = showInt i " ms"
                 in PicText fmt s)

--
-- LOCN 50 1925 true
-- STYLE Picture
--
import Numeric
--
-- LOCN 593 1935 true
-- STYLE 
{-
Any aspect of a picture may be animated. For example, here the "showInt" function is
used to convert real time to a string and the PicText constructor to convert the string
to a picture:
-}
-- LOCN 50 1825 true
-- STYLE comment
{-
Assuming the Animate button has been pressed, this will show the number of milli-seconds
since the start of 1970. (The least significant digit is likely to be unreliable due to
stroboscopic effects.)
-}
-- LOCN 50 2025 true
-- STYLE comment
{-
Animated images
-}
-- LOCN 25 2550 true
-- STYLE heading1
--
pics = let { prefix = "dukes/T";
             suffix = ".gif";
             names  = [prefix ++ showInt i suffix | i <- [1 .. nPics]]
       } in  map (PicImage . imageFile) names
--
-- LOCN 25 2850 true
-- STYLE bareList 100 0 Picture
--
Animation  ((pics !!) . (`mod` nPics) . (`div` 100))
--
-- LOCN 41 3122 true
-- STYLE Picture
--
nPics = 10
--
-- LOCN 25 2775 true
-- STYLE Hide
{-
The Duke image is a registered trademark
of Sun Microsystems Inc.
-}
-- LOCN 700 3025 true
-- STYLE comment
{-
A film-strip animation is easily achieved; it is simply necessary to sequence through
a sequence of images in rapid succession.
-}
-- LOCN 25 2625 true
-- STYLE comment
{-
For example, these declarations read in (from file) a list of ten images (in .gif format)
and convert them (using the PicImage constructor) to a list of pictures:
-}
-- LOCN 25 2725 true
-- STYLE comment
--
pics :: [Pic]
--
-- LOCN 25 2825 true
-- STYLE 
{-
The following expression displays the pictures in this list in 
succession (one every 100 ms):
-}
-- LOCN 50 3050 true
-- STYLE comment
{-
Back to previous document
-}
-- LOCN 50 3300 true
-- STYLE open ""
--
Animation (\i -> let s = showTime 0 i
                 in PicText fmt s)

--
-- LOCN 50 2425 true
-- STYLE Picture
{-
A rather more useful display can be created using the function
        showTime :: Int -> Int -> String
(defined in the Numeric module) such that
        showTime  offset  n
yields a string describing the day number and the time of day of the time n milli-seconds after 
the start of 1970  (where offset is an offset, in hours, relative to GMT).  For instance:

-}
-- LOCN 50 2125 true
-- STYLE comment
--
showTime 0 0
--
-- LOCN 25 2300 true
-- STYLE 
--
showTime 0 1000000000
--
-- LOCN 462 2291 true
-- STYLE 
{-
Using this function, the day number and the time of day can be displayed:
-}
-- LOCN 50 2375 true
-- STYLE comment
{-
Back to previous document
-}
-- LOCN 625 75 true
-- STYLE open ""
{-
Reading from right to left, the animated function:
   - scales real time (measured in ms) by a factor of 100;
   - reduces it modulo the number of pictures in the sequence;
   - uses "!!" to select the i-th picture from the list of pictures.
-}
-- LOCN 500 3175 true
-- STYLE comment
{-
Simple examples illustrating the use of the Animation picture constructor
for displaying animated pictures.
-}
-- LOCN 50 -50 true
-- STYLE abstract
