-- Vital 2.1
--
import SS
--
-- LOCN 525 25 true
-- STYLE 
--
import Graphics
--
-- LOCN 525 50 true
-- STYLE 
{-
The Graphics module
-}
-- LOCN 50 50 true
-- STYLE title
{-
The Graphics library module provides a collection of functions for drawing and animating
geometric pictures, including in particular the function:
        animate :: Double -> (Double -> Pic) -> Pic
such that
        animate  f  fn
yields an animated representation of the picture-producing function fn, evolving at the
rate of 2*pi*f  units/s.
-}
-- LOCN 50 100 true
-- STYLE comment
--
polar theta = let r = 50.0 
              in line 0.0 0.0 (r * cos theta) (r * sin theta) red
--
-- LOCN 51 462 true
-- STYLE Hide
{-
For instance:
-}
-- LOCN 50 550 true
-- STYLE comment
--
polar  0.0
--
-- LOCN 51 621 true
-- STYLE Picture
{-
Now, using the "animate" function described above, a vector rotating at  1.0 Hz can be
drawn by:
-}
-- LOCN 50 775 true
-- STYLE comment
--
vec = animate 1.0 polar
--
-- LOCN 50 860 true
-- STYLE Picture
{-
. . . and, likewise, vectors rotating at frequencies of -1.0 Hz and 0.25 Hz can be drawn by:
-}
-- LOCN 50 975 true
-- STYLE comment
--
animate (-1.0) polar
--
-- LOCN 50 1066 true
-- STYLE Picture
--
animate 0.25 polar
--
-- LOCN 377 1065 true
-- STYLE Picture
{-
For example, to create a picture of a rotating vector (of length 50) first define a
function to draw such a vector at an angle of theta:
-}
-- LOCN 50 375 true
-- STYLE comment
--
polar (pi / 4.0)
--
-- LOCN 275 624 true
-- STYLE Picture
{-
This function makes use of the function:
        line x y x' y' c
defined in the Graphics module that draws a line in colour c
from (x,y) to (x',y')
-}
-- LOCN 600 450 true
-- STYLE comment
{-
Rotating vectors
-}
-- LOCN 50 325 true
-- STYLE heading1
{-
NB: Press the Animate button in the toolbar in order to
animate these diagrams
-}
-- LOCN 400 850 true
-- STYLE comment
{-
Gear wheels
-}
-- LOCN 50 1225 true
-- STYLE heading1
--
bigWheel = gearWheel 50.0 60.0 white red 150.0 600.0 100.0
--
-- LOCN 50 1525 true
-- STYLE Hide
{-
The Graphics library includes a function
        gearWheel :: Double -> Double -> Color -> Color -> Double -> Double -> Double -> Pic
such that
        gearWheel td tw bc ec radius x y phase
yields a picture of a gear wheel where:  td is the tooth depth, tw is the tooth width, 
bc is the body colour, ec is the edge colour, radius is the average radius, x and y
are the coords of its centre and phase is the angular position of the wheel (where 2*pi 
radians corresponds to a rotation of one tooth of the wheel).
-}
-- LOCN 50 1275 true
-- STYLE comment
{-
For example, this function
-}
-- LOCN 50 1450 true
-- STYLE comment
--
bigWheel :: Double -> Pic
--
-- LOCN 50 1500 true
-- STYLE 
{-
takes a phase angle and yields a picture of a fairly large, coarse-toothed wheel
-}
-- LOCN 50 1575 true
-- STYLE comment
{-
For instance:
-}
-- LOCN 50 1625 true
-- STYLE comment
--
pic1 = bigWheel pi
--
-- LOCN 50 1675 true
-- STYLE Picture
{-
Now, using the "animate" function, a rotating version of the same
wheel can be created:
-}
-- LOCN 50 2050 true
-- STYLE comment
--
pic2 = animate 0.5 bigWheel
--
-- LOCN 50 2100 true
-- STYLE Picture
{-
Rotating gear wheels
-}
-- LOCN 50 2000 true
-- STYLE heading1
{-
Though the mills of God grind slowly,
    Yet they grind exceeding small;
Though with patience he stands waiting,
    With exactness grinds he all.









 Henry Longfellow
-}
-- LOCN 600 3000 true
-- STYLE comment
--
gearTrain :: Double -> Double -> Double -> Double -> [(Double, Color)] -> Pic
--
-- LOCN 50 2525 true
-- STYLE 
--
gearTrain toothWidth freq x y rcs =
  case rcs of {
    []   ->  NoPic;
    (radius,col) : rcs' -> 
             let { toothDepth = toothWidth * 0.6;
                   x1 = x + radius;
                   pic = animate freq (gearWheel toothDepth toothWidth col black radius x1 y);
                   pic' = gearTrain  toothWidth (- freq) x y rcs'
             } in pic  `Super` move (2.0 * radius) 0.0 pic'
  }
--
-- LOCN 50 2550 true
-- STYLE Hide
--
pic3 = gearTrain 10.0 0.5 50.0 150.0 [(60.0, darkGray), (100.0, lightGray)]
--
-- LOCN 50 2950 true
-- STYLE Picture
{-
Gear trains
-}
-- LOCN 50 2425 true
-- STYLE heading1
{-
Gear wheels are, of course, used in groups. The following function
creates a picture of a gear train:
-}
-- LOCN 50 2475 true
-- STYLE comment
--
sizes = map fromInt [10, 20 ..]
--
-- LOCN 50 3375 true
-- STYLE Hide
--
colors = cycle [magenta, green, white]
--
-- LOCN 50 3400 true
-- STYLE Hide
--
pic4 = gearTrain 12.0 1.0 50.0 80.0 (zip sizes colors)
--
-- LOCN 50 3450 true
-- STYLE Picture
{-
The arguments of this function are:
  - the width of each tooth (from which the depth is also derived)
  - the frequency (in teeth/s) of the animation
  - the X and Y coordinates of the start of the gear train
  - a list of the radii and colours of each gear wheel in the train.
the individual gear wheels in the train. 
-}
-- LOCN 50 2750 true
-- STYLE comment
{-
For example:
-}
-- LOCN 50 2900 true
-- STYLE comment
{-
As usual, pictures can be of unbounded extent . . .
-}
-- LOCN 50 3325 true
-- STYLE comment
{-
Back to previous document
-}
-- LOCN 50 3675 true
-- STYLE open ""
{-
Back to previous document
-}
-- LOCN 713 96 true
-- STYLE open ""
--
  Trans 10 (-70) $ 
  picFile 
  ("people/" ++
  "longfellow.jpg")
--
-- LOCN 600 3100 true
-- STYLE Picture
{-
Illustrates the use of the Graphics library for displaying animated pictures.
Includes several examples on gear wheels.
-}
-- LOCN 50 -75 true
-- STYLE abstract
