-- Vital 2.1
--
import SS
--
-- LOCN 575 25 true
-- STYLE 
--
import Graphics
--
-- LOCN 575 50 true
-- STYLE 
{-
Heat engine
-}
-- LOCN 50 50 true
-- STYLE title
{-
Using the simple shapes (rect, circle, bar, etc) defined in the Graphics library
animated diagrams illustrating simple mechanisms may be constructed. As an
illustration, here is a function that yields a picture of a simple heat engine:
-}
-- LOCN 50 125 true
-- STYLE comment
--
engine  theta =
  let { crankLength = 40.0;  conRodLength = 100.0;   pistonRodLength = 150.0; width = 5.0;
        cylinderRadius = 30.0; cylinderThickness = 10.0; pistonWidth = 20.0; pistonGap = 15.0;
        x1 = crankLength * cos theta;   y1 = crankLength * sin theta;
        x2 = x1 + sqrt ((conRodLength ^ 2) - (y1 ^ 2));
        x3 = x2 + pistonRodLength;
        x4 = conRodLength + pistonRodLength;
        link = \x1 y1 x2 y2 -> bar x1 y1 x2 y2 width gray black red white;
        crank = link 0.0 0.0 x1 y1;
        conRod = link x1 y1 x2 0.0;
        pistonRod = link x2 0.0 x3 0.0;
        piston = rect x3 0.0  pistonWidth cylinderRadius gray black;
        u1 = crankLength + pistonWidth + pistonGap;
        innerCylinder = rect x4 0.0 u1 cylinderRadius yellow yellow;
        outerCylinder = rect x4 0.0 (u1 + cylinderThickness) (cylinderRadius + cylinderThickness) black black;
        x5 = (x3 + x4 + u1) / 2.0;
        greenComp = round (100.0 * (1.0 - (x1 / crankLength)));
        gasColor = RGB 255 greenComp 50;
        gas = rect x5 0.0 ((x4 + u1) - x5) cylinderRadius gasColor gasColor
  } in super [outerCylinder, innerCylinder, pistonRod, conRod, crank, gas, piston]
--
-- LOCN 52 251 true
-- STYLE Hide
--
engine :: Double -> Pic
--
-- LOCN 52 224 true
-- STYLE 
{-
The single argument, theta, taken by the above function defines the angle of the engine's crankshaft.
This expression yields a series of snapshots showing the engine at different phases of its cycle:
-}
-- LOCN 50 700 true
-- STYLE comment
{-
(Aside: a good approach to developing functions with as much fine detail as this one is to develop
the function incrementally, adding components one by one, checking for correctness and adjusting the
aesthetics as the overall function is developed.)
-}
-- LOCN 50 625 true
-- STYLE comment
--
pics = [engine (pi / 4.0 * fromInt i) | i <- [0 ..]]
--
-- LOCN 51 767 true
-- STYLE Hide
--
pics
--
-- LOCN 50 825 true
-- STYLE bareList 400 0 Picture
{-
. . . and here is an animated view of the same engine (rotating
at a frequency of 0.1 Hz):
-}
-- LOCN 50 1000 true
-- STYLE comment
--
pic = animate 0.1 engine
--
-- LOCN 50 1130 true
-- STYLE Picture
{-
(As usual, press the Animate button 
 to animate the display.)
-}
-- LOCN 600 1125 true
-- STYLE warning2
--
engines f = let { pic1 = animate f engine;
                  pic2 = engines (1.2 * f)
            } in pic1 `Super` Trans 0 150 pic2
--
-- LOCN 52 1397 true
-- STYLE Hide
--
engines 0.5
--
-- LOCN 52 1537 true
-- STYLE Picture
{-
(These two views are duals: the first shows successive phases distributed
 in space, the second, distributed in time.)
-}
-- LOCN 600 1000 true
-- STYLE comment
{-
Often, animations will consist of a collection of animated components, each
running at its own speed. As a (rather contrived!) example:
-}
-- LOCN 50 1325 true
-- STYLE comment
{-
(That is, a sequence of engines, the first running at a
 frequency of f Hz, the next (positioned below it) at a
 frequency of 1.2 f Hz, and so on.)
-}
-- LOCN 600 1400 true
-- STYLE comment
{-
Back to previous document
-}
-- LOCN 650 125 true
-- STYLE open ""
{-
Back to previous document
-}
-- LOCN 600 1550 true
-- STYLE open ""
{-
An animated diagram of a heat engine is constructed using
functions defined in the Graphics library.
-}
-- LOCN 50 -50 true
-- STYLE abstract
