-- Vital 2.1
--
import Animation3
--
-- LOCN 675 100 true
-- STYLE 
{-
Multi-phase heat engine
-}
-- LOCN 50 50 true
-- STYLE title
--
import Pictures
--
-- LOCN 675 75 true
-- STYLE 
--
rotatedEngine phi = let psi = - phi
                    in turn' phi . scale 0.5 . engine . (+ psi)
--
-- LOCN 52 1370 true
-- STYLE Hide
--
import Graphics
--
-- LOCN 675 50 true
-- STYLE 
--
rotaryEngine n = let k = 2.0 * pi / fromInt n
                 in super [animate 0.3 (rotatedEngine (k * fromInt i)) | i <- [1 .. n]]
--
-- LOCN 52 1813 true
-- STYLE Hide
--
import SS
--
-- LOCN 675 25 true
-- STYLE 
{-
The various functions introduced in the Pictures library module (as demonstrated in
the PictureWork modules) apply equally well to animated pictures.  We give some
examples based on the "engine" function defined in the Animation3 module.
-}
-- LOCN 50 150 true
-- STYLE comment
{-
For example, here is the original animated engine followed by some
transformed versions:
-}
-- LOCN 50 300 true
-- STYLE comment
--
pic1 = animate 0.1 engine
--
-- LOCN 51 369 true
-- STYLE Hide
--
pic1
--
-- LOCN 402 369 true
-- STYLE Picture
--
pic2 = scale 0.5 pic1
--
-- LOCN 52 514 true
-- STYLE Hide
--
pic2
--
-- LOCN 402 514 true
-- STYLE Picture
--
pic3 = flipV pic2
--
-- LOCN 52 637 true
-- STYLE Hide
--
pic3
--
-- LOCN 402 637 true
-- STYLE Picture
--
pic4 = turn'  (pi / 4.0) pic3
--
-- LOCN 51 772 true
-- STYLE Hide
--
pic4
--
-- LOCN 644 771 true
-- STYLE Picture
{-
A more complex mechanism
-}
-- LOCN 50 1125 true
-- STYLE heading2
{-
Using some of the above functions, more complex mechanisms can be constructed. As an example,
consider the definition of a n-phase rotary heat engine.
-}
-- LOCN 50 1175 true
-- STYLE comment
{-
Alternatively, rather than apply functions to an animated picture (as above) one can
use functional composition to apply the transforms to the picture-generating function
_before_ applying the animation function:
-}
-- LOCN 50 900 true
-- STYLE comment
--
pic4' = let engine' = turn' (pi / 4.0) . flipV . scale 0.5 . engine
        in animate 0.1 engine'
--
-- LOCN 51 993 true
-- STYLE Hide
--
pic4'
--
-- LOCN 646 990 true
-- STYLE Picture
{-
Such an engine consists of a circular arrangement of (single-piston) engines, with the
phase of each one determined by its position in the circle.  Thus, begin by defining
a function that both rotates an engine and delays its relative phase:
-}
-- LOCN 50 1225 true
-- STYLE comment
--
rotatedEngine :: Double -> (Double -> Pic)
--
-- LOCN 51 1342 true
-- STYLE 
{-
For example:
-}
-- LOCN 50 1425 true
-- STYLE comment
--
animate 0.1 (rotatedEngine 0.0)
--
-- LOCN 51 1550 true
-- STYLE Picture
--
animate 0.1 (rotatedEngine (pi / 2.0))
--
-- LOCN 368 1549 true
-- STYLE Picture
{-
Notice how the phase of second engine is
90 degrees behind that of the first one.
-}
-- LOCN 475 1600 true
-- STYLE comment
--
Pictures.move 500 50 pic
--
-- LOCN 51 1984 true
-- STYLE Picture
--
rotaryEngine :: Int -> Pic
--
-- LOCN 52 1784 true
-- STYLE 
{-
One can now realise a complete n-piston engine by superimposing the pictures of
a sequence of single-piston engines of varying phases:
-}
-- LOCN 50 1725 true
-- STYLE comment
{-
For example, a 5-piston engine is:
-}
-- LOCN 50 1900 true
-- STYLE comment
--
pic = rotaryEngine 5
--
-- LOCN 51 1945 true
-- STYLE Hide
{-
Notice how the order of firing of the cylinders proceeds in a contrary direction
to that of the crankshaft.

But this is not the only possibility. For example, try modifying the
the "let" clause in the above declaration for "rotatedEngine"  from:
        let psi = - phi
to
        let psi = 0.0
or to
        let psi = phi
-}
-- LOCN 50 2275 true
-- STYLE comment
{-
Back to previous document
-}
-- LOCN 50 2525 true
-- STYLE open ""
{-
Back to previous document
-}
-- LOCN 675 150 true
-- STYLE open ""
{-
(As usual, press the Animate button 
 to animate these displays.)
-}
-- LOCN 700 500 true
-- STYLE warning2
{-
Using functions from the Graphics and Pictures library, the animation of
the single-phase heat engine (developed in Animation3) is further 
developed as a multi-phase heat engine.
-}
-- LOCN 50 -75 true
-- STYLE abstract
