-- Vital 2.1
--
class Demo a where {
  identify :: a -> String;
  size :: a -> Int;

  identify x = "<Unknown type>"
}
--
-- LOCN 75 225 true
-- STYLE 
--
instance Demo Char where {
  identify ch = "Character";
  size ch = 1
}
--
-- LOCN 75 425 true
-- STYLE 
--
instance (Demo a, Demo b) => Demo (a,b) where {
  identify p = "Pair (" ++ identify (fst p) ++ 
               ", " ++ identify (snd p) ++ ")";
  size p = size (fst p)  +  size (snd p)
}
--
-- LOCN 75 1025 true
-- STYLE 
--
instance (Demo a) => Demo [a] where {
  identify xs = let s = if null xs then 
                           "UNKNOWN" 
                        else 
                           identify (head xs)
                in "List of " ++ s;
  size xs = sum (map size xs)
}
--
-- LOCN 75 1650 true
-- STYLE 
--
size c
--
-- LOCN 600 500 true
-- STYLE 
--
size p
--
-- LOCN 625 1125 true
-- STYLE 
--
size s
--
-- LOCN 650 1750 true
-- STYLE 
--
instance Demo Int where {
  identify n = "Integer";
  size n = 1
}
--
-- LOCN 75 550 true
-- STYLE 
--
instance Demo Double where {
  identify d = "Double";
  size d = 2
}
--
-- LOCN 75 675 true
-- STYLE 
--
size True
--
-- LOCN 600 2875 true
-- STYLE 
--
identify c
--
-- LOCN 600 450 true
-- STYLE 
--
c = 'a'
--
-- LOCN 600 425 false
-- STYLE 
--
p = (17, 'a')
--
-- LOCN 625 1025 false
-- STYLE 
--
identify p
--
-- LOCN 625 1075 true
-- STYLE 
--
data Direction = Down | Up
--
-- LOCN 75 2175 true
-- STYLE 
--
instance Demo Direction where {
  identify d = "Direction";
  size d = 1
}
--
-- LOCN 75 2225 true
-- STYLE 
--
instance (Demo a, Demo b, Demo c) => Demo (a,b,c) where {
  identify x = case x of
                 (x1,x2,x3) -> "Pair (" ++ identify x1 ++ 
                               ", " ++ identify x2 ++ 
                               ", " ++ identify x3 ++ ")";
  size x = case x of (x1,x2,x3) -> size x1 + size x2 + size x3
}
--
-- LOCN 75 1200 true
-- STYLE 
--
instance Demo Image where {
  identify im = "Image";
  size im = let p = imageSize im
            in fst p * snd p
}
--
-- LOCN 75 800 true
-- STYLE 
--
identify s
--
-- LOCN 650 1700 true
-- STYLE 
--
s = "Hello there!"
--
-- LOCN 650 1650 false
-- STYLE 
--
identify []
--
-- LOCN 650 1875 true
-- STYLE 
--
instance Demo () where {
  identify x = "Unit";
  size x = 0
}
--
-- LOCN 75 1425 true
-- STYLE 
--
im = imageFile "curry.jpg"
--
-- LOCN 600 800 true
-- STYLE Hide
--
identify im
--
-- LOCN 600 825 true
-- STYLE 
{-
Demo of type classes
-}
-- LOCN 50 75 true
-- STYLE title
--
identify  triple
--
-- LOCN 700 1250 true
-- STYLE 
--
triple = (100.0,  (1,2), 'a')
--
-- LOCN 700 1200 false
-- STYLE 
--
size triple
--
-- LOCN 700 1300 true
-- STYLE 
{-
A class declaration introducing a new class, "Demo", and 
two methods, "identify" and "size", and a default declaration for "identify".
-}
-- LOCN 50 150 true
-- STYLE comment
{-
Instance declaration for the primitive types (Char, Int, Double, Image)
-}
-- LOCN 50 375 true
-- STYLE comment
--
size im
--
-- LOCN 600 900 true
-- STYLE 
{-
Instance declarations for tuple types
-}
-- LOCN 50 950 true
-- STYLE comment
{-
Instance declaration for the unit type
-}
-- LOCN 50 1375 true
-- STYLE comment
--
size ()
--
-- LOCN 650 1500 true
-- STYLE 
{-
Instance declaration for lists
-}
-- LOCN 50 1600 true
-- STYLE comment
--
size []
--
-- LOCN 650 1825 true
-- STYLE 
{-
Instance declaration for a user-defined datatype
-}
-- LOCN 50 2125 true
-- STYLE comment
--
size Down
--
-- LOCN 625 2275 true
-- STYLE 
--
identify Down
--
-- LOCN 625 2225 true
-- STYLE 
{-
Example of using a method at a type for which
no instance has been declared.
-}
-- LOCN 50 2750 true
-- STYLE comment
{-
Instance declaration for a function type (note that only
one of the two methods has been declared).
-}
-- LOCN 50 2350 true
-- STYLE comment
--
instance Demo (a -> b) where
  identify d = "Function"
--
-- LOCN 75 2425 true
-- STYLE 
--
identify (+)
--
-- LOCN 600 2425 true
-- STYLE 
--
size (+)
--
-- LOCN 600 2500 true
-- STYLE 
--
size nss
--
-- LOCN 650 1975 true
-- STYLE 
--
nss = [[1,2,3], [4,5], [6,7,8,9]]
--
-- LOCN 650 1950 false
-- STYLE 
--
identify nss
--
-- LOCN 650 2025 true
-- STYLE 
--
identify ()
--
-- LOCN 650 1425 true
-- STYLE 
{-
Back to previous document
-}
-- LOCN 50 2925 true
-- STYLE open  ""
--
identify (\x -> x)
--
-- LOCN 600 2575 true
-- STYLE 
--
identify identify
--
-- LOCN 600 2650 true
-- STYLE 
--
identify True
--
-- LOCN 600 2800 true
-- STYLE 
--
import SS
--
-- LOCN 725 50 true
-- STYLE 
{-
Some simple tests of the implementation of type classes.
-}
-- LOCN 50 -50 true
-- STYLE abstract
