-- Vital 2.1
--
import SS
--
-- LOCN 725 75 true
-- STYLE 
{-
File input and parsing
-}
-- LOCN 50 75 true
-- STYLE title
{-
Text files may be input using the function:
    textFile  :: Path -> String
(The "path" is interpreted relative to the subdirectory named "files".)
-}
-- LOCN 50 150 true
-- STYLE comment
{-
For example, to input the text file named "sharePrices.txt":
-}
-- LOCN 50 250 true
-- STYLE comment
--
companyData = textFile "sharePrices.txt"
--
-- LOCN 50 300 true
-- STYLE Text 0 defaultFmt
{-
Parsing
-}
-- LOCN 50 675 true
-- STYLE heading1
{-
The library module "Parse" defines a set of parsing combinators.
(They are very flexible -- they can deal with arbitrary grammars and, if the grammars are
ambiguous, they will return a list of all possible parses. But they are not at all efficient
and so are not suitable for use on large strings.)
-}
-- LOCN 50 725 true
-- STYLE comment
--
import Parse
--
-- LOCN 50 825 true
-- STYLE 
--
integer "12345 cats"
--
-- LOCN 50 1025 true
-- STYLE 
{-
For example, the function
        integer :: String -> [(Int, String)]
will attempt to parse an integer and will return a list of pairs of:
  - the integer it has parsed, and
  - the remaining unparsed tail of the string.
-}
-- LOCN 50 900 true
-- STYLE comment
{-
The function
      parser :: (String -> [(a, String)]) -> a
can be used to construct a parser that attempts to parse the entire string (and
fails if the entire string is unparsable or the parse is ambiguous). For example:
-}
-- LOCN 50 1125 true
-- STYLE comment
--
parser integer "12345  "
--
-- LOCN 50 1250 true
-- STYLE 
{-
Example
-}
-- LOCN 50 1400 true
-- STYLE heading1
{-
Here is a parser that will parse one record (ie, a company identifier and a list of
its share prices) in the format of the above text file of company data:
-}
-- LOCN 50 1450 true
-- STYLE comment
--
company = (spaced identifier >> intList)
--
-- LOCN 50 1525 true
-- STYLE hide
{-
For example:
-}
-- LOCN 50 1575 true
-- STYLE comment
--
parser company  "widgetCo   [12, 15, 19, 30, 25, 18, 17]"
--
-- LOCN 50 1625 true
-- STYLE 
{-
Here is a parser that parses the entire file, and present the data as a list of
records, each consisting of a string and a list of its share prices:
-}
-- LOCN 50 1725 true
-- STYLE comment
--
records = parser (many company) companyData
--
-- LOCN 50 1775 true
-- STYLE 
{-
  "identifier"       parses an identifier
  "spaced"           ignores leading white space
  "intList"          parses a list of integers
  ">>"               an operator that sequentially combines two parsers
-}
-- LOCN 675 1550 true
-- STYLE commentM
{-
By selecting the second component of each pair, and using the "table2" style, the
data can be presented in tabular form:
-}
-- LOCN 50 1900 true
-- STYLE comment
--
prices = map snd records
--
-- LOCN 50 1950 true
-- STYLE table2
{-
Back to previous document
-}
-- LOCN 50 2125 true
-- STYLE open1 ""
{-
Back to previous document
-}
-- LOCN 725 125 true
-- STYLE open ""
{-
Exercise!
Try modifying this expression to (eg)
      parser integer  "12345 hello"
and notice that the parser function returns an 'error' value
-}
-- LOCN 475 1250 true
-- STYLE warning2
{-
Illustrates how the 'textFile' function may be used to read in a text file, and
how the parsing combinators (defined in the 'Parse' library) may be used to
construct a parser to parse it.
-}
-- LOCN 50 -75 true
-- STYLE abstract
