-- Vital 2.1
{-
Direct manipulation of datatype values
-}
-- LOCN 75 75 true
-- STYLE title
--
nss = map (flip replicate 0) [1 .. 6]
--
-- LOCN 100 400 false
-- STYLE Hide
--
import SS
--
-- LOCN 775 75 true
-- STYLE 
--
ns = [0 .. 10]
--
-- LOCN 100 350 false
-- STYLE Hide
--
data Tree = Tip Int
          | Tree :+: Tree
--
-- LOCN 100 1550 true
-- STYLE 
--
mkTree n = if n > 0 then
             (Tip 0 :+: (Tip 0 :+: mkTree (n-1))) :+: Tip 0
           else
             Tip 0
--
-- LOCN 100 1700 true
-- STYLE Hide
--
[0 .. 10]
--
-- LOCN 450 2250 true
-- STYLE list2
{-
Structural changes
-}
-- LOCN 75 1325 true
-- STYLE heading1
{-
An important feature of Vital is that (just as with spreadsheets) the values of
displayed data structures can be changed by Copy/Paste operations. Further, any such
changes are reflected back into the source code of the original declaration.
-}
-- LOCN 75 175 true
-- STYLE comment
{-
To illustrate this, here are declarations for two typical data structures: a list
of integers and a triangular array of 0s:
-}
-- LOCN 75 275 true
-- STYLE comment
--
sum (map sum nss')
--
-- LOCN 650 700 true
-- STYLE 
--
sum ns'
--
-- LOCN 100 625 true
-- STYLE 
{-
Exercise!
Now perform some Copy and Paste operations on these data structure. Select an element of the
list (by double-clicking), then press the Copy button (to copy this value to the clipboard),
then select an element of the array and press the Paste button (to overwrite the
existing value with the copied value).  Repeat this kind of operation a few times.
[Recollect that the effect of an edit can be undone by pressing the Undo button.]

Notice that:
    - the declarations of the data structures are updated to reflect
      their modified values;

    - the values of the expressions that compute the sum of these data structures are
      correspondingly updated.
-}
-- LOCN 75 775 true
-- STYLE commentB
{-
The expression that is pasted into a data structure can be taken from either the keyboard
or the clipboard.
-}
-- LOCN 75 1100 true
-- STYLE comment
{-
The above operations affected only the element values of a data structure. They could
not affect its structure since none of the linking elements were visible. But by adopting
a style where linking elements are visible, structural editing operations may also
be performed. 
-}
-- LOCN 75 1375 true
-- STYLE comment
--
nss' = nss
--
-- LOCN 650 550 true
-- STYLE table2
--
ns' = ns
--
-- LOCN 100 550 true
-- STYLE list2
{-
To demonstrate structural editing, first introduce a datatype for binary trees
-}
-- LOCN 75 1500 true
-- STYLE comment
{-
. . . and a function to build a (finite) tree:
-}
-- LOCN 75 1625 true
-- STYLE comment
{-
Two caveats:
  - the present implementation of Vital does not perform type-checking, and
    hence badly-typed structures can be created.

  - the particular style used to display this tree uses fixed offsets and hence
    structural changes may result in subtrees overlapping.
-}
-- LOCN 50 2600 true
-- STYLE comment
{-
Back to previous document
-}
-- LOCN 75 3075 true
-- STYLE open ""
{-
Back to previous document
-}
-- LOCN 778 129 true
-- STYLE open ""
{-
. . . and here are these values displayed graphically along with expressions
that compute the sum of their elements:
-}
-- LOCN 75 475 true
-- STYLE comment
{-
Exercise:
Double-click to select an element in one of the above data structures. Then type (eg) the 
expression "3 * 29" into the Item field at the top of the window. Then, while holding down 
the Shift key, press the Paste button.
-}
-- LOCN 75 1175 true
-- STYLE commentB
--
mkTree :: Int -> Tree
--
-- LOCN 100 1675 true
-- STYLE 
{-
For example:
-}
-- LOCN 75 1800 true
-- STYLE comment
--
tree = mkTree 3
--
-- LOCN 125 1850 true
-- STYLE 
{-
. . . or, displayed graphically:
-}
-- LOCN 75 1950 true
-- STYLE comment
--
tree
--
-- LOCN 125 2000 true
-- STYLE blueTree'
{-
Exercise!
* First copy some numbers from the list into the Tips of the tree by:
    - double-click in the list to select an element, then press the Copy button;
    - double-click in a tree Tip to select an element, then press the Paste button.

* Now change the structure of the tree:
    - double-click in a tree Node to select a subtree, then press the Copy button;
    - double-click in a tree Node to select a subtree, then press the Paste button.

-}
-- LOCN 50 2375 true
-- STYLE commentB
{-
(Can press the Undo button
 to undo unwanted changes.)
-}
-- LOCN 700 2600 true
-- STYLE comment
{-
Back to previous document
-}
-- LOCN 50 2750 true
-- STYLE open ""
{-
Detailed explanation of how to carry out Direct Manipulation
on values of concrete datatypes. Depending on the Style used
to display the datatype, the values in a data structure or its
structure or both its values and its structure may be manipulated.
-}
-- LOCN 75 -75 true
-- STYLE abstract
