-- Vital 2.1
--
import SS
--
-- LOCN 725 75 true
-- STYLE 
--
import Graphics
--
-- LOCN 725 100 true
-- STYLE 
{-
Simple interaction
-}
-- LOCN 50 75 true
-- STYLE title
{-
The state of the "blob" is represented by its x and y coordinates:
-}
-- LOCN 50 350 true
-- STYLE comment
--
data Blob = Blob {x :: Position, y :: Position}
--
-- LOCN 75 425 true
-- STYLE 
{-
For example, a typical blob state is
-}
-- LOCN 50 475 true
-- STYLE comment
--
blob = Blob 150.0 40.0
--
-- LOCN 75 525 true
-- STYLE Hide
{-
This function displays the blob
-}
-- LOCN 50 600 true
-- STYLE comment
--
displayBlob blob  =
    case blob of
      Blob x y -> circle x y 30.0 cyan blue
--
-- LOCN 50 650 true
-- STYLE Hide
--
displayBlob blob
--
-- LOCN 575 650 true
-- STYLE Picture
--
blobDynamics tm xi yi pressed blob =
  let {x = fromInt xi;  y = fromInt yi;
       blob' = drag x y blob ;
       pic   = displayBlob blob
  } in (blob', pic)
--
-- LOCN 75 1875 true
-- STYLE Hide
{-
This function "drags" the blob towards the point (x,y).
-}
-- LOCN 50 775 true
-- STYLE comment
--
drag :: Position -> Position -> Blob -> Blob
--
-- LOCN 50 825 true
-- STYLE 
--
drag x y blob =
  case blob of
    Blob x1 y1 -> 
      let {dx = x - x1;  dy = y - y1;
           vx = elasticity * dx;  vy = elasticity * dy;
           x1' = x1 + (dt * vx);  y1' = y1 + (dt * vy)
      } in Blob x1' y1'
--
-- LOCN 50 850 true
-- STYLE Hide
--
elasticity = 0.5
--
-- LOCN 100 1075 true
-- STYLE Hide
--
dt = 0.05
--
-- LOCN 350 1075 true
-- STYLE Hide
--
Interaction initState blobDynamics
--
-- LOCN 100 2125 true
-- STYLE Picture
{-
In order for the display to be animated, set the mode
of the system to "animated" by clicking on the "Anim"
radio button.
-}
-- LOCN 575 2300 true
-- STYLE warning2
{-
where the "elasticity" of the (invisible!) cord between the mouse cursor and
the blob, and the (assumed) increment of time between successive screen refreshes
are specified by:
-}
-- LOCN 50 1000 true
-- STYLE comment
{-
For example, the state of the blob after being 
dragged (for one time-step) towards the 
point (400, 200) would be:
-}
-- LOCN 675 1000 true
-- STYLE comment
--
drag 400.0 200.0 blob
--
-- LOCN 700 1075 true
-- STYLE 
{-
The type of the top-level interaction function the system requires is:
-}
-- LOCN 50 1225 true
-- STYLE comment
--
blobDynamics :: Interact
--
-- LOCN 75 1850 true
-- STYLE 
--
type State = Blob
--
-- LOCN 75 1625 true
-- STYLE 
{-
where the arguments represent:
  - the real-time (in milli-seconds since 1 January, 1970)
  - the instantaneous x-position of the mouse cursor
  - the instantaneous y-position of the mouse cursor
  - whether or not the mouse button is pressed
  - the current state of the animation
and the function has to compute a pair representing:
  - the next state of the animation
  - a picture value to be displayed
-}
-- LOCN 50 1325 true
-- STYLE comment
{-
Here is a function that will yield an animation of a massless blob being
dragged (with an elastic cord, through a viscous medium) towards the mouse cursor
-}
-- LOCN 50 1800 true
-- STYLE comment
{-
We assume that the "state" of the animation is simply the
current position of the blob
-}
-- LOCN 50 1550 true
-- STYLE comment
{-
In order to present this simulation as an animated display, the "Interaction"
constructor of the "Pic" datatype is used. Its two arguments are:
  - the initial state of the simulation
  - the above function
-}
-- LOCN 50 2000 true
-- STYLE comment
{-
Back to previous document
-}
-- LOCN 50 2525 true
-- STYLE open ""
{-
Back to previous document
-}
-- LOCN 425 75 true
-- STYLE open ""
--
type Interact = Int -> Int -> Int -> Bool -> State -> (State, Pic)
--
-- LOCN 75 1275 true
-- STYLE 
--
initState = Blob 100.0  100.0
--
-- LOCN 75 1725 true
-- STYLE Hide
{-
This document explains how a "blob" dragged around by an elastic cord can be
simulated. (It is one of the components of the "Dot game" module.)
-}
-- LOCN 50 150 true
-- STYLE comment
{-
Dynamics: it is assumed that the force acting on the blob is proportional to
the distance to the mouse cursor, and it assume massless, viscous dynamics (ie,
the blob velocity is proportional the force acting on it), and it uses
first-order integration to determine the distance moved in dt, the time increment
between successive frames.
-}
-- LOCN 50 225 true
-- STYLE comment
{-
For example:
-}
-- LOCN 550 600 true
-- STYLE comment
--
type Position = Double
--
-- LOCN 75 400 true
-- STYLE 
{-
Interaction
-}
-- LOCN 50 1175 true
-- STYLE heading1
{-
. . . and we define the initial state of the simulation to be:
-}
-- LOCN 50 1675 true
-- STYLE comment
{-
An explanation of how, using the "Interaction" picture constructor, a
simple interactive game can be programmed.
-}
-- LOCN 50 -75 true
-- STYLE abstract
