-- Vital 2.1
{-
Adjusting parameters in an ADT
-}
-- LOCN 50 50 true
-- STYLE title
--
r = SelRect 100 100 yellow red
--
-- LOCN 650 275 true
-- STYLE Picture
--
instance Edit Pic where {
  adjust val path x y lhs rhs =
    let { x' = " (" ++ show x ++ ") ";
          y' = " (" ++ show y ++ ") ";
          rhs' = "SelRect" ++ x' ++ y' ++ "yellow red";
          message' = "x = " ++ show x
    } in   (Just rhs', Just message')
}
--
-- LOCN 75 825 true
-- STYLE 
{-
The 'adjust' method of the Edit class:
      class Edit a where
           style :: a -> . . .
           edit  :: a -> . . .
           adjust :: a -> Path -> Int -> Int -> String -> String -> (MString, MString) 
allows a parameter in a declaration to be adjusted by right-dragging with the mouse.
-}
-- LOCN 50 125 true
-- STYLE comment
{-
Back to previous document
-}
-- LOCN 700 75 true
-- STYLE open ""
--
import SS
--
-- LOCN 700 50 true
-- STYLE 
{-
Try this:
Select the adjacent rectangle (by double-clicking within it)
and then drag its lower righthand corner (using the Right mouse
button). Observe how, as this is done, the Haskell source code of
the declaration is continuously adjusted to yield the desired 
size of rectangle.
-}
-- LOCN 50 275 true
-- STYLE commentB
{-
The 'adjust' method is broadly similar to the 'edit functions' used in the 'edit' method.

Whenever the user selects a SelRect ('selectable rectangle') component of a picture
and drags with the mouse, the 'adjust' method (if an appropriate instance has been
defined) is repeatedly invoked by the system and supplied with the
following parameters:
    - the value of the declaration or expression;
    - the path to the selected component of the picture;
    - the X and Y coordinates of the mouse;
    - the LHS and RHS parts of the Haskell declaration or expression.

The task of the 'adjust' method is (optionally) to generate:
   - a replacement text for the RHS of the declaration/expression, 
   - a status message for display in the Status field at the bottom of the window.
-}
-- LOCN 50 450 true
-- STYLE comment
{-
For example, here is the instance declaration for the Pic datatype (of pictures) that handles the
adjustment of the above picture:
-}
-- LOCN 50 775 true
-- STYLE comment
{-
(It simply takes the X and Y mouse coordinates (as Int values), converts them to numerals (and parenthesises
them in case they should be negative), and constructs the replacement text for the RHS of the declaration
together with a message to display in the Status field.)
-}
-- LOCN 50 975 true
-- STYLE comment
{-
Example: A colour selector
-}
-- LOCN 50 1100 true
-- STYLE heading1
--
data ColorBox = ColorBox {color :: Color}
--
-- LOCN 75 1325 true
-- STYLE 
--
instance Edit ColorBox where {
  style cb = View colorBoxToPic Picture;
  adjust cb path x y lhs rhs =
    let rhs' = "mkColorBox (" ++ show x ++ ") (" ++ show y ++ ")"
    in (Just rhs', Nothing)
}
--
-- LOCN 75 1800 true
-- STYLE 
--
colorBoxToPic cb = SelRect 255 255 (getColor cb) black
--
-- LOCN 75 1675 true
-- STYLE Hide
--
mkColorBox (76) (158)
--
-- LOCN 625 1250 true
-- STYLE 
--
mkColorBox r g = let {r' = max 0 (min 255 r);
                      g' = max 0 (min 255 g);
                      b' = 128
                   } in ColorBox (RGB r' g' b')
--
-- LOCN 75 1400 true
-- STYLE Hide
{-
The parameter adjusted by the 'adjust' method does not necessarily have to be a linear dimension.
As an example, here is a definition for the 'adjust' method that implements a colour selector box.
-}
-- LOCN 50 1150 true
-- STYLE comment
{-
Introduce a datatype to represent a colour box, and constructor
and projector functions for this type:
-}
-- LOCN 50 1250 true
-- STYLE comment
--
mkColorBox :: Int -> Int -> ColorBox
--
-- LOCN 75 1375 true
-- STYLE 
--
getColor :: ColorBox -> Color
--
-- LOCN 75 1500 true
-- STYLE 
--
getColor cb = case cb of ColorBox color -> color
--
-- LOCN 75 1525 true
-- STYLE Hide
{-
Introduce a function to render a ColorBox value as an
appropriately coloured 'selectable rectangle' picture:
-}
-- LOCN 50 1575 true
-- STYLE comment
--
colorBoxToPic :: ColorBox -> Pic
--
-- LOCN 75 1650 true
-- STYLE 
{-
Finally, define ColorBox instances for the 'style' and 'adjust' methods
of the Edit class:
-}
-- LOCN 50 1750 true
-- STYLE comment
{-
Try this:
Double-click in this box to select it, then 
right-drag the mouse within the box to change
its colour. Notice how the text of the declaration
is continuously updated as the mouse is dragged.
-}
-- LOCN 625 1575 true
-- STYLE commentB
{-
Back to previous document
-}
-- LOCN 50 1950 true
-- STYLE open ""
{-
Demo of the Edit.adjust method. Shows how the size or the colour
of a picture element may be changed by right-mouse dragging.
-}
-- LOCN 50 -50 true
-- STYLE abstract
