-- Vital 2.1
{-
Editing a directed graph
-}
-- LOCN 50 50 true
-- STYLE title
--
import SS
--
-- LOCN 700 50 true
-- STYLE 
--
data Relation = Rel [Int]
--
-- LOCN 50 178 true
-- STYLE 
--
displayRel ns x =
  case ns of {
    []     -> NoPic;
    (m:ms) -> displayNode x .+. displayRel ms (x + 100) .+. displayEdge m x
  }
--
-- LOCN 49 1018 true
-- STYLE 
--
(.+.) = Super
--
-- LOCN 604 927 true
-- STYLE 
--
displayNode x = Trans (x-10) (-10) (SelRect 20 20 yellow red)
--
-- LOCN 47 1174 true
-- STYLE 
--
displayEdge m x = if m >0 then
             Trans 0 100 (Line x (-100) red)
           else
             NoPic
--
-- LOCN 48 1215 true
-- STYLE 
--
displayRelation :: Relation -> Pic
--
-- LOCN 49 926 true
-- STYLE 
--
rel =Rel [1, 0, 0, 1, 1, 0]
--
-- LOCN 50 356 true
-- STYLE 
{-
Introduce a new type to represent relations and then
specify the "look and feel" of a relation by defining an
instance of the Edit class.
-}
-- LOCN 50 100 true
-- STYLE comment
--
instance Edit (a -> b) where {
  style f = Hide
}
--
-- LOCN 700 78 true
-- STYLE 
--
set index value rel = 
  case rel of Rel ns -> let ns' = subst index value ns
                        in Rel ns'
--
-- LOCN 49 1372 true
-- STYLE 
--
subst index value ns =
  if index == 0 then
    value : tail ns
  else
    head ns : subst (index-1) value (tail ns)
--
-- LOCN 49 1442 true
-- STYLE 
--
instance Edit Relation where {
  style rel = View displayRelation Picture;
  edit rel path = [("Break", modify 0),
                   ("Connect", modify 1)]
}
--
-- LOCN 50 207 true
-- STYLE 
--
modify edge rel path lhs rhs clip =
    let { i = (length path - 2) `div` 2;
          message = (if edge == 0 then "Remove link " else "Insert link ") ++ show i;
          rhs' = "set " ++ show i ++ " " ++ show edge ++ " (" ++ rhs ++ ")"
    } in (Just rhs', Nothing, Just message)
--
-- LOCN 49 750 true
-- STYLE 
{-
The "set" function (that modifies a relation).
-}
-- LOCN 50 1325 true
-- STYLE comment
{-
The "modify" function (that generates the modified
source text).
-}
-- LOCN 50 675 true
-- STYLE comment
{-
Here is a typical relation. Double click on a node to add or remove an edge.
-}
-- LOCN 50 325 true
-- STYLE comment
{-
Subsidiary definitions
-}
-- LOCN 50 600 true
-- STYLE heading1
{-
The "showRel" function (that displays a relation).
-}
-- LOCN 50 900 true
-- STYLE comment
--
displayRelation rel = 
  case rel of
    Rel ns -> Trans 0 100 (displayNode 0) .+. displayRel ns 0
--
-- LOCN 49 954 true
-- STYLE 
{-
Functions for displaying a single node or edge
-}
-- LOCN 50 1150 true
-- STYLE comment
{-
A simple demo of the Edit.style and Edit.edit methods. Shows how
an edge can be added/removed from a simple graph.
-}
-- LOCN 50 -50 true
-- STYLE abstract
