-- Vital 2.1
--
import SS
--
-- LOCN 850 75 true
-- STYLE 
{-
Defining a default style for a type
-}
-- LOCN 50 75 true
-- STYLE title
{-
The "Edit" class is defined (in the Prelude module) as:

          class Edit a where
              style :: a -> Style
              . . . 

If, for a particular declaration or expression, no style is explicitly specified, then the
system uses the style specified by the "style" method of the appropriate instance of the
"Edit" class.
-}
-- LOCN 50 250 true
-- STYLE comment
{-
The "Edit" class (defined in the Prelude module) can be used to define a default style
for a particular type. This will result in values of that type automatically being 
displayed using that style (unless the user explicitly associates a different style 
with a particular declaration or expression).
-}
-- LOCN 50 150 true
-- STYLE comment
{-
Example
-}
-- LOCN 50 475 true
-- STYLE heading1
{-
Suppose, as an example, that we want all Bool values to be displayed graphically, as a tick or a cross,
rather than (as happens by default) displayed textually, as the string "True" or "False".
-}
-- LOCN 50 525 true
-- STYLE comment
{-
First, define a "representation function" that maps values of the type (in this case, Bool)
to a chosen pictorial representation:
-}
-- LOCN 50 600 true
-- STYLE comment
--
boolRep :: Bool -> Pic
--
-- LOCN 75 675 true
-- STYLE 
--
boolRep b = 
  let fileName = if b then "logical/tick.gif" else "logical/cross.gif"
  in picFile fileName
--
-- LOCN 75 700 true
-- STYLE Hide
{-
For example:
-}
-- LOCN 50 800 true
-- STYLE comment
--
boolRep False
--
-- LOCN 200 850 true
-- STYLE Picture
--
boolRep True
--
-- LOCN 475 850 true
-- STYLE Picture
{-
Next, use the "View" style constructor to define a style (ie, a value of type Style) for Bool
values that will display them in this way.
-}
-- LOCN 50 975 true
-- STYLE comment
--
boolStyle :: Style
--
-- LOCN 75 1050 true
-- STYLE 
--
boolStyle = View boolRep Picture
--
-- LOCN 75 1075 true
-- STYLE Hide
{-
The "View" constructor takes two arguments:
  - a function (in this case, boolStyle) to map the value (in this case, of type Bool) to 
    a value of some displayable type (in this case, Pic);
  - a style (in this case, Picture) with which to display this mapped value.
-}
-- LOCN 425 1050 true
-- STYLE comment
{-
By specifying this style (explicitly), we can now display Bool values in this form.
For example:
-}
-- LOCN 50 1175 true
-- STYLE comment
--
2 + 2 == 4
--
-- LOCN 200 1250 true
-- STYLE boolStyle
--
2 + 2 == 5
--
-- LOCN 475 1250 true
-- STYLE boolStyle
{-
(Click on the handle of an expression to
 see its style displayed in the "Style"
 field at the top of the window.)
-}
-- LOCN 675 1250 true
-- STYLE comment
{-
Finally, we can define a Bool instance of the Edit class, so as to specify
that "boolStyle" should be the default style for Bool values:
-}
-- LOCN 50 1375 true
-- STYLE comment
--
instance Edit Bool where
  style b = boolStyle
--
-- LOCN 75 1425 true
-- STYLE 
{-
For example:
-}
-- LOCN 50 1500 true
-- STYLE comment
--
2 + 2 == 4
--
-- LOCN 200 1550 true
-- STYLE 
--
2 + 2 == 5
--
-- LOCN 475 1550 true
-- STYLE 
{-
(Click on the handles of these expressions
 to confirm that no style has been
 explicitly specified for them.)
-}
-- LOCN 675 1550 true
-- STYLE comment
{-
Another example
-}
-- LOCN 50 1700 true
-- STYLE heading1
{-
In practice, the steps illustrated above will often be merged. For example, an instance
declaration for the type Color to cause values of type Color to be displayed as squares of 
the relevant colour can be specified by:
-}
-- LOCN 50 1750 true
-- STYLE comment
--
instance Edit Color where
  style c = View (\c -> Rect 20 20 c black) Picture
--
-- LOCN 75 1850 true
-- STYLE 
{-
For example:
-}
-- LOCN 50 1925 true
-- STYLE comment
--
red
--
-- LOCN 75 1975 true
-- STYLE 
--
green
--
-- LOCN 175 1975 true
-- STYLE 
--
blue
--
-- LOCN 275 1975 true
-- STYLE 
--
brown
--
-- LOCN 375 1975 true
-- STYLE 
--
cyan
--
-- LOCN 475 1975 true
-- STYLE 
--
yellow
--
-- LOCN 575 1975 true
-- STYLE 
--
RGB 100 100 200
--
-- LOCN 275 2075 true
-- STYLE 
--
Transparent
--
-- LOCN 75 2075 true
-- STYLE 
--
magenta
--
-- LOCN 775 1975 true
-- STYLE 
--
white
--
-- LOCN 675 1975 true
-- STYLE 
--
pi
--
-- LOCN 175 2625 true
-- STYLE 
--
- pi
--
-- LOCN 475 2625 true
-- STYLE 
{-
Another example
-}
-- LOCN 50 2225 true
-- STYLE heading1
{-
In both of the above examples:
  - the values to be displayed were mapped to Pic (picture) values;
  - the style for the mapped value was a constant

Here, by contrast, is an instance declaration (for the type Double) in which:
  - the value to be displayed is mapped (by "show") to a String value;
  - the style adopted depends upon the value:
-}
-- LOCN 50 2275 true
-- STYLE comment
--
instance Edit Double where
  style d = let {color = if d < 0.0 then red else black;
                 fmt = Format SansSerif 18 color False True;
                 sty  = Text 0 fmt
            } in View show sty
--
-- LOCN 75 2450 true
-- STYLE 
{-
For example:
-}
-- LOCN 50 2575 true
-- STYLE comment
{-
(This defines a "Text" style, based 
 upon an 18pt, italic SansSerif font.)
-}
-- LOCN 675 2475 true
-- STYLE comment
{-
Back to previous document
-}
-- LOCN 50 2750 true
-- STYLE open1 ""
{-
Back to previous document
-}
-- LOCN 600 75 true
-- STYLE open ""
{-
For example, later on we show how
values of type Color could automatically
be shown as small coloured squares
(instead of as a string of RGB component
values), thus:
-}
-- LOCN 700 150 true
-- STYLE comment
--
red
--
-- LOCN 725 275 true
-- STYLE 
--
cyan
--
-- LOCN 825 275 true
-- STYLE 
--
brown
--
-- LOCN 925 275 true
-- STYLE 
{-
Detailed explanation of how, using the Edit.style method, default styles
can be specified for particular types. Illustrated by displaying Bool values
as tick or cross, and displaying Color values as coloured squares.
-}
-- LOCN 50 -75 true
-- STYLE abstract
