-- Vital 2.1
{-
Exact Real Arithmetic
-}
-- LOCN 50 50 true
-- STYLE title
--
import SS
--
-- LOCN 625 25 true
-- STYLE 
{-
Exact real arithmetic is the art of performing arithmetic exactly on real numbers (unlike the rough and ready approximations
that floating-point arithmetic offers).

Of course, since the set of computer programs is countable but the set of reals is uncountable, not all the reals can be
represented (in fact, almost none of them can be!). The ones that are representable are termed the Computable Reals.
Many different approaches to representing the computable reals have been explored, including:
        - positional notation (infinite lists of digits, lazily evaluated)
        - continued fraction expansions
        - iterated linear transforms
        - functions yielding Cauchy sequences of rationals
-}
-- LOCN 50 100 true
-- STYLE comment
{-
Here is an example of a computable real
(the infinite decimal expansion of pi)
-}
-- LOCN 50 325 true
-- STYLE open "PiDigits"
{-
Algorithms for computing pi using exact real arithmetic
-}
-- LOCN 50 675 true
-- STYLE heading1
{-
Using the Bailey-Borwein-Plouffe hexadecimal expansion of pi
-}
-- LOCN 50 775 true
-- STYLE open "PiBBP"
{-
Using a mixed-radix base (Gibbons)
-}
-- LOCN 500 775 true
-- STYLE open "PiSpigot"
{-
Here are two documents exploring different approaches to computing pi:
-}
-- LOCN 50 725 true
-- STYLE comment
{-
Having a representation of the computable reals is only part of the task; an "effective" implementation is needed of
the usual arithmetic operations {+, -, x, /} and possibly of the constants and functions of traditional analysis {e, pi, exp, log, sin, . . .}.
(Here, "effective" means an implementation that is guaranteed always to produce a result, never to become mired in endless 
recursion.) Not all representations are equally suitable. For instance, adding together the ordinary decimal representation
of 1/3 and 2/3 will yield the numeral 0.999... , which would have to be explored to infinite depth before it could be concluded
to be equal to one.

Not all operations have an effective implmentation; in particular, relations (such as equality) and discontinuous functions (such as 
signum) are only semi-decidable. (An infinite amount of information would have to be inspected before one could conclude that
two computable reals were, in fact, equal. . .)

-}
-- LOCN 50 400 true
-- STYLE comment
{-
Back to previous document
-}
-- LOCN 625 50 true
-- STYLE open ""
{-
Back to previous document
-}
-- LOCN 50 825 true
-- STYLE open1 ""
{-
Top-level document on exact real arithmetic. Contains a brief summary
of the notion of a computable real and of an effective algorithm.
-}
-- LOCN 50 -50 true
-- STYLE abstract
