-- Vital 2.1
--
import SS
--
-- LOCN 900 75 true
-- STYLE 
{-
Experiments with mapping a (high-level) discrete event FSM
down on to a lower-level discrete time FSM
-}
-- LOCN 25 50 true
-- STYLE title
{-
Interaction {s0 :: State, fn :: Time -> Int -> Int -> Bool -> State -> (State, Pic)}
-}
-- LOCN 175 825 true
-- STYLE comment
--
data State = State [Bool]
--
-- LOCN 25 250 true
-- STYLE 
{-
The high-level FMS
-}
-- LOCN 25 150 true
-- STYLE heading1
--
s0 :: State
--
-- LOCN 25 300 true
-- STYLE 
--
s0 = State (replicate n False)
--
-- LOCN 25 325 true
-- STYLE Hide
{-
The game consists of a row of n squares. The player clicks a square to place a stone on it. The
machine responds by placing a stone somewhere else.
-}
-- LOCN 25 200 true
-- STYLE comment
--
n = 16
--
-- LOCN 200 300 true
-- STYLE Hide
--
displayState :: State -> Pic
--
-- LOCN 25 450 true
-- STYLE 
--
getBools :: State -> [Bool]
--
-- LOCN 425 275 true
-- STYLE 
--
getBools s = case s of State bs -> bs
--
-- LOCN 425 300 true
-- STYLE Hide
--
displayState s = displayBools (getBools s)
--
-- LOCN 25 475 true
-- STYLE 
--
instance Edit (a -> b) where
  style f = Hide
--
-- LOCN 25 1475 true
-- STYLE 
--
displayBools :: [Bool] -> Pic
--
-- LOCN 25 525 true
-- STYLE 
--
displayBools bs = 
  case bs of {
    []   -> NoPic;
    b:bs -> displayBool b `Super` Trans sqSize 0 (displayBools bs)
  }
--
-- LOCN 25 550 true
-- STYLE 
--
displayBool :: Bool -> Pic
--
-- LOCN 25 650 true
-- STYLE 
--
displayBool b =
  let bc = if b then red else green
  in Rect sqSize sqSize bc black
--
-- LOCN 25 675 true
-- STYLE 
--
sqSize = 10
--
-- LOCN 25 750 true
-- STYLE Hide
--
displayState s0
--
-- LOCN 557 450 true
-- STYLE 
--
displayBool True
--
-- LOCN 550 725 true
-- STYLE 
--
displayBool False
--
-- LOCN 550 675 true
-- STYLE 
--
instance Edit Pic where
  style p = Picture
--
-- LOCN 25 1425 true
-- STYLE 
--
displayBools [True, False, True]
--
-- LOCN 550 525 true
-- STYLE 
{-
Functions for display
-}
-- LOCN 25 400 true
-- STYLE heading1
{-
Interaction
-}
-- LOCN 25 825 true
-- STYLE heading1
--
fsm t x y pressed s =
  let {im = decode n x y;
       s' = case im of {
              Nothing  -> s;
              Just i   -> setState i s
            }; 
        pic = displayState s'
    } in (s', pic)
--
-- LOCN 25 875 true
-- STYLE 
{-
Supporting code
-}
-- LOCN 25 1375 true
-- STYLE comment
--
setState :: Int -> State -> State
--
-- LOCN 425 350 true
-- STYLE 
--
setBool i bs = if i == 0 then True : (tail bs) 
                         else head bs : (setBool (i-1) (tail bs))
--
-- LOCN 425 400 true
-- STYLE 
--
displayState (setState 2 s0)
--
-- LOCN 875 450 true
-- STYLE 
--
setState i s = State (setBool i (getBools s))
--
-- LOCN 425 375 true
-- STYLE 
{-
Decoding the position
-}
-- LOCN 25 1050 true
-- STYLE comment
--
decode :: Int -> Int -> Int -> Maybe Int
--
-- LOCN 25 1100 true
-- STYLE 
--
decode n x y =
  if ((x >= 0) && (x < (sqSize * n)) && (y >= 0) && (y <= sqSize)) then
    Just (x `div` sqSize)
  else
    Nothing
    
--
-- LOCN 25 1125 true
-- STYLE 
--
decode 5 50 3
--
-- LOCN 750 1100 true
-- STYLE 
--
Interaction s0 fsm
--
-- LOCN 25 1250 true
-- STYLE 
{-
Note that one does NOT need to have the ANIM mode 
in operation: the mouse click by itself serves 
to refresh the display!

But, if the ANIM mode is on, then one doesn't even
need to click; just passing the mouse over
a square suffices!
-}
-- LOCN 350 1250 true
-- STYLE warning2
{-
Showing how the 'Interaction' picture constructor can be used to allow the user to
set squares (in a row of squares) simply by moving the mouse over the desired
squares.
-}
-- LOCN 25 -75 true
-- STYLE abstract
