-- Vital 2.1
--
import SS
--
-- LOCN 875 75 true
-- STYLE 
{-
Experiments with mapping a (high-level) discrete
event FSM down onto a lower-leve discrete time FSM
-}
-- LOCN 25 50 true
-- STYLE title
--
s0 :: State
--
-- LOCN 25 275 true
-- STYLE 
--
displayState :: State -> Pic
--
-- LOCN 25 500 true
-- STYLE 
--
getBools :: State -> [Bool]
--
-- LOCN 25 350 true
-- STYLE 
--
getBools s = case s of State bs -> bs
--
-- LOCN 25 375 true
-- STYLE Hide
--
displayState s = displayBools (getBools s)
--
-- LOCN 25 525 true
-- STYLE 
--
instance Edit (a -> b) where
  style f = Hide
--
-- LOCN 25 2675 true
-- STYLE 
--
displayBools :: [Bool] -> Pic
--
-- LOCN 25 575 true
-- STYLE 
--
displayBools bs = 
  case bs of {
    []   -> NoPic;
    b:bs -> displayBool b `Super` Trans sqSize 0 (displayBools bs)
  }
--
-- LOCN 25 600 true
-- STYLE 
--
displayBool :: Bool -> Pic
--
-- LOCN 25 750 true
-- STYLE 
--
displayBool b =
  let bc = if b then red else green
  in Rect sqSize sqSize bc black
--
-- LOCN 25 775 true
-- STYLE 
--
sqSize = 10
--
-- LOCN 25 700 true
-- STYLE Hide
--
displayBool True
--
-- LOCN 575 750 true
-- STYLE 
--
displayBool False
--
-- LOCN 575 800 true
-- STYLE 
--
instance Edit Pic where
  style p = Picture
--
-- LOCN 25 2625 true
-- STYLE 
--
displayBools [True, False, True]
--
-- LOCN 650 600 true
-- STYLE 
{-
Supporting code
-}
-- LOCN 25 2575 true
-- STYLE comment
--
setState :: Int -> State -> State
--
-- LOCN 25 1000 true
-- STYLE 
--
setBool i bs = if i == 0 then True : (tail bs) 
                         else head bs : (setBool (i-1) (tail bs))
--
-- LOCN 25 1075 true
-- STYLE 
--
displayState (setState 2 s0)
--
-- LOCN 625 1000 true
-- STYLE 
--
setState i s = State (setBool i (getBools s))
--
-- LOCN 25 1025 true
-- STYLE 
{-
Decoding a mouse click
-}
-- LOCN 25 1875 true
-- STYLE comment
--
decode :: Int -> Int -> Maybe Int
--
-- LOCN 25 1900 true
-- STYLE 
--
decode x y =
  if ((x >= 0) && (x < (sqSize * n)) && (y >= 0) && (y <= sqSize)) then
    Just (x `div` sqSize)
  else
    Nothing
    
--
-- LOCN 25 1925 true
-- STYLE 
--
decode 17 6
--
-- LOCN 700 1900 true
-- STYLE 
--
Interaction s0' fsm'
--
-- LOCN 25 2400 true
-- STYLE 
{-
High-level FSM
-}
-- LOCN 25 150 true
-- STYLE heading1
--
data State = State [Bool]
--
-- LOCN 25 225 true
-- STYLE 
--
n = 16
--
-- LOCN 25 200 true
-- STYLE Hide
--
s0 = State (replicate n False)
--
-- LOCN 25 300 true
-- STYLE Hide
{-
Display of state
-}
-- LOCN 25 450 true
-- STYLE comment
--
instance Edit State where
  style p = View displayState Picture
--
-- LOCN 25 875 true
-- STYLE 
--
s0
--
-- LOCN 523 287 true
-- STYLE 
{-
Setting the state (not used!)
-}
-- LOCN 25 950 true
-- STYLE comment
--
decode 200 6
--
-- LOCN 700 1975 true
-- STYLE 
{-
The high-level finite-state machine
-}
-- LOCN 25 1425 true
-- STYLE comment
--
fsm :: Int -> State -> (State, Pic)
--
-- LOCN 25 1475 true
-- STYLE 
--
fsm i state =
  let { newState = toggleState i state;
        pic      = displayState newState
  } in  (newState, pic)
--
-- LOCN 25 1500 true
-- STYLE 
--
fst (fsm 2 (fst (fsm 5 s0)))
--
-- LOCN 575 1500 true
-- STYLE 
{-
The low-level finite-state machine
-}
-- LOCN 25 1700 true
-- STYLE heading1
--
fsm' :: Time -> Int -> Int -> Bool -> State' -> (State', Pic)
--
-- LOCN 25 2100 true
-- STYLE 
--
fsm' t x y isPressed state' =
  case state' of
    State' statePic wasPressed -> 
      let {state = fst statePic;
           pic   = snd statePic;
           mi = decode x y;
           click = isPressed && (not wasPressed);
           nextStatePic = case (click, mi) of {
                            (True, Just i) -> fsm i state;
                            _              -> statePic
                          };
           nextState' = State' nextStatePic isPressed
      } in (nextState', pic)
  
--
-- LOCN 25 2125 true
-- STYLE 
{-
The low-level state
-}
-- LOCN 25 1750 true
-- STYLE comment
--
data State' = State' (State, Pic) Bool
--
-- LOCN 25 1800 true
-- STYLE 
{-
Toggling the state
-}
-- LOCN 25 1150 true
-- STYLE comment
--
toggleState :: Int -> State -> State

--
-- LOCN 25 1200 true
-- STYLE 
--
toggleState i s = State (toggleBool i (getBools s))
--
-- LOCN 25 1225 true
-- STYLE 
--
toggleBool i bs 
  = if i == 0 then 
      not (head bs) : tail bs
    else
      head bs : toggleBool (i-1) (tail bs)
--
-- LOCN 25 1275 true
-- STYLE 
--
displayState (toggleState 2 s0)
--
-- LOCN 625 1200 true
-- STYLE 
--
s0' = let pic0 = displayState s0
      in State' (s0, pic0) False
--
-- LOCN 25 1825 true
-- STYLE Hide
{-
Click on this image to toggle a bit. (Note: it works equally
well in Eval or in Anim mode.)
-}
-- LOCN 325 2400 true
-- STYLE warning2
{-
(Seems to be similar to the FSM document.)
-}
-- LOCN 25 -50 true
-- STYLE abstract
