-- Vital 2.1
--
import SS
--
-- LOCN 875 75 true
-- STYLE 
{-
Experiments with mapping a (high-level) discrete
event FSM down onto a lower-leve discrete time FSM
-}
-- LOCN 25 50 true
-- STYLE title
--
s0 :: State
--
-- LOCN 25 300 true
-- STYLE 
--
stateToPic :: State -> Pic
--
-- LOCN 25 725 true
-- STYLE 
--
stateToPic = displayBools
--
-- LOCN 25 750 true
-- STYLE 
--
instance Edit (a -> b) where
  style f = Hide
--
-- LOCN 25 2750 true
-- STYLE 
--
displayBools :: [Bool] -> Pic
--
-- LOCN 25 800 true
-- STYLE 
--
displayBools bs = 
  case bs of {
    []   -> NoPic;
    b:bs -> displayBool b `Super` Trans sqSize 0 (displayBools bs)
  }
--
-- LOCN 25 825 true
-- STYLE 
--
displayBool :: Bool -> Pic
--
-- LOCN 25 975 true
-- STYLE 
--
displayBool b =
  let bc = if b then red else green
  in Rect sqSize sqSize bc black
--
-- LOCN 25 1000 true
-- STYLE 
--
sqSize = 50
--
-- LOCN 25 925 true
-- STYLE Hide
--
displayBool True
--
-- LOCN 675 975 true
-- STYLE 
--
displayBool False
--
-- LOCN 675 1025 true
-- STYLE 
--
instance Edit Pic where
  style p = Picture
--
-- LOCN 25 2700 true
-- STYLE 
--
displayBools [True, False, True]
--
-- LOCN 675 825 true
-- STYLE 
{-
Supporting code
-}
-- LOCN 25 2675 true
-- STYLE comment
{-
Decoding a mouse click
-}
-- LOCN 25 1150 true
-- STYLE comment
--
decode :: Int -> Int -> Maybe Input
--
-- LOCN 25 1200 true
-- STYLE 
--
decode x y =
  if ((x >= 0) && (x < (sqSize * n)) && (y >= 0) && (y <= sqSize)) then
    Just (x `div` sqSize)
  else
    Nothing
    
--
-- LOCN 25 1225 true
-- STYLE 
--
decode 17 6
--
-- LOCN 750 1250 true
-- STYLE 
{-
High-level FSM
-}
-- LOCN 25 150 true
-- STYLE heading1
--
type State = [Bool]
--
-- LOCN 25 225 true
-- STYLE 
--
n = 16
--
-- LOCN 25 200 true
-- STYLE Hide
--
s0 = replicate n False
--
-- LOCN 25 325 true
-- STYLE Hide
{-
Display of state
-}
-- LOCN 25 675 true
-- STYLE comment
--
s0
--
-- LOCN 525 325 true
-- STYLE 
--
decode 200 6
--
-- LOCN 750 1175 true
-- STYLE 
{-
The low-level finite-state machine
-}
-- LOCN 25 1400 true
-- STYLE heading1
--
fsmL :: Time -> Int -> Int -> Bool -> State' -> (State', Pic)
--
-- LOCN 25 1700 true
-- STYLE 
--
fsmL t x y isPressed stateL =
  case stateL of
    (statePic, wasPressed) -> 
      let {click = isPressed && (not wasPressed);
           mi = decode x y;
           state = fst statePic;
           statePic' = 
             case (click, mi) of {
               (True, Just i) -> 
                 let {state = fst statePic;
                      state' = fsm i state;
                      pic' = stateToPic state'
                 } in (state', pic');
               _   ->  statePic
             };
           stateL' = (statePic', isPressed);
           pic' = snd statePic'
      } in (stateL', pic')
  
--
-- LOCN 25 1725 true
-- STYLE 
{-
The low-level state
-}
-- LOCN 25 1450 true
-- STYLE comment
--
type StateL =  (StatePic, Bool)
--
-- LOCN 25 1550 true
-- STYLE 
--
toggleBool i bs 
  = if i == 0 then 
      not (head bs) : tail bs
    else
      head bs : toggleBool (i-1) (tail bs)
--
-- LOCN 25 475 true
-- STYLE 
--
s0L = let {pic0 = stateToPic s0;
           statePic0 = (s0, pic0);
           mousePressed = False
      } in (statePic0, mousePressed)
--
-- LOCN 25 1575 true
-- STYLE Hide
{-
Click on this image to toggle a bit. (Note: it works equally
well in Eval or in Anim mode.)
-}
-- LOCN 925 2100 true
-- STYLE warning2
--
type StatePic = (State, Pic)
--
-- LOCN 25 1500 true
-- STYLE 
--
toggleBool :: Int -> [Bool] -> [Bool]
--
-- LOCN 25 450 true
-- STYLE 
--
fsm :: Input -> State -> State
--
-- LOCN 25 375 true
-- STYLE 
--
fsm = toggleBool
--
-- LOCN 25 400 true
-- STYLE 
--
s1 = fsm 3 s0
--
-- LOCN 525 400 true
-- STYLE 
{-
Implementation of I/O representation
-}
-- LOCN 25 625 true
-- STYLE heading1
--
stateToPic s1
--
-- LOCN 675 725 true
-- STYLE 
--
type Input = Int
--
-- LOCN 25 250 true
-- STYLE 
--
interact :: state -> (input -> state -> state) -> (Int -> Int -> Maybe input) -> (state -> Pic) -> Pic
--
-- LOCN 50 2046 true
-- STYLE 
--
interact s0 fsm decode display =
  let {s0L  = let {pic0 = stateToPic s0;
                   statePic0 = (s0, pic0);
                   mousePressed = False
              } in (statePic0, mousePressed);

       fsmL = \t x y isPressed stateL ->
                case stateL of
                  (statePic, wasPressed) -> 
                    let {click = isPressed && (not wasPressed);
                         mi = decode x y;
                         state = fst statePic;
                         statePic' = 
                           case (click, mi) of {
                             (True, Just i) -> 
                               let {state = fst statePic;
                                    state' = fsm i state;
                                    pic' = stateToPic state'
                               } in (state', pic');
                             _   ->  statePic
                           };
                         stateL' = (statePic', isPressed);
                         pic' = snd statePic'
                    } in (stateL', pic')
  } in Interaction s0L fsmL
--
-- LOCN 50 2100 true
-- STYLE 
--
interact s0 fsm decode stateToPic
--
-- LOCN 25 2575 true
-- STYLE 
{-
(Seems to be similar to the FSM document.)
-}
-- LOCN 25 -50 true
-- STYLE abstract
