-- Vital 2.1
--
import SS
--
-- LOCN 625 50 true
-- STYLE 
{-
MIDI synthesis
-}
-- LOCN 25 75 true
-- STYLE title
{-
Formation, playing and saving to file of Midi sequences.
-}
-- LOCN 25 -50 true
-- STYLE abstract
{-
The Midi standard allows a musical composition to be represented by sequences
of Midi events, such as the turning on or off of a note played on a given
instrument.
-}
-- LOCN 50 150 true
-- STYLE comment
--
event1, event2, event3, event4 :: MidiEvent
--
-- LOCN 50 250 true
-- STYLE 
--
event1 =
  let {pitch = 60;
       intensity = 100;
       tick = 0
  } in NoteOn pitch intensity tick
--
-- LOCN 50 325 true
-- STYLE hide
--
import Midi
--
-- LOCN 625 75 true
-- STYLE 
{-
A pitch of 60 corresponds to middle C.
An intensity of 100 corresponds to loud.
There are 8 ticks per second.
-}
-- LOCN 475 250 true
-- STYLE comment
--
event2 =
  let {pitch = 60;
       intensity = 100;
       tick = 12
  } in NoteOff pitch intensity tick
--
-- LOCN 475 325 true
-- STYLE hide
--
sequence :: Sequence
--
-- LOCN 50 750 true
-- STYLE 
--
sequence = Sequence [track1]
--
-- LOCN 50 775 true
-- STYLE 
{-
Midi events are gathered together into Tracks, and Tracks gathered together
into Sequences
-}
-- LOCN 50 575 true
-- STYLE comment
--
track1 = 
  Track instrument [event1, event2, event3, event4]
--
-- LOCN 50 650 true
-- STYLE hide
--
instrument :: Int
--
-- LOCN 625 675 true
-- STYLE 
--
instrument = 19
--
-- LOCN 625 700 true
-- STYLE hide
{-
Midi instrument 19 represents a church organ
-}
-- LOCN 625 650 true
-- STYLE comment
{-
The primitive function
        mkMidiSequence :: Sequence -> MidiSequence
converts a Sequence into a MidiSequence (an abstract type representing
playable Midi sequences).
-}
-- LOCN 50 925 true
-- STYLE comment
--
midiSequence = mkMidiSequence sequence
--
-- LOCN 50 1050 true
-- STYLE 
{-
Such a sequence can be embedded in an Action using the primitive function
        playMidi :: MidiSequence -> Action
-}
-- LOCN 50 1175 true
-- STYLE comment
--
act = playMidi midiSequence
--
-- LOCN 50 1250 true
-- STYLE 
{-
. . . and this action can be played by double-clicking this button:
-}
-- LOCN 50 1375 true
-- STYLE comment
--
test = button "Play simple sequence" act
--
-- LOCN 50 1425 true
-- STYLE 
--
import ActionLib
--
-- LOCN 625 100 true
-- STYLE 
{-
Back to previous document
-}
-- LOCN 725 150 true
-- STYLE open1 ""
--
event3 =
  let {pitch = 48;
       intensity = 100;
       tick = 8
  } in NoteOn pitch intensity tick
--
-- LOCN 50 450 true
-- STYLE hide
--
event4 =
  let {pitch = 48;
       intensity = 100;
       tick = 30
  } in NoteOff pitch intensity tick
--
-- LOCN 475 450 true
-- STYLE hide
{-
Midi instrument names
-}
-- LOCN 25 1600 true
-- STYLE heading1
{-
The Midi standard using numbers to represent musical instruments: 0 for a piano,
. . . , 19 for a church organ, and so on.
-}
-- LOCN 50 1650 true
-- STYLE comment
{-
The complete set of instruments (not all of which may be available on a
given platform) is defined by the function:
        midiInstrumentName :: Int -> String
-}
-- LOCN 50 1725 true
-- STYLE comment
{-
For example:
-}
-- LOCN 50 1825 true
-- STYLE comment
--
midiInstrumentName 0
--
-- LOCN 100 1875 true
-- STYLE 
--
midiInstrumentName 10
--
-- LOCN 400 1875 true
-- STYLE 
--
midiInstrumentName 20
--
-- LOCN 700 1875 true
-- STYLE 
{-
The entire list of names is given by:
-}
-- LOCN 50 1975 true
-- STYLE comment
{-
Back to previous document
-}
-- LOCN 50 2125 true
-- STYLE open1 ""
--
button "Play twice" (act >> delay 3000 act)
--
-- LOCN 600 1425 true
-- STYLE 
--
[(n, midiInstrumentName n) | n <- [0 ..]]
--
-- LOCN 100 2025 true
-- STYLE 
