-- Vital 2.1
--
import SS
--
-- LOCN 675 75 true
-- STYLE 
{-
Direct manipulation of scores
-}
-- LOCN 50 75 true
-- STYLE title
{-
The original development of the MidiLib module that
displays a score, allows it to be manipulated, and converts
it to a Midi sequence.
-}
-- LOCN 50 -75 true
-- STYLE abstract
--
h1 = 4
--
-- LOCN 50 200 true
-- STYLE hide
--
w1 = 12
--
-- LOCN 50 400 true
-- STYLE hide
{-
Quarter distance between adjacent lines of the stave
-}
-- LOCN 325 200 true
-- STYLE comment
{-
Quarter distance between adjacent bars on the stave
-}
-- LOCN 325 400 true
-- STYLE comment
{-
Draw a stave
-}
-- LOCN 50 925 true
-- STYLE heading1
--
stavePic :: Pic
--
-- LOCN 75 1875 true
-- STYLE 
--
(<+>) = Super
--
-- LOCN 150 700 true
-- STYLE 
--
nBars = 18
--
-- LOCN 50 550 true
-- STYLE hide
--
line c x y w h = Trans x y (Line w h c)
--
-- LOCN 150 725 true
-- STYLE 
--
stavePic = 
  drawLines 0 [False, False, True, True, True, True, True, False] <+> 
  drawBars 0 nBars <+>
  bBox
--
-- LOCN 75 1900 true
-- STYLE 
--
drawLines y darks =
  case darks of {
    [] -> NoPic;
    dark:darks -> 
      let {color = if dark then black else lightGray;
           ln = line color 0 y wTotal 0 
      } in ln <+> drawLines (y + h3) darks
  }
--
-- LOCN 75 1000 true
-- STYLE 
--
wTotal = nBars * w3
--
-- LOCN 150 600 true
-- STYLE hide
--
drawLines :: Int -> [Bool] -> Pic
--
-- LOCN 75 975 true
-- STYLE 
--
drawBars x n =
  line black x h4 0 h5 <+>
  if n == 0 then NoPic else drawBars (x + w3) (n-1)
    
--
-- LOCN 75 1400 true
-- STYLE 
--
drawBars 0 nBars
--
-- LOCN 75 1475 true
-- STYLE 
--
h2 = h1 + h1
--
-- LOCN 150 250 true
-- STYLE hide
--
h3 = h2 + h2
--
-- LOCN 150 275 true
-- STYLE 
--
w2 = w1 + w1
--
-- LOCN 150 425 true
-- STYLE hide
--
w3 = w2 + w2
--
-- LOCN 150 450 true
-- STYLE 
{-
Distance between adjacent lines of the stave
-}
-- LOCN 325 275 true
-- STYLE comment
{-
Distance between adjacent bars on the stave
-}
-- LOCN 325 450 true
-- STYLE comment
{-
Number of bars
-}
-- LOCN 375 550 true
-- STYLE comment
{-
Total width of the stave
-}
-- LOCN 375 600 true
-- STYLE comment
{-
Some drawing functions
-}
-- LOCN 50 675 true
-- STYLE comment
--
lines = drawLines 0 [False, False, True, True, True, True, True, False]
--
-- LOCN 75 1175 true
-- STYLE 
--
h4 = 2 * h3
--
-- LOCN 650 1400 true
-- STYLE hide
--
h5 = 4 * h3
--
-- LOCN 650 1425 true
-- STYLE hide
{-
Selection mechanism
-}
-- LOCN 75 2350 true
-- STYLE heading1
--
data Score1 = Score1 
--
-- LOCN 75 2450 true
-- STYLE 
--
data Note = Note Tick Pitch DurationIndex Loudness
--
-- LOCN 50 3275 true
-- STYLE 
--
instance Edit Score1 where {
  style score = View (const stavePic) Picture;
  edit score path =
    [("Test 1", testFn1),
     ("Test 2", testFn2),
     ("Test 3", testFn3)]
}
--
-- LOCN 75 2550 true
-- STYLE 
--
testFn1 score path x y lhs rhs clip = 
  (Nothing, Nothing, Just ("Add note at x = " ++ show x ++ ", y = " ++ show y))
--
-- LOCN 75 2675 true
-- STYLE 
--
testFn2 score path x y lhs rhs clip = 
  (Nothing, Nothing, Just ("Delete note. Path is: " ++ show path ++ " and rhs is: " ++ rhs))
--
-- LOCN 75 2725 true
-- STYLE 
--
Score1
--
-- LOCN 75 3025 true
-- STYLE 
--
bBox = 
  let {w = nBars * w3;
       h = 7 * h3
  } in SelRect w h trans trans
--
-- LOCN 75 1750 true
-- STYLE hide
{-
The bounding box, part of the selection mechanism, is tranparent. Its boundary
is displayed as grey when the box is selected.
-}
-- LOCN 75 1650 true
-- STYLE comment
--
bBox :: Pic
--
-- LOCN 75 1725 true
-- STYLE 
{-
Introduce a trial datatype for testing the selection mechanism
-}
-- LOCN 75 2425 true
-- STYLE comment
{-
. . . and define an instance of Score1 with some test functions for the methods
-}
-- LOCN 75 2525 true
-- STYLE comment
{-
Here is a test picture: test it by right-clicking within its boundary
-}
-- LOCN 75 2975 true
-- STYLE comment
--
testFn3 score path x y lhs rhs clip =
    (Nothing, Nothing, Just ("Tick: " ++ show( decodeTick x) ++ ", pitch: " ++ show (decodePitch y)))
--
-- LOCN 75 2800 true
-- STYLE 
--
decodeTick :: Int -> Tick
--
-- LOCN 75 2875 true
-- STYLE 
--
decodeTick x = x `div` w3
--
-- LOCN 75 2900 true
-- STYLE 
--
decodePitch :: Int -> StaveIndex
--
-- LOCN 475 2875 true
-- STYLE 
--
decodePitch y = (y + h1) `div` h2
--
-- LOCN 475 2900 true
-- STYLE 
{-
Define the score change functions
-}
-- LOCN 50 3225 true
-- STYLE heading1
--
data Score = Score [Note]
--
-- LOCN 50 3300 true
-- STYLE 
--
addNote :: Note -> Score -> Score
--
-- LOCN 50 3775 true
-- STYLE 
--
addNote note score =
  case score of
    Score notes -> Score (note : notes)
--
-- LOCN 50 3800 true
-- STYLE 
--
deleteNote :: Tick -> Pitch -> Score -> Score
--
-- LOCN 50 3900 true
-- STYLE 
--
deleteNote tick pitch score =
  case score of
    Score notes -> Score [note | note <- notes, differsFrom tick pitch note]
--
-- LOCN 50 3925 true
-- STYLE 
--
differsFrom :: Tick -> Pitch -> Note -> Bool
--
-- LOCN 50 4025 true
-- STYLE 
--
differsFrom tick pitch note =
  case note of 
    Note tick' pitch' di loud -> 
      (tick /= tick') || (pitch /= pitch')
--
-- LOCN 50 4050 true
-- STYLE 
--
note1 = Note 2 6 3 100
--
-- LOCN 725 4050 true
-- STYLE hide
--
note2 = Note 3 6 2 100
--
-- LOCN 725 4075 true
-- STYLE hide
--
note3 = Note 2 0 1 100
--
-- LOCN 725 4100 true
-- STYLE hide
--
note4 = Note 2 6 5 80
--
-- LOCN 725 4125 true
-- STYLE hide
{-
Test data
-}
-- LOCN 725 4025 true
-- STYLE comment
--
differsFrom 2 6 note1
--
-- LOCN 50 4175 true
-- STYLE 
--
differsFrom 2 6 note2
--
-- LOCN 50 4250 true
-- STYLE 
{-
Some tests
-}
-- LOCN 50 4150 true
-- STYLE comment
--
emptyScore = Score []
--
-- LOCN 50 3325 true
-- STYLE hide
--
score1 = addNote note3 (addNote note2 (addNote note1 emptyScore))
--
-- LOCN 50 4350 true
-- STYLE defaultStyle
--
deleteNote 3 6 score1
--
-- LOCN 50 4425 true
-- STYLE defaultStyle
{-
Note durations
-}
-- LOCN 50 4525 true
-- STYLE heading1
--
data TimeValue =
  Demisemiquaver | Semiquaver | Quaver | Crotchet | Minim | Semibreve | Breve
--
-- LOCN 50 4575 true
-- STYLE 
--
durationOf :: DurationIndex -> Int
--
-- LOCN 50 4700 true
-- STYLE 
--
durationOf i = 2 ^ i
--
-- LOCN 50 4725 true
-- STYLE 
--
durationIndexOf Demisemiquaver
--
-- LOCN 500 4650 true
-- STYLE 
--
durationIndexOf Crotchet
--
-- LOCN 500 4700 true
-- STYLE 
{-
(In USA parlance, a crotchet is known as a quarter-note)
-}
-- LOCN 400 4550 true
-- STYLE comment
{-
Time-value pictures
-}
-- LOCN 50 5200 true
-- STYLE heading1
--
drawTimeValue :: Color -> TimeValue -> Pic
--
-- LOCN 50 5300 true
-- STYLE 
{-
Draw (at the origin) a note with the specified time value
-}
-- LOCN 50 5275 true
-- STYLE comment
--
drawTimeValue color tv =
  let {solidBlob  = Trans (-w1) (-h2) (Ellipse w2 h3 color trans);
       hollowBlob = Trans (-w1) (-h2) (Ellipse w2 h3 trans color);
       top        = - 4 * h3;
       cLine      = line color;
       vLine      = cLine w1 0 0 top;
       hook1      = cLine w1 top w1 h3;
       hook2      = Trans 0 h3 hook1;
       hook3      = Trans 0 h3 hook2;
       crotchet   = vLine <+> solidBlob;
       quaver     = crotchet <+> hook1;
       semiquaver = quaver <+> hook2;
       leftBar    = cLine (-w1) (-h2) 0 h3;
       rightBar   = Trans w2 0 leftBar
  } in case tv of {
         Demisemiquaver -> semiquaver <+> hook3;
         Semiquaver     -> semiquaver;
         Quaver         -> quaver;
         Crotchet       -> crotchet;
         Minim          -> vLine <+> hollowBlob;
         Semibreve      -> hollowBlob;
         Breve          -> hollowBlob <+> leftBar <+> rightBar
      }
--
-- LOCN 50 5325 true
-- STYLE 
--
Score []
--
-- LOCN 25 5750 true
-- STYLE 
--
drawTimeValue red Minim
--
-- LOCN 400 5750 true
-- STYLE 
--
drawTimeValue red Demisemiquaver
--
-- LOCN 650 5750 true
-- STYLE 
--
drawTimeValue red Breve
--
-- LOCN 950 5750 true
-- STYLE 
{-
Pitch values
-}
-- LOCN 50 8350 true
-- STYLE heading1
{-
In the MIDI standard:
  - middle C is represented by 60
  - successive half-tones are represented by successive integers
  - an octave corresponds to +12
-}
-- LOCN 50 8400 true
-- STYLE comment
--
type KeySignature = StaveIndex -> MidiPitch
--
-- LOCN 50 8600 true
-- STYLE 
{-
A key signature is a mapping from the Stave Index (in which upper C is represented by 0
and middle C is represented by 14) to the Midi Pitch (in which middle C is represented by 60
and upper C by 72).
-}
-- LOCN 50 8525 true
-- STYLE comment
{-
- The Tick represents the time of the start of the note: 4 ticks correspond
   to a crotchet (aka quarter note).
- The Pitch represents the position on the stave: 0 corresponds to upper C,
   14 corresponds to middle C.
- The DurationIndex is 0 (demisemiquaver) through to 6 (breve).
- The loudness is an integer in the range 0 .. 100 (silent to max intensity)
-}
-- LOCN 500 3275 true
-- STYLE comment
{-
Direct manipulation
-}
-- LOCN 50 6950 true
-- STYLE heading1
--
instance Edit Score where {
  style score = View drawScore Picture;
  edit score path =
    [("Insert demisemiquaver", insertNoteFn 0),
     ("Insert semiquaver",     insertNoteFn 1),
     ("Insert quaver",         insertNoteFn 2),
     ("Insert crotchet",       insertNoteFn 3),
     ("Insert minim",          insertNoteFn 4),
     ("Insert semibreve",      insertNoteFn 5),
     ("Insert breve",          insertNoteFn 6),

     ("Delete note",           deleteNoteFn),
     ("Forte",                 error "Not yet defined"),
     ("Piano",                 error "Not yet defined"),
     ("Simplify",              simplifyFn),
     ("TEST", testFn)]
}
--
-- LOCN 50 7000 true
-- STYLE 
--
drawScore :: Score -> Pic
--
-- LOCN 50 6000 true
-- STYLE 
{-
Display a score
-}
-- LOCN 50 5950 true
-- STYLE heading1
--
drawScore score =
  case score of 
    Score notes -> stavePic <+> drawNotes notes
--
-- LOCN 50 6025 true
-- STYLE 
--
drawNotes :: [Note] -> Pic
--
-- LOCN 50 6150 true
-- STYLE 
--
drawNotes notes =
  case notes of {
    []         -> NoPic;
    note:notes -> drawNote note <+> drawNotes notes
  }
--
-- LOCN 50 6175 true
-- STYLE 
--
drawNote :: Note -> Pic
--
-- LOCN 50 6300 true
-- STYLE 
--
drawNote note =
  case note of
    Note tick pitch i loudness -> 
      let {x = tick * w3 + w2;
           y = pitch * h2;
           color = blue;
           tv = timeValueOf i;
           notePic = drawTimeValue color tv
      } in Trans x y notePic
--
-- LOCN 50 6325 true
-- STYLE 
--
drawNote note1
--
-- LOCN 750 6225 true
-- STYLE 
--
drawNote note2
--
-- LOCN 750 6375 true
-- STYLE 
--
note2
--
-- LOCN 600 6375 true
-- STYLE 
{-
Note: not difficult to extend this function to draw a pair (bass, treble) of staves
-}
-- LOCN 75 2150 true
-- STYLE comment
--
durationIndexOf Breve
--
-- LOCN 500 4775 true
-- STYLE 
--
durationIndexOf :: TimeValue -> Int
--
-- LOCN 50 4625 true
-- STYLE 
--
durationIndexOf tv = ord tv
--
-- LOCN 50 4650 true
-- STYLE 
--
timeValueOf :: Int -> TimeValue
--
-- LOCN 50 4875 true
-- STYLE 
--
timeValueOf i = 
  case i of {
    0 -> Demisemiquaver;
    1 -> Semiquaver;
    2 -> Quaver;
    3 -> Crotchet;
    4 -> Minim;
    5 -> Semibreve;
    6 -> Breve
  }
--
-- LOCN 50 4900 true
-- STYLE 
{-
(Ideally, there would be 
  an Enum.toEnum  method . . .)
-}
-- LOCN 400 4925 true
-- STYLE comment
--
note1
--
-- LOCN 600 6225 true
-- STYLE 
--
note3
--
-- LOCN 600 6477 true
-- STYLE 
--
drawNote note3
--
-- LOCN 750 6475 true
-- STYLE 
--
score2 = Score [Note 10 8 1 100, Note 10 9 3 100, Note 7 10 6 100, Note 0 4 0 100, Note 12 9 6 100, Note 9 9 4 100, Note 7 5 0 100, Note 3 6 2 100, Note 2 0 1 100, Note 2 6 5 80, Note 5 12 4 100]
--
-- LOCN 50 6700 true
-- STYLE 
--
notes = [note2, note3, note4, note5]
--
-- LOCN 50 6625 true
-- STYLE hide
--
drawNotes notes
--
-- LOCN 600 6575 true
-- STYLE 
--
note5 = Note 5 12 4 100
--
-- LOCN 50 6550 true
-- STYLE hide
--
insertNoteFn :: DurationIndex ->
                Score -> Path -> Coord -> Coord -> String -> String -> String -> (MString, MString, MString)
--
-- LOCN 50 7400 true
-- STYLE 
--
testFn score path x y lhs rhs clip =
    (Nothing, Nothing, Just ("Tick: " ++ show( decodeTick x) ++ ", pitch: " ++ show (decodePitch y)))
--
-- LOCN 50 7325 true
-- STYLE 
--
insertNoteFn i score path x y lhs rhs clip =
  let {tick = show (decodeTick x);
       staveIndex = show (decodePitch y);
       durationIndex = show i;
       loudness = show (100);
       note = "Note " ++ tick ++ " " ++ staveIndex ++ " " ++ durationIndex ++ " " ++ loudness;
       rhs' = "addNote (" ++ note ++ ") (" ++ rhs ++ ")"
  } in (Just rhs', Nothing, Just "Note added")
--
-- LOCN 50 7450 true
-- STYLE 
--
deleteNoteFn :: Score -> Path -> Coord -> Coord -> String -> String -> String -> (MString, MString, MString)
--
-- LOCN 50 7650 true
-- STYLE 
--
deleteNoteFn score path x y lhs rhs clip =
  let {tick = show (decodeTick x);
       pitch = show (decodePitch y);
       rhs' = "deleteNote " ++ tick ++ " " ++ pitch ++ " (" ++ rhs ++ ")";
       message' = "OK"
  } in (Just rhs', Nothing, Just "Note deleted")
--
-- LOCN 50 7675 true
-- STYLE 
--
instance Show Note where {
  shows note s = 
    case note of
      Note tick pitch di loud -> "Note " ++ show tick ++ " " ++ show pitch ++
                                              " " ++ show di ++ " " ++ show loud ++ s
}
--
-- LOCN 50 3425 true
-- STYLE 
--
shows note1  ""
--
-- LOCN 50 3550 true
-- STYLE 
--
show note1
--
-- LOCN 325 3550 true
-- STYLE 
--
show [note1, note2, note3]
--
-- LOCN 550 3550 true
-- STYLE 
--
instance Show Score where
  shows score s =
    case score of
      Score notes -> "Score " ++ show notes
--
-- LOCN 50 3650 true
-- STYLE 
--
show score1
--
-- LOCN 550 3650 true
-- STYLE 
--
simplifyFn :: Score -> Path -> Coord -> Coord -> String -> String -> String -> (MString, MString, MString)
--
-- LOCN 50 7825 true
-- STYLE 
--
simplifyFn score path x y lhs rhs clip =
  let rhs' = show score
  in (Just rhs', Nothing, Just "Simplified")
--
-- LOCN 50 7850 true
-- STYLE 
--
cMajor :: KeySignature
--
-- LOCN 50 8725 true
-- STYLE 
{-
The Midi pitches for a chromatic scale:
  60   C   - middle C
  61   C#
  62   D
  63   D#
  64   E
  65   F
  66   F#
  67   G
  68   G#
  69   A
  70   A#
  71   B
  72   C   - upper C
-}
-- LOCN 675 8625 true
-- STYLE comment
--
cMajorPitches = 
  [60, 62, 64, 65,67, 69, 71] ++ map (+ 12) cMajorPitches
  
--
-- LOCN 50 8650 true
-- STYLE hide
--
cMajor i = cMajorPitches !! (14 - i)
--
-- LOCN 50 8750 true
-- STYLE 
--
score3 = Score [Note 1 14 2 100]
--
-- LOCN 60 8133 true
-- STYLE 
--
cMajor 14
--
-- LOCN 450 8725 true
-- STYLE 
--
cMajor 10
--
-- LOCN 450 8800 true
-- STYLE 
{-
Playing a score
-}
-- LOCN 50 8900 true
-- STYLE heading1
--
notesToMidiEvents :: [Note] -> [MidiEvent]
--
-- LOCN 50 9000 true
-- STYLE 
--
notesToMidiEvents notes =
  case notes of {
    []    ->   [];
    (Note tick staveIndex durIndex loud) : notes ->
      let {midiPitch = cMajor staveIndex;
           dur = durationOf durIndex;
           event1 = NoteOn  midiPitch loud tick;
           event2 = NoteOff midiPitch loud (tick + dur)
      } in event1 : event2 : notesToMidiEvents notes
   }
          
--
-- LOCN 50 9025 true
-- STYLE 
--
instance Show MidiEvent where
  shows me s = 
    case me of {
      NoteOff midiPitch loud tick -> "NoteOff " ++ show midiPitch ++ " " ++ show loud ++ " " ++ show tick ++ s;
      NoteOn  midiPitch loud tick -> "NoteOn  " ++ show midiPitch ++ " " ++ show loud ++ " " ++ show tick ++ s
    }
--
-- LOCN 50 9225 true
-- STYLE 
--
import Midi
--
-- LOCN 50 8950 true
-- STYLE 
--
scoreToMidiSequence :: Instrument -> Score -> MidiSequence
--
-- LOCN 50 9350 true
-- STYLE 
--
scoreToMidiSequence instrument score =
  case score of
    Score notes ->
      let {midiEvents = notesToMidiEvents notes;
           track = Track instrument midiEvents
      } in Sequence [track]
--
-- LOCN 50 9375 true
-- STYLE 
--
play :: Instrument -> Score -> Pic
--
-- LOCN 50 9550 true
-- STYLE 
--
play instrument score =
  let {seq = scoreToMidiSequence instrument score;
       midiSeq = mkMidiSequence seq;
       action = playMidi midiSeq;
       pic = PicAction 80 20 cyan blue action
  } in pic
--
-- LOCN 50 9575 true
-- STYLE 
--
theScore = deleteNote 4 11 (addNote (Note 4 5 1 100) (addNote (Note 17 14 6 100) (deleteNote 17 14 (addNote (Note 17 14 1 100) (Score [Note 1 4 3 100, Note 15 2 0 100, Note 14 1 0 100, Note 13 0 0 100, Note 12 0 0 100, Note 11 10 5 100, Note 10 9 4 100, Note 9 8 2 100, Note 8 7 1 100, Note 7 6 0 100, Note 6 6 1 100, Note 4 11 6 100, Note 3 7 4 100, Note 2 5 3 100])))))
--
-- LOCN 50 9725 true
-- STYLE 
--
play 70 theScore
--
-- LOCN 50 9925 true
-- STYLE 
