-- Vital 2.1
--
stream next safe prod cons z xs =
  let y = next z
  in if safe z y then
       y : stream next safe prod cons (prod z y) xs
     else
       stream next safe prod cons (cons z (head xs)) (tail xs)
--
-- LOCN 50 1625 true
-- STYLE Hide
--
import Ratio
--
-- LOCN 50 1225 true
-- STYLE 
--
intPart r = (numerator r) `div` (denominator r)
--
-- LOCN 50 1250 true
-- STYLE Hide
--
decimalise ds =
  let {es = map (10 *) ds;
       es' = norm es;
       d = head es';  ds' = tail es';
       q = d `div` 10;  r = d `mod` 10}
  in q : decimalise (r : ds')
--
-- LOCN 50 700 true
-- STYLE Hide
--
piDecimal = decimalise piMixed
--
-- LOCN 50 1850 true
-- STYLE Hide
{-
Derivation of spigot algorithm for pi  (Jeremy Gibbons, 2003)
-}
-- LOCN 50 50 true
-- STYLE title
--
norm es = 
  let {d = head es;  ds = tail es;
       next = \x -> case x of (j,i,qrst) -> intPart (ext qrst 0);
       safe = \x n -> case x of (j,i,qrst) -> intPart (ext qrst 20) == n;
       prod = \x n -> case x of (j,i,qrst) -> (j+1, i, mm ((2*j)+3, (-n)*((2*j)+3),0,j+1) qrst);
       cons = \x d -> case x of (j,i,qrst) -> (j, i+1, mm qrst (i, i*d, 0, (2*i)+1));
       init = \d -> (0,1,(1,d,0,1));
       ext = \y x -> case y of (q,r,s,t) -> ((q*x)+r) % ((s*x)+t);
       mm = \x y -> case (x,y) of ((q,r,s,t),(q',r',s',t')) -> ((q*q')+(r*s'), (q*r')+(r*t'), (s*q')+(t*s'), (s*r')+(t*t'))}
  in stream next safe prod cons (init d) ds
--
-- LOCN 50 925 true
-- STYLE Hide
--
import SS
--
-- LOCN 800 100 true
-- STYLE 
{-
The normalising function needs to be able to alternate between consuming input digits and
producing output ones. The way it does this is defined using the following generic function to create
a streaming process.

stream ::  
           next :: (a -> c)           ->           Function to generate next element from the state
           safe :: (a -> c -> Bool)   ->           Is it "safe" to commit to the present output value?
           prod :: (a -> c -> a)      ->           Function that produces an element and updates the state
           cons :: (a -> b -> a)      ->           Function that consumes an element and updates the state
           initState :: a             ->           The initial state
           xs : [b]                   ->           The input sequence

           [c]                                     The output sequence

-}
-- LOCN 50 1325 true
-- STYLE commentM
{-
The above function relies upon the following function to "normalise" the digits of a  mixed-radix numeral (that is, 
to ensure that each digit lies in the range of 0 to 2) . It does this by propagating carries as it scans
the numeral (in the "contrary" direction, from left to right).
-}
-- LOCN 50 850 true
-- STYLE comment
--
piMixed = repeat 2
--
-- LOCN 50 525 true
-- STYLE numeral
{-
The task of producing the decimal expansion of pi is thus simply a matter of converting from this mixed-radix
base to base 10. The following function does this by pointwise multiplying its digits by ten, normalising the
resultant sequence and extracting the integer part, and repeating the process on the fractional part.
-}
-- LOCN 50 625 true
-- STYLE comment
{-
Rabinowitz & Wagon showed (1995) that the value of pi can be expressed as the limit of the infinite series
        pi = 2 + (1/3)(2 + (2/5)(2 + (3/7)(2 +  . . . )))

This series can be viewed as the numeral
        2.22222222 . . .
in a mixed-radix number system, where the successive bases are (1/3, 2/5, 3/7, . . .).

The above numeral can be represented by a lazily-evaluated infinite list:
-}
-- LOCN 50 350 true
-- STYLE comment
{-
Given the above definitions, it is now trivial to produce the decimal expansion of pi; it is simply a matter of
applying the radix-conversion  function to the original, mixed-radix definition of pi:
-}
-- LOCN 50 1775 true
-- STYLE comment
{-
Without extensive optimisation, the above algorithm is rather expensive (in time and space). Thus, it
is prudent not to attempt to explore too many digits of this infinite numeral . . .
-}
-- LOCN 50 1925 true
-- STYLE comment
--
take 10 piDecimal
--
-- LOCN 50 2000 true
-- STYLE numeral
--
numeral = list2
--
-- LOCN 800 125 true
-- STYLE hide
{-
Click here to see this alternative algorithm
-}
-- LOCN 50 2200 true
-- STYLE open "PiDigits"
{-
Abstract:    Rabinowitz and Wagon (American Mathematical Monthly 102(3):195--203, 1995) present a spigot 
algorithm for computing the digits of p. A spigot algorithm yields its outputs incrementally, and does 
not reuse them after producing them. Their algorithm is inherently bounded; it requires a commitment in 
advance to the number of digits to be computed, and in fact might still produce an incorrect last few 
digits. We propose two streaming algorithms based on the same characterization of p, with the same 
incremental characteristics but without requiring the prior bound. 
-}
-- LOCN 50 200 true
-- STYLE comment
{-
The above function uses infinite precision rational numbers (constructed by the "%" operator imported from 
the Ratio library module) and an auxiliary function
    intPart :: Ratio -> Integer
that extracts the integer part of a rational
-}
-- LOCN 50 1125 true
-- STYLE comment
{-
But (as explained in the above-referenced paper) by starting from the same expansion for pi, but instead regarding
it as the composition of a series of linear fractional transformations, a simpler and very much more efficient
algorithm can be derived. 
-}
-- LOCN 50 2125 true
-- STYLE comment
{-
http://web.comlab.ox.ac.uk/oucl/work/jeremy.gibbons/publications/index.html#spigot
-}
-- LOCN 50 100 true
-- STYLE comment
{-
Back to previous document
-}
-- LOCN 800 150 true
-- STYLE open1 ""
{-
Back to previous document
-}
-- LOCN 50 2250 true
-- STYLE open1 ""
{-
Detailed exposition of a 'spigot' algorithm (Gibbons, 2003) for computing the
infinite decimal expansion of pi.
-}
-- LOCN 50 -50 true
-- STYLE abstract
