-- Vital 2.1
--
barChart bar width height title values =
    content bar width height values # axes width height title

--
-- LOCN 75 2575 true
-- STYLE Hide
--
line x1 y1 x2 y2 = Trans x1 y1 (Line (x2 - x1) (y2 - y1) black)
--
-- LOCN 75 275 true
-- STYLE Hide       
--
(#) = Super
--
-- LOCN 75 574 true
-- STYLE Hide
--
axes width height title =
  let { overhang = height `div` 5;
        xAxis = line ( -overhang) height (width + overhang) height;
        yAxis = line 0 0 0 (height + overhang);
        titleFormat = Format SansSerif 16 black False False;
        titleLine = Trans (width `div` 3) 
                          (height + overhang) 
                          (PicText titleFormat title) }
  in xAxis # yAxis # titleLine

--
-- LOCN 75 2175 true
-- STYLE Hide
--
rect color lineColor x1 y1 x2 y2 = 
    Trans x1 y1 (Rect (x2 - x1) (y2 - y1) color lineColor)
--
-- LOCN 75 425 true
-- STYLE Hide
--
prices = [42, 56, 66, 81, 91, 78, 61, 54, 37,
          43, 44, 50, 66, 60, 58, 71, 74, 77,
          74, 61, 51, 43, 47, 53, 51, 60, 58,
          53, 49, 43, 31, 22, 25, 18, 20, 23]
--
-- LOCN 75 3025 true
-- STYLE Hide
{-
Development & testing of the Bar Chart library   
-}
-- LOCN 75 75 true
-- STYLE title
{-
A black line from (x1,y1) to (x2,y2)
-}
-- LOCN 75 225 true
-- STYLE comment
{-
A rectangle bounded by (x1,y1) and (x2,y2) with specified colours.
-}
-- LOCN 75 375 true
-- STYLE comment
{-
An infixed version of the superposition function
-}
-- LOCN 75 525 true
-- STYLE comment
{-
A subsidiary function that draws axes and a title.
-}
-- LOCN 75 2125 true
-- STYLE comment
{-
Now, the subsidiary function that draws the contents of the bar chart. Its first argument is a
a function that draws a single bar, the next two arguments specify the size of the bar chart and the
last argument is a list of the values to be displayed. The function computes:
  * barWidth        - the width of each bar.
  * maxValue        - the value of the max element in the given list of values. (If the list
                      is empty, it takes the maximum value to be 1).
  * normValues      - the original list of values, normalised so as lie within the alloted height
                      of the bar chart.
  * bars            - a list of rectangles whose heights correspond to the list of normalised values.
  * f               - a function that takes two pictures, p and q, shifts q to the right by one
                      bar width, and yields p superimposed on the shifted version of q.
  * foldr f NoPics bars
                    - the overall picture, obtained by right-folding f over the list of individual
                      bars (with the null picture, NoPic, as the zero).
-}
-- LOCN 75 1525 true
-- STYLE commentM
--
yellowBarChart 900 150 "Share prices (enlarged view)" prices
--
-- LOCN 75 3425 true
-- STYLE Picture
--
blueBarChart 250 100 "Share prices" prices
--
-- LOCN 76 3224 true
-- STYLE Picture
--
p1 = line 0 0 100 50
--
-- LOCN 675 225 true
-- STYLE      Picture
--
p2 = rect cyan blue 0 0 80 40
--
-- LOCN 675 375 true
-- STYLE      Picture
--
p2 # p1
--
-- LOCN 675 525 true
-- STYLE      Picture
--
blueGreenBarChart 800 100 "Share prices" prices
--
-- LOCN 75 4775 true
-- STYLE Picture
--
solidBar barColor sideBarColor lineColor barPitch height value =
  let { barWidth = barPitch `div` 2;
        barSide  = barPitch `div` 4;
        hv    = height - value;
        bws   = barWidth + barSide;
        hvb   = height - (value + barSide);
        front = rect barColor lineColor 0 hv barWidth height;
        top   = Poly [(0, hv), (barWidth, hv), (bws, hvb),(barSide, hvb)] barColor lineColor;
        side  = Poly [(barWidth, height), (bws, (height - barSide)), (bws, hvb), (barWidth, hv)] sideBarColor lineColor
  }
  in front # (top # side)
--
-- LOCN 75 3975 true
-- STYLE Hide
--
blueGreenBar = solidBar blue green white
--
-- LOCN 75 4275 true
-- STYLE Hide
{-
"3-D" look barcharts
-}
-- LOCN 75 3775 true
-- STYLE heading1
{-
By defining a "solid" bar function, a bar chart can be given a 3-D appearance
-}
-- LOCN 75 3875 true
-- STYLE comment
--
plainBar color edgeColor barWidth height value =
    rect color edgeColor 0 (height - value) barWidth height
--
-- LOCN 75 975 true
-- STYLE Hide
--
yellowBar 20 80 30
--
-- LOCN 75 1275 true
-- STYLE   Picture
--
content bar width height values =
    let { barWidth = width `div` length values;
          maxValue = maximum (1 : values);
          normValues = [v * height `div` maxValue | v <- values];
          bars = [bar barWidth height v | v <- normValues];
          f = \p q -> p # (Trans barWidth 0 q)
    }
    in foldr f NoPic bars
--
-- LOCN 75 1925 true
-- STYLE Hide
--
yellowBar = plainBar yellow red
--
-- LOCN 75 1125 true
-- STYLE Hide
--
blueGreenBar 50 80 50
--
-- LOCN 575 4275 true
-- STYLE   Picture
--
redMagentaBar = solidBar red magenta yellow
--
-- LOCN 75 4325 true
-- STYLE Hide
--
redMagentaBar 50 80 70
--
-- LOCN 825 4275 true
-- STYLE   Picture
--
redMagentaBarChart 500 80 "Share prices" prices
--
-- LOCN 75 5024 true
-- STYLE Picture
--
blueBar = plainBar blue white
--
-- LOCN 375 1125 true
-- STYLE Hide
--
blueBar 20 80 50
--
-- LOCN 375 1275 true
-- STYLE   Picture
--
blueGreenBarChart = barChart blueGreenBar
--
-- LOCN 75 4525 true
-- STYLE Hide
--
redMagentaBarChart = barChart redMagentaBar
--
-- LOCN 75 4575 true
-- STYLE Hide
--
outlineBarChart = barChart outlineBar
--
-- LOCN 75 2875 true
-- STYLE Hide
--
outlineBarChart 500 100 "Share prices" prices
--
-- LOCN 525 3225 true
-- STYLE Picture
{-
Definition of the Barchart functions
-}
-- LOCN 75 725 true
-- STYLE heading1
{-
First, a function to draw a plain bar, in specified colours, with specified size
-}
-- LOCN 75 825 true
-- STYLE comment
{-
. . . and some specialised versions of this function
-}
-- LOCN 75 1075 true
-- STYLE comment
--
outlineBar = plainBar Transparent black
--
-- LOCN 675 1125 true
-- STYLE Hide
--
outlineBar 20 80 70
--
-- LOCN 675 1275 true
-- STYLE   Picture
{-
. . . and some examples
-}
-- LOCN 75 1225 true
-- STYLE comment
{-
Some subsidiary functions
-}
-- LOCN 75 175 true
-- STYLE comment
--
plainBar :: Color -> Color -> Int -> Int -> Int -> Pic
--
-- LOCN 75 925 true
-- STYLE   
--
content :: (Int -> Int -> Int -> Pic) -> Int -> Int -> [Int] -> Pic
--
-- LOCN 75 1875 true
-- STYLE   
{-
Example
-}
-- LOCN 675 1875 true
-- STYLE comment
--
content blueBar 200 80 [1 .. 20]
--
-- LOCN 675 1925 true
-- STYLE   Picture
{-
Finally, the top-level function for drawing a generic bar chart, is obtained by
superimposing the axes and the content functions
-}
-- LOCN 75 2425 true
-- STYLE comment
--
barChart :: (Int -> Int -> Int -> Pic) -> Int -> Int -> String -> [Int] -> Pic
--
-- LOCN 77 2517 true
-- STYLE   
{-
Some specialised versions of this function
-}
-- LOCN 75 2675 true
-- STYLE heading1
--
yellowBarChart, blueBarChart, outlineBarChart :: Int -> Int -> String -> [Int] -> Pic
--
-- LOCN 75 2725 true
-- STYLE   
--
yellowBarChart = barChart yellowBar
--
-- LOCN 75 2775 true
-- STYLE Hide
--
blueBarChart = barChart blueBar
--
-- LOCN 75 2825 true
-- STYLE Hide
{-
Some sample data
-}
-- LOCN 75 2975 true
-- STYLE heading1
{-
. . . and some sample charts
-}
-- LOCN 75 3150 true
-- STYLE comment
--
solidBar :: Color -> Color -> Color -> Int -> Int -> Int -> Pic
--
-- LOCN 75 3925 true
-- STYLE   
{-
Some specialised versions of this function
-}
-- LOCN 75 4225 true
-- STYLE comment
{-
Examples
-}
-- LOCN 575 4225 true
-- STYLE comment
{-
. . . and some specialised versions of the bar chart function
-}
-- LOCN 75 4475 true
-- STYLE comment
{-
. . . and some sample charts
-}
-- LOCN 75 4675 true
-- STYLE comment
--
multiContent bar width height values =
    let { barWidth = width `div` length values;
          bars = [bar barWidth height v | v <- values];
          f = \p q -> p # (Trans barWidth 0 q)
    }
    in foldr f NoPic bars
--
-- LOCN 75 6275 true
-- STYLE Hide
--
multiContent :: (Int -> Int -> [Int] -> Pic) -> Int -> Int -> [[Int]] -> Pic
--
-- LOCN 75 6225 true
-- STYLE   
--
multiBar :: Int -> Int -> [Int] -> Pic
--
-- LOCN 75 5525 true
-- STYLE   
--
multiBar barWidth height values =
   let { n = length values;
         w = barWidth `div` (n + 1);
         gap = 2;
         bs = [plainBar (fst p) black (w - gap) height (snd p) | p <- zip colors values];
         f = \p q -> p # Trans w 0 q
   }
   in foldr f NoPic bs
--
-- LOCN 75 5575 true
-- STYLE Hide
--
colors = [red, blue, yellow, magenta, cyan, black, green]
--
-- LOCN 75 5825 true
-- STYLE Hide
--
transpose xss = case xss of {
                  []:_  -> [];
                  xss -> (map head xss) : transpose (map tail xss)
                 }
--
-- LOCN 75 6725 true
-- STYLE Hide
{-
Barcharts for more than one sequence
-}
-- LOCN 75 5275 true
-- STYLE heading1
{-
The "bar" function becomes
-}
-- LOCN 75 5475 true
-- STYLE comment
{-
. . . where "colors" is a list defining the colour to be used for each sequence.
-}
-- LOCN 75 5775 true
-- STYLE comment
{-
The "content" function (with the "normalising" function omitted, for simplicity) becomes
-}
-- LOCN 75 6175 true
-- STYLE comment
{-
For example
-}
-- LOCN 75 5925 true
-- STYLE comment
--
multiBar 40 80  [30, 50, 20]
--
-- LOCN 75 5975 true
-- STYLE Picture
--
multiBar 40 80  [30, 50, 20, 80, 70, 15, 25]
--
-- LOCN 448 5972 true
-- STYLE Picture
--
transpose :: [[a]] -> [[a]]
--
-- LOCN 75 6675 true
-- STYLE  
{-
The top-level bar chart function is
-}
-- LOCN 75 6425 true
-- STYLE comment
--
multiBarChart :: Int -> Int -> String -> [[Int]] -> Pic
--
-- LOCN 75 6475 true
-- STYLE  
--
multiBarChart width height title listOfSequences =
      multiContent multiBar width height (transpose listOfSequences) # axes width height title
--
-- LOCN 75 6525 true
-- STYLE Hide
--
multiBarChart 800 100 "Mirror images"   [prices, reverse prices]
--
-- LOCN 75 6925 true
-- STYLE Picture
{-
. . . where the function "transpose" (which transposes a list of lists) is defined as
-}
-- LOCN 75 6625 true
-- STYLE comment
{-
Here is an example of the function applied to a list of two sequences
-}
-- LOCN 75 6875 true
-- STYLE comment
{-
. . . and here applied to a (somewhat contrived) list of four sequences
-}
-- LOCN 75 7175 true
-- STYLE comment
--
let { xs1 = prices;
      xs2 = drop 5 prices ++ take 5 prices;
      xs3 = [1, 5 ..];
      xs4 = [100 - x | x <- xs1]
    }
    in multiBarChart 900 150 "Four sequences" [xs1, xs2, xs3, xs4]
--
-- LOCN 75 7225 true
-- STYLE Picture
{-
Back to previous document
-}
-- LOCN 75 7596 true
-- STYLE open ""
--
import SS
--
-- LOCN 675 125 true
-- STYLE 
{-
Back to previous document
-}
-- LOCN 675 150 true
-- STYLE open ""
{-
The approach can easily be generalised to allow the creation of bar charts for more
than one sequence. It simply involves lifting the "bar" function to operate on a list
of values/colours rather than on just a single value and colour.
-}
-- LOCN 75 5325 true
-- STYLE comment
{-
Detailed, interactive, step-by-step development of the functions present
in the Barchart library module.
-}
-- LOCN 75 -50 true
-- STYLE abstract
