-- Vital 2.1
--
barChart1 "Sequence of Primes" primes100
--
-- LOCN 75 725 true
-- STYLE Picture
--
diffs ns = [fst p - snd p | p <- zip (tail ns) ns]
--
-- LOCN 75 1075 true
-- STYLE Hide
--
primeDiffs = diffs primes100
--
-- LOCN 75 1200 true
-- STYLE 
--
primes100 = take 100 primes
--
-- LOCN 75 550 true
-- STYLE Hide
--
barChart1 "Differences between adjacent primes" primeDiffs
--
-- LOCN 75 1375 true
-- STYLE Picture
{-
Distribution of the prime numbers
-}
-- LOCN 50 75 true
-- STYLE title
--
import Pic2
--
-- LOCN 75 200 true
-- STYLE        
--
import Primes
--
-- LOCN 275 200 true
-- STYLE        
--
primes
--
-- LOCN 75 425 true
-- STYLE 
{-
Import modules for drawing bar charts and for generating a list of the prime numbers
-}
-- LOCN 50 150 true
-- STYLE comment
{-
The sequence of prime numbers is:
-}
-- LOCN 50 375 true
-- STYLE comment
{-
. . . and define two specialised versions of the bar chart function
-}
-- LOCN 50 250 true
-- STYLE comment
{-
Take the first 100 prime numbers
-}
-- LOCN 50 500 true
-- STYLE comment
{-
The following chart plots the relation between the position of a prime number in
the sequence and the number itself:
-}
-- LOCN 50 650 true
-- STYLE comment
{-
This function computes the difference between adjacent numbers in a sequence of numbers
-}
-- LOCN 50 1000 true
-- STYLE comment
--
diffs :: [Int] -> [Int]
--
-- LOCN 75 1050 true
-- STYLE    
{-
The differences between adjacent prime numbers are:
-}
-- LOCN 50 1150 true
-- STYLE comment
{-
As the following chart suggests, "prime pairs" (ie, adjacent primes differing by 2) occur
more often would be expected if the primes were randomly distributed.
-}
-- LOCN 50 1300 true
-- STYLE comment
--
last primes100
--
-- LOCN 500 550 true
-- STYLE    
{-
These primes run between 2 and:
-}
-- LOCN 500 500 true
-- STYLE comment
--
import Histogram
--
-- LOCN 75 1725 true
-- STYLE   
--
hist = histogram primeDiffs
--
-- LOCN 75 1775 true
-- STYLE   
{-
The histogram function (imported from the module Histogram) can be used to
display the distribution of these differences
-}
-- LOCN 50 1675 true
-- STYLE comment
--
barChart2 "" hist
--
-- LOCN 76 1874 true
-- STYLE Picture
--
reducedPrimes = map ((flip div) 20) primes
--
-- LOCN 75 2175 true
-- STYLE   
--
h = histogram (take 300 reducedPrimes)
--
-- LOCN 75 2450 true
-- STYLE 
--
barChart1 "Histogram showing the distribution of prime numbers" h
--
-- LOCN 75 2600 true
-- STYLE Picture
{-
The distribution of primes (ie, the number of primes within each of a set of equi-spaced
intervals) can be computed by:
  - dividing (using integer division) each prime in the sequence by the interval size (eg, 20);
  - using the histogram function to determine the distribution of the resultant numbers.
-}
-- LOCN 50 2025 true
-- STYLE comment
{-
For example, using a "bin" size of 20, gives the following sequence:
-}
-- LOCN 50 2125 true
-- STYLE comment
{-
(ie, there are 8 primes in the interval 0 - 19 and 4 primes in the interval 20 - 39, and so on).
-}
-- LOCN 50 2275 true
-- STYLE comment
{-
Applying the histogram function (over the first 300 primes) yields the sequence:
-}
-- LOCN 50 2400 true
-- STYLE comment
{-
Displayed graphically the histogram is:
-}
-- LOCN 50 2550 true
-- STYLE comment
{-
Detailed inspection of the histogram shows that there are no primes in the 67-th interval (ie, the 
interval (1340 .. 1359).  

The relevant part of the sequence is:
-}
-- LOCN 50 2875 true
-- STYLE comment
--
filter (> 1300) primes
--
-- LOCN 75 2975 true
-- STYLE 
{-
Note: the following expressions may take 
       a few seconds to evaluate.
-}
-- LOCN 400 2325 true
-- STYLE warning2
--
barChart2 = barChart (plainBar red black) 200 75
--
-- LOCN 75 325 true
-- STYLE Hide
--
barChart1 = barChart (plainBar blue white) 800 150
--
-- LOCN 75 300 true
-- STYLE Hide
--
import SS
--
-- LOCN 725 75 true
-- STYLE 
{-
Back to previous document
-}
-- LOCN 50 3100 true
-- STYLE open ""
{-
Back to previous document
-}
-- LOCN 725 150 true
-- STYLE open ""
{-
Demonstrates use of the barchart functions (defined in Pic2) and the histogram function for
plotting the distribution of prime numbers.
-}
-- LOCN 50 -50 true
-- STYLE abstract
