-- Vital 2.1
{-
Pictures and transformations
-}
-- LOCN 50 50 true
-- STYLE title
--
import Pictures
--
-- LOCN 675 100 true
-- STYLE 
--
sq = Rect 40 40 yellow gray
--
-- LOCN 50 250 true
-- STYLE Picture
--
circ = Ellipse 30 30 cyan blue
--
-- LOCN 350 250 true
-- STYLE Picture
--
poly = let vs = [(10,30), (30,30), (30,10), (10,5), (20, 15)]
       in Poly vs lightGray red
--
-- LOCN 675 250 true
-- STYLE Picture
--
tile = let circ1 = move 5 5 circ in
       superimpose [sq, circ1, poly]
--
-- LOCN 50 975 true
-- STYLE Picture
--
tile' = invertColor tile
--
-- LOCN 225 1400 true
-- STYLE Picture
--
tile1 = flipV tile
--
-- LOCN 225 1625 true
-- STYLE Picture
--
tile2 = flipH tile
--
-- LOCN 475 1625 true
-- STYLE Picture
--
p1 = tile <|> tile'
--
-- LOCN 50 1875 true
-- STYLE Picture
--
p2 = tile <-> tile'
--
-- LOCN 300 1875 true
-- STYLE Picture
--
p3 = p1 <-> p2
--
-- LOCN 550 1875 true
-- STYLE Picture
--
p4 = flipH p3
--
-- LOCN 750 1875 true
-- STYLE Picture
--
4 <*|> p1
--
-- LOCN 50 2325 true
-- STYLE Picture
--
2 <*-> p2
--
-- LOCN 525 2325 true
-- STYLE Picture
--
beside [tile, tile', p2, p3]
--
-- LOCN 50 2725 true
-- STYLE Picture
--
let pics = repeat (p3 <|> p4)
in beside pics
--
-- LOCN 50 3325 true
-- STYLE Picture
--
above [tile, tile', p2, p3]
--
-- LOCN 500 2725 true
-- STYLE Picture
--
bigTile = scale 4.0 tile
--
-- LOCN 575 3800 true
-- STYLE Picture
--
beside [smallTiles, bigTile, smallTiles]
--
-- LOCN 475 4175 true
-- STYLE Picture
--
turn 45 tile
--
-- LOCN 300 4650 true
-- STYLE Picture
--
poly <+> sq
--
-- LOCN 50 625 true
-- STYLE Picture
--
shift   (polygon 9 100 green black)
--
-- LOCN 75 5775 true
-- STYLE Picture
--
sun = let { duke' = move (-25) (-30) duke;
            star  = stelGon 20 100 50 yellow red
      } in shift (star <+> duke')
--
-- LOCN 75 6525 true
-- STYLE Picture
--
import SS
--
-- LOCN 675 50 true
-- STYLE 
{-
One picture may be superimposed on another using the function
    (<+>) :: Pic -> Pic -> Pic
-}
-- LOCN 50 475 true
-- STYLE comment
{-
The colours in a picture may be inverted using the function
    invertColor :: Pic -> Pic
-}
-- LOCN 50 1250 true
-- STYLE comment
{-
A picture may be reflected about a horizontal or a vertical axis
using the functions
    flipH, flipV :: Pic -> Pic
-}
-- LOCN 50 1525 true
-- STYLE comment
--
tile
--
-- LOCN 50 1625 true
-- STYLE Picture
{-
Pictures may be juxtaposed either horizontally or vertically using the operators
    (<|>), (<->) :: Pic -> Pic -> Pic
-}
-- LOCN 50 1800 true
-- STYLE comment
{-
n copies of a picture may be juxtaposed horizontally or vertically using the operators
    (<*|>), (<*->) :: Int -> Pic -> Pic
-}
-- LOCN 50 2250 true
-- STYLE comment
{-
A list of pictures may be juxtaposed horizontally or vertically using the functions
    beside, above :: [Pic] -> Pic
-}
-- LOCN 50 2625 true
-- STYLE comment
{-
In Haskell, expressions are evaluated "lazily" (that is, their evaluation is deferred until
their values are actually needed for producing output).

In particular, this means that expressions representing pictures are evaluated and displayed lazily, 
and so may be of unbounded extent.
-}
-- LOCN 50 3125 true
-- STYLE comment
{-
Pictures may be scaled using the function
    scale :: Double -> Pic -> Pic
-}
-- LOCN 75 3725 true
-- STYLE comment
{-
Pictures may be rotated using the functions
    turn  :: Int -> Pic -> Pic    
    turn' :: Double -> Pic -> Pic  
(The argument of the first is specified in degrees, of the second, in radians.)
-}
-- LOCN 75 4475 true
-- STYLE comment
--
turn' (-pi / 4.0) tile
--
-- LOCN 525 4650 true
-- STYLE Picture
--
tile
--
-- LOCN 75 4650 true
-- STYLE Picture
{-
The function 
    shift :: Pic -> Pic
allows this problem to be avoided. It shifts a picture so that the top lefthand corner of 
its bounding box is at the local origin.
-}
-- LOCN 75 4975 true
-- STYLE comment
{-
The function
    polygon :: Int -> Int -> Color -> Color -> Pic

forms a regular n-gon with specified radius, and with specified
body and edge colours.
-}
-- LOCN 75 5650 true
-- STYLE comment
{-
The function
    stelGon :: Int -> Int -> Int -> Color -> Color -> Pic

forms a regular stellated n-gon with specified outer and inner radii,
and with specified body and edge colours.
-}
-- LOCN 525 5650 true
-- STYLE comment
--
shift  (stelGon 3 100 10 cyan blue)
--
-- LOCN 525 5775 true
-- STYLE Picture
{-
This function imports a bit-mapped image from a file with the specified name
    image :: String -> Pic
-}
-- LOCN 75 6175 true
-- STYLE comment
--
duke = image "duke.gif"
--
-- LOCN 75 6300 true
-- STYLE Picture
{-
The "duke" image is a registered
trademark of Sun MicroSystems Inc.
-}
-- LOCN 475 6300 true
-- STYLE commentB
{-
Composite images may be formed by superimposing simpler images.
-}
-- LOCN 75 6475 true
-- STYLE comment
{-
Back to previous document
-}
-- LOCN 75 6925 true
-- STYLE open1 ""
{-
Earlier, we introduced definitions for these three simple pictures:
-}
-- LOCN 50 175 true
-- STYLE comment
{-
For example:
-}
-- LOCN 50 550 true
-- STYLE comment
{-
A list variant of the same function
      superimpose :: [Pic] -> Pic

allows a whole list of pictures to be superimposed in one go
-}
-- LOCN 50 800 true
-- STYLE comment
--
let {
    circ1 = move (-15) 5 circ;
    circ2 = move 25 5 circ
} in circ1 <+> circ2 <+> sq
--
-- LOCN 400 625 true
-- STYLE Picture
{-
For example:
-}
-- LOCN 50 900 true
-- STYLE comment
{-
Exercise 2.1
-}
-- LOCN 50 1100 true
-- STYLE open2' "PicEx"
{-
Picture transformations
-}
-- LOCN 50 1200 true
-- STYLE heading1
{-
For example, taking the above "tile" picture and inverting its colours gives:
-}
-- LOCN 50 1325 true
-- STYLE comment
--
tile
--
-- LOCN 50 1400 true
-- STYLE Picture
{-
Exercise 2.2
-}
-- LOCN 50 2075 true
-- STYLE open2' "PicEx"
{-
Recap
-}
-- LOCN 50 125 true
-- STYLE heading1
{-
Superimposing pictures
-}
-- LOCN 50 400 true
-- STYLE heading1
{-
n-fold juxtaposition of pictures
-}
-- LOCN 50 2200 true
-- STYLE heading1
{-
Juxtaposition of lists of pictures
-}
-- LOCN 50 2575 true
-- STYLE heading1
{-
For example, here is an infinitely wide picture. (You can drag the document
sideways to see more of it . . . )
-}
-- LOCN 50 3250 true
-- STYLE comment
{-
Exercise 2.3
-}
-- LOCN 75 3525 true
-- STYLE open2' "PicEx"
{-
Changing the sizes of pictures
-}
-- LOCN 75 3650 true
-- STYLE heading1
--
tile
--
-- LOCN 75 3800 true
-- STYLE Picture
--
smallTile = scale 0.5 tile'
--
-- LOCN 250 3800 true
-- STYLE Picture
{-
Using the various functiond defined so far allows many
interesting compositions. For example:
-}
-- LOCN 75 4075 true
-- STYLE comment
--
smallTiles = 8 <*-> smallTile
--
-- LOCN 75 4175 true
-- STYLE Picture
{-
Rotations
-}
-- LOCN 75 4425 true
-- STYLE heading1
{-
For example:
-}
-- LOCN 75 4600 true
-- STYLE comment
{-
Notice that the rotation takes place about the local origin, in this case, the top
left-hand corner of the tile.  This means that pictures can often end up in awkward
positions, as for example here, where the expression is 
      turn 120 tile
-}
-- LOCN 75 4775 true
-- STYLE comment
--
turn 120 tile
--
-- LOCN 75 4925 true
-- STYLE Picture
{-
Here is the same expression as above, but now with the shifting function applied to it
-}
-- LOCN 75 5075 true
-- STYLE comment
--
let pic = turn 120 tile
in shift pic
--
-- LOCN 75 5125 true
-- STYLE Picture
{-
. . . and here is the same picture, with axes superimposed:
-}
-- LOCN 75 5275 true
-- STYLE comment
--
withAxes (let pic = turn 120 tile 
          in shift pic)
--
-- LOCN 75 5350 true
-- STYLE Picture
{-
Exercise 2.4
-}
-- LOCN 75 5500 true
-- STYLE open2 "PicEx_2_4"
{-
Polygons and stellated polygons
-}
-- LOCN 75 5575 true
-- STYLE heading1
{-
Exercise 2.5
-}
-- LOCN 75 6850 true
-- STYLE open2' "PicEx"
{-
Images
-}
-- LOCN 75 6100 true
-- STYLE heading1
{-
For example
-}
-- LOCN 75 6250 true
-- STYLE comment
--
import SSExercises
--
-- LOCN 675 75 true
-- STYLE 
{-
Introducing operations on pictures: superposition, translation, scaling, rotation.
Also illustrates construction of polygons.
-}
-- LOCN 50 -50 true
-- STYLE abstract
{-
Back to previous document
-}
-- LOCN 675 125 true
-- STYLE open1 ""
