-- Vital 2.1
{-
Higher-order functions
-}
-- LOCN 50 50 true
-- STYLE title
--
import SS
--
-- LOCN 625 50 true
-- STYLE 
--
import Pictures
--
-- LOCN 625 100 true
-- STYLE 
{-
One of the distinguishing features of Haskell is the way in which higher-order
functions are used to capture commonly-occurring computational idioms. The Haskell
prelude contains definitions for many such "combinators". 

For example, two such combinators are
    map :: (a -> b) -> [a] -> [b]

    cycle :: [a] -> [a]

The first takes a function and a list, and forms a new list by applying the function to every element of the list.
The second takes a list and repeats its endlessly.
-}
-- LOCN 50 575 true
-- STYLE comment
{-
For instance, if we define a list of pictures:
-}
-- LOCN 50 825 true
-- STYLE comment
--
sq   = Rect 40 40 yellow gray
--
-- LOCN 120 246 true
-- STYLE Picture
--
circ = move 5 5 (Ellipse 30 30 cyan blue)
--
-- LOCN 117 364 true
-- STYLE Picture
--
poly = Poly [(10,30), (30,30), (30,10), (10,5), (20, 15)] lightGray red
--
-- LOCN 499 246 true
-- STYLE Picture
--
tile = superimpose [poly, circ, sq]
--
-- LOCN 501 363 true
-- STYLE Picture
{-
Recap
-}
-- LOCN 50 150 true
-- STYLE heading1
{-
Earlier, we introduced some simple pictures for use as examples:
-}
-- LOCN 50 200 true
-- STYLE comment
{-
Library-defined combinators
-}
-- LOCN 50 525 true
-- STYLE heading1
--
pics =  cycle [sq, gap, circ, gap, poly, gap, tile, gap]
--
-- LOCN 50 900 true
-- STYLE Hide
--
beside pics
--
-- LOCN 50 950 true
-- STYLE Picture
--
gap =  Rect 10 10 trans trans
--
-- LOCN 700 900 true
-- STYLE Picture
--
pics :: [Pic]
--
-- LOCN 50 875 true
-- STYLE 
--
gap :: Pic
--
-- LOCN 700 875 true
-- STYLE 
{-
. . . and define a function that operates on pictures:
-}
-- LOCN 50 1075 true
-- STYLE comment
--
f =  flipH . invertColor . scale 0.75 . (\pic -> pic <-> turn 30 pic)
--
-- LOCN 50 1150 true
-- STYLE Hide
--
f :: Pic -> Pic
--
-- LOCN 50 1125 true
-- STYLE 
{-
. . . then we could use "map" to apply this function to every picture in the list:
-}
-- LOCN 50 1225 true
-- STYLE comment
--
pics' = map f pics
--
-- LOCN 50 1275 true
-- STYLE Hide
--
beside pics'
--
-- LOCN 50 1300 true
-- STYLE Picture
{-
Exercise 3.1
-}
-- LOCN 50 1450 true
-- STYLE open2' "PicEx"
{-
User-defined combinators
-}
-- LOCN 50 1575 true
-- STYLE heading1
--
map' :: [a -> b] -> a -> [b]
--
-- LOCN 50 1700 true
-- STYLE 
--
map' fs x = [f x | f <- fs]
--
-- LOCN 50 1725 true
-- STYLE Hide
--
pics2 = let fs = [ flipV
                 , flipH
                 , invertColor
                 , scale 0.6
                 , (4 <*->)
                 , turn 45 . invertColor . scale 2.0
                 , id
                 ]  
        in map' fs tile
--
-- LOCN 50 2025 true
-- STYLE hide
--
beside pics2
--
-- LOCN 50 2250 true
-- STYLE Picture
{-
Often it is useful to introduce user-defined combinators in solving problems. For example, here
is a definition of a variant of the usual "map" function. Can you see what it does?
-}
-- LOCN 50 1625 true
-- STYLE comment
{-
Since the type of this combinator is polymorphic, we can use it on values
of any type. For example, here is a use of this combinator with integers:
-}
-- LOCN 50 1800 true
-- STYLE comment
--
map'  [ (* 2), negate, (+12), mod 3]   7
--
-- LOCN 50 1875 true
-- STYLE 
{-
. . . and here is a use of the same combinator on the "tile" picture:
-}
-- LOCN 50 1975 true
-- STYLE comment
{-
Explanation:  
    *  fs is a list of functions, each of type Pic -> Pic)
    *  the map' function applies each of these to the "tile" picture
    *  it returns a list of pictures as its result
-}
-- LOCN 550 2025 true
-- STYLE comment
{-
Exercise 3.2
-}
-- LOCN 50 2550 true
-- STYLE open2' "PicEx"
{-
Back to previous document
-}
-- LOCN 50 2600 true
-- STYLE open1 ""
--
import SSExercises
--
-- LOCN 625 75 true
-- STYLE 
{-
Back to previous document
-}
-- LOCN 625 125 true
-- STYLE open1 ""
{-
Illustrates the way that higher-order functions can be used
to construct complex pictures.
-}
-- LOCN 50 -50 true
-- STYLE abstract
