-- Vital 2.1
{-
Displaying chess boards
-}
-- LOCN 50 125 true
-- STYLE title
--
import SS
--
-- LOCN 625 25 true
-- STYLE 
--
import Pictures
--
-- LOCN 625 75 true
-- STYLE 
{-
Suppose we need a function that generates pictures of chess boards. There are many ways in which
such a function could be written.
-}
-- LOCN 50 200 true
-- STYLE comment
{-
Here is one way:
-}
-- LOCN 50 275 true
-- STYLE comment
--
mkBoard :: Color -> Color -> Int -> Pic
--
-- LOCN 50 325 true
-- STYLE 
--
mkBoard c1 c2 ss =
  let { sq1 = Rect ss ss c1 trans;
        sq2 = Rect ss ss c2 trans;
        row = 4 <*|> (sq1 <|> sq2)
  } in  4 <*-> (row <-> flipV row)
--
-- LOCN 50 350 true
-- STYLE Hide
{-
. . . and here is a typical chess board produced by that function
-}
-- LOCN 525 300 true
-- STYLE comment
--
board = mkBoard yellow brown 40
--
-- LOCN 525 350 true
-- STYLE Picture
{-
Exercise 4.1
-}
-- LOCN 50 700 true
-- STYLE open2 "PicEx_4_1"
{-
Chess pieces
-}
-- LOCN 50 850 true
-- STYLE heading1
{-
The "Pictures" library contains definitions of pictures of some chess pieces:
-}
-- LOCN 50 900 true
-- STYLE comment
--
beside [pawn, pawn', knight, queen']
--
-- LOCN 50 975 true
-- STYLE Picture
{-
[The identifiers that are primed denote white pieces, 
  the unprimed ones, black pieces.]
-}
-- LOCN 525 975 true
-- STYLE comment
{-
Since these are ordinary pictures, the usual functions can be applied to them . . .
-}
-- LOCN 50 1150 true
-- STYLE comment
{-
Placing chess pieces on a board
-}
-- LOCN 50 1475 true
-- STYLE heading1
{-
Here is a function that places a given chess piece wrt an imaginary chess board
-}
-- LOCN 50 1525 true
-- STYLE comment
--
place :: Int -> (Pic, (Int,Int)) -> Pic
--
-- LOCN 50 1575 true
-- STYLE 
--
place ss picIJ  =
    let { pic = fst picIJ;
          sf   = (fromInt ss)/(fromInt 40);
          pic' = scale sf pic;
          ij  = snd picIJ;
          x = ss * fst ij;
          y = ss * snd ij
    } in move x y pic'
--
-- LOCN 50 1600 true
-- STYLE Hide
--
let knights = iterate (scale 1.2) knight
in beside knights
--
-- LOCN 50 1200 true
-- STYLE Picture
{-
Explanation:
  - pic  is the chess piece to be placed
  - sf  is the amount it needs to be scaled (so as to fit in a square of size ss x ss)
  - pic'  is the scaled chess piece
  - ij  is the coordinate pair where the chess piece is to be placed
  - x and y  are the offsets (in pixels) of where the chess piece is to be placed
-}
-- LOCN 525 1575 true
-- STYLE comment
{-
For example:
-}
-- LOCN 75 1800 true
-- STYLE comment
--
let pic = place 20 (knight, (4,2))
in withAxes pic
--
-- LOCN 425 1850 true
-- STYLE Picture
--
let pic = place 20 (knight, (7,7))
in withAxes pic
--
-- LOCN 800 1850 true
-- STYLE Picture
--
let pic = place 20 (knight, (0,0))
in withAxes pic
--
-- LOCN 75 1850 true
-- STYLE Picture
{-
. . . and here is a function that uses the above function to superimpose the chess piece
on a picture of a chess board
-}
-- LOCN 75 2100 true
-- STYLE comment
--
placeOnBoard :: Int -> (Pic, (Int,Int)) -> Pic
--
-- LOCN 75 2175 true
-- STYLE 
--
placeOnBoard c1 c2 ss picIJ = 
  let { pic1 = place ss picIJ;
        pic2 = mkBoard c1 c2 ss}
  in pic2 <+> pic1
--
-- LOCN 75 2200 true
-- STYLE Hide
{-
For example:
-}
-- LOCN 75 2325 true
-- STYLE comment
--
placeOnBoard yellow gray 40 (queen', (5,1))
--
-- LOCN 75 2375 true
-- STYLE Picture
{-
Exercise 4.2
-}
-- LOCN 75 2800 true
-- STYLE open2 "PicEx_4_2"
{-
Back to previous document
-}
-- LOCN 75 2875 true
-- STYLE open1 ""
--
import SSExercises
--
-- LOCN 625 50 true
-- STYLE 
{-
Back to previous document
-}
-- LOCN 625 100 true
-- STYLE open1 ""
{-
Illustrates how a chess board can be displayed pictorially.
-}
-- LOCN 50 -50 true
-- STYLE abstract
