-- Vital 2.1
--
import SS
--
-- LOCN 700 50 true
-- STYLE 
{-
Propositional Logic
-}
-- LOCN 50 75 true
-- STYLE title
{-
The library module PropLogicLib defines a representation for the formulae and
theorems of propositional logic.
-}
-- LOCN 50 150 true
-- STYLE comment
--
import PropLogicLib
--
-- LOCN 75 200 true
-- STYLE 
--
fm1 = Conj (Var "p") (Var "q")
--
-- LOCN 50 425 true
-- STYLE Hide
--
show fm1
--
-- LOCN 50 575 true
-- STYLE 
--
fm2 = pf "~ p | q"
--
-- LOCN 50 1150 true
-- STYLE Hide
{-
Formulae
-}
-- LOCN 50 300 true
-- STYLE heading1
{-
Theorems
-}
-- LOCN 50 1350 true
-- STYLE heading1
--
instance Edit Form where
  style fm = Text 1 (Format SansSerif 14 blue True False)
--
-- LOCN 50 750 true
-- STYLE 
--
thm1 = taut "h" fm
--
-- LOCN 50 1675 true
-- STYLE Hide
--
instance Edit Theorem where
  style thm = Text 1 (Format SansSerif 14 red True False)
--
-- LOCN 50 1975 true
-- STYLE 
--
thm1
--
-- LOCN 100 2100 true
-- STYLE 
--
fm2
--
-- LOCN 50 1200 true
-- STYLE 
--
thm2 = disch "h" thm1
--
-- LOCN 50 1825 true
-- STYLE Hide
{-
Commutativity of disjunction
-}
-- LOCN 50 2450 true
-- STYLE commentB
--
thm5 = let fm = pf "p | q"
       in taut "h" fm
--
-- LOCN 75 2825 true
-- STYLE 
--
thm6 = let {fm1 = pf "p";
            fm2 = pf "q";
            thm = taut "h1" fm1
       } in orIntroR thm fm2
--
-- LOCN 375 2825 true
-- STYLE 
--
thm7 = let {fm1 = pf "p";
            fm2 = pf "q";
            thm = taut "h2" fm2
       } in orIntroL thm fm1
--
-- LOCN 700 2825 true
-- STYLE 
--
thm8 = orElim thm5 thm6 thm7
--
-- LOCN 375 2975 true
-- STYLE 
--
thm9 = disch "h" thm8
--
-- LOCN 375 3050 true
-- STYLE 
{-
Back to previous document
-}
-- LOCN 700 100 true
-- STYLE open ""
{-
data Form = F
          | T
          | Var String
          | Neg Form
          | Conj Form Form
          | . . . 
-}
-- LOCN 700 350 true
-- STYLE commentM
{-
Formulae are represented as values of the "Form" datatype.
For example:
-}
-- LOCN 50 350 true
-- STYLE comment
{-
The "show" method (of the "Show" type class) displays formulae in infix notation:
-}
-- LOCN 50 525 true
-- STYLE comment
{-
The "pf" function
      pf :: String -> Form
parses a formula (or yields an error if it is unparsable)
-}
-- LOCN 50 1000 true
-- STYLE comment
{-
Convention:
  &        - conjunction
  |        - disjunction
  ~        - negation
  =>       - implication
  ==       - equivalence
-}
-- LOCN 700 675 true
-- STYLE commentM
{-
By defining a "Form" instance of the "Edit" type class, values of type Form
can be displayed in a distinctive style:
-}
-- LOCN 50 675 true
-- STYLE comment
{-
For example:
-}
-- LOCN 50 825 true
-- STYLE comment
--
fm1
--
-- LOCN 50 875 true
-- STYLE 
{-
For example:
-}
-- LOCN 50 1100 true
-- STYLE comment
{-
Theorems are represented as values of the "Theorem" ADT, and are constructed
using a set of functions corresponding to the inference rules of the logic (intuitionistic
propositional logic).
-}
-- LOCN 50 1400 true
-- STYLE comment
{-
data Theorem

taut     :: Name -> Form -> Theorem
disch    :: Name -> Theorem -> Theorem
mp       :: Theorem -> Theorem -> Theorem
andIntro :: Theorem -> Theorem -> Theorem
. . .
-}
-- LOCN 700 1400 true
-- STYLE commentM
{-
For example, the "taut" inference rule takes a name and a formula and forms
a theorem having that formula as a named hypothesis:
-}
-- LOCN 50 1525 true
-- STYLE comment
{-
By defining a "Theorem" instance of the "Edit" type class, values of type Theorem
can be displayed in a distinctive style:
-}
-- LOCN 50 1925 true
-- STYLE comment
--
fm = Var "p"
--
-- LOCN 55 1588 true
-- STYLE 
{-
. . . and the "disch" inference rule takes a theorem and discharges the
named hypothesis. For example:
-}
-- LOCN 50 1750 true
-- STYLE comment
{-
For example:
-}
-- LOCN 50 2050 true
-- STYLE comment
--
thm2
--
-- LOCN 325 2100 true
-- STYLE 
--
let {fm = pf "F";
     thm = taut "h" fm
} in notIntro "h" thm
--
-- LOCN 100 2200 true
-- STYLE 
{-
   h     p
------------ taut
  {h} |- p
-}
-- LOCN 700 1650 true
-- STYLE rule
{-
Define the "rule" style used for the inference rule boxes
-}
-- LOCN 525 3275 true
-- STYLE comment
--
rule = let {fmt = Format Monospaced 14 blue False False;
            lightBlue = RGB 200 200 255
       } in Comment lightBlue gray fmt Nothing
--
-- LOCN 525 3300 true
-- STYLE Hide
{-
  {h: p} |- q
--------------- disch
    p => q
-}
-- LOCN 700 1825 true
-- STYLE rule
{-
  {p} |- F
------------ notIntro
    ~ p
-}
-- LOCN 700 2225 true
-- STYLE rule
{-
Example
-}
-- LOCN 50 2375 true
-- STYLE heading1
{-
With patience, larger proofs can be constructed. For example, a proof along the
lines of:
-}
-- LOCN 50 2500 true
-- STYLE comment
{-
                                       -------- taut                     -------- taut
                                        p |- p                            q |- q
    ------------------ taut         ------------- orIntroR            ------------- orIntroL
    (p | q) |- (p | q)               p |- (q | p)                      q |- (q | p)
-------------------------------------------------------------------------------------------- orElim 
                                  (p | q) |- (q | p)
                                ----------------------- disch
                                 |- (p | q) => (q | p)
-}
-- LOCN 50 2575 true
-- STYLE rule
{-
. . . can be undertaken in stages, following the same pattern:
-}
-- LOCN 50 2775 true
-- STYLE comment
{-
Back to previous document
-}
-- LOCN 50 3150 true
-- STYLE open1 ""
{-
Illustrates the representation of formula and proof rules for
a propositional logic. Includes some simple forward proofs.
-}
-- LOCN 50 -50 true
-- STYLE abstract
