-- Vital 2.1
--
import SS
--
-- LOCN 775 75 true
-- STYLE 
{-
Propositional logic
-}
-- LOCN 50 75 true
-- STYLE title
{-
Back to previous document
-}
-- LOCN 775 100 true
-- STYLE open ""
{-
The library module PropLogicProofLib defines a language in which IPL proofs may
be expressed and provides a parser and an interpreter for the language.
-}
-- LOCN 50 275 true
-- STYLE comment
--
import PropLogicLib
--
-- LOCN 50 350 true
-- STYLE 
--
import PropLogicProofLib
--
-- LOCN 50 375 true
-- STYLE 
--
s3 = "    ImpIntro h: p|q.                  \
     \      OrElim h of                     \
     \        Left  h1 -> OrIntroR h1 q     \
     \        Right h2 -> OrIntroL h2 p     "
--
-- LOCN 50 2200 true
-- STYLE Hide
--
inf (pp s3)
--
-- LOCN 50 2325 true
-- STYLE 
--
s4 = "    ImpIntro h: p | (q & r).                           \
     \       AndIntro                                        \
     \          (OrElim h of                                 \
     \               Left  h2 -> OrIntroL h2 q               \
     \               Right h2 -> OrIntroR (AndElimL h2) p)   \
     \          (OrElim h of                                 \
     \               Left  h2 -> OrIntroL h2 r               \
     \               Right h2 -> OrIntroR (AndElimR h2) p)   "
--
-- LOCN 50 2675 true
-- STYLE Hide
--
inf (pp s4)
--
-- LOCN 50 2850 true
-- STYLE 
--
prf1 = pp "ImpIntro h:p & q. AndIntro (AndElimR h) (AndElimL h)"
--
-- LOCN 50 1025 true
-- STYLE 
--
s2 = "ImpIntro h1: p. ImpIntro h2: ~p. NotElim h1 h2"
--
-- LOCN 50 1900 true
-- STYLE Hide
{-
In practice, it is awkward to carry out theorem-proving working directly in terms of the
functions of the "Theorem" ADT. An alternative approach (which is particularly well suited for
"forward" theorem proving") is to introduce a concrete notation in which natural-deduction
proofs may be expressed.
-}
-- LOCN 50 150 true
-- STYLE comment
{-
The parsing function
    pp :: String -> Proof
translates a string expressing a proof into a value of type Proof that represents
the abstract syntax of a (putative) proof.
-}
-- LOCN 50 425 true
-- STYLE comment
{-
For example, here is a string representing a proof of the theorem
    |- (p & q) => (q & p)
-}
-- LOCN 50 700 true
-- STYLE comment
--
s = "ImpIntro h:p & q. AndIntro (AndElimR h) (AndElimL h)"
--
-- LOCN 50 775 true
-- STYLE Hide
{-
. . . and here is the corresponding abstract syntax the parser yields:
-}
-- LOCN 50 975 true
-- STYLE comment
{-
data Proof = Axiom String
           | AndIntro Proof Proof
           | AndElimL Proof
           | . . .
-}
-- LOCN 700 425 true
-- STYLE commentM
{-
(Normally, one would not wish to view the abstract syntax.)
-}
-- LOCN 50 1100 true
-- STYLE comment
{-
The proof interpreting function
     inf :: Proof -> Either String Theorem
takes a proof and yields either a theorem or an error message.
-}
-- LOCN 50 1175 true
-- STYLE comment
--
inf prf1
--
-- LOCN 50 1250 true
-- STYLE 
--
s' = "ImpIntro h:p & q. AndIntro (AndElimR k) (AndElimL h)"
--
-- LOCN 50 1475 true
-- STYLE Hide
--
inf (pp s')
--
-- LOCN 50 1575 true
-- STYLE 
{-
Badly-formed proofs
-}
-- LOCN 50 1375 true
-- STYLE heading2
{-
Here is an erroneous version of the above proof:
-}
-- LOCN 50 1425 true
-- STYLE comment
{-
. . . and here is the corresponding result:
-}
-- LOCN 50 1525 true
-- STYLE comment
{-
Some further proofs
-}
-- LOCN 50 1725 true
-- STYLE heading1
--
inf (pp s2)
--
-- LOCN 50 1950 true
-- STYLE 
{-
This proof uses the "Not Elim" rule
-}
-- LOCN 50 1850 true
-- STYLE comment
{-
 |- p      |- ~p
----------------- NotElim
      |- F
-}
-- LOCN 650 1850 true
-- STYLE rule
{-
Commutativity of disjunction
-}
-- LOCN 50 2150 true
-- STYLE heading2
{-
|-  p | q        {p} |- r        {q} |- r
------------------------------------------ OrElim
                     |- r
-}
-- LOCN 650 2300 true
-- STYLE rule
{-
    |- p
------------- OrIntroL
  |- p | q
-}
-- LOCN 650 2200 true
-- STYLE rule
{-
    |- q
------------- OrIntroR
  |- p | q
-}
-- LOCN 900 2200 true
-- STYLE rule
--
rule = let {fmt = Format Monospaced 14 blue False False;
            lightBlue = RGB 200 200 255
       } in Comment lightBlue gray fmt Nothing
--
-- LOCN 600 3125 true
-- STYLE Hide
{-
(Define the "rule" style used for the inference rule boxes)
-}
-- LOCN 600 3075 true
-- STYLE comment
{-
Disjunction distributes over conjunction
-}
-- LOCN 50 2625 true
-- STYLE heading2
{-
Contradiction
-}
-- LOCN 50 1800 true
-- STYLE heading2
{-
The following proof is like to take some seconds to evaluate:
please be patient!
-}
-- LOCN 100 2450 true
-- STYLE warning2
{-
Commutativity of conjunction
-}
-- LOCN 50 650 true
-- STYLE heading2
{-
  |- p        |- q
-------------------- AndIntro
     |- p & q
-}
-- LOCN 650 775 true
-- STYLE rule
{-
 |- p & q
----------- AndElimL
  |- p
-}
-- LOCN 650 675 true
-- STYLE rule
{-
 |- p & q
----------- AndElimR
  |- q
-}
-- LOCN 875 675 true
-- STYLE rule
{-
 { ..., h: p, ...} |- q
------------------------ ImpIntro
 { ..., ...} |- p => q
-}
-- LOCN 650 875 true
-- STYLE rule
{-
Back to previous document
-}
-- LOCN 50 3000 true
-- STYLE open1 ""
{-
Example
-}
-- LOCN 50 575 true
-- STYLE heading1
{-
(Aside:  most of the time to evaluate this expression is taken up in
 parsing the input string; a much more efficient parser is needed . . .)
-}
-- LOCN 600 2850 true
-- STYLE comment
{-
Introduces a simple language to describe proofs and, using it, proves
some simple theorems, including the distributivity of disjunction over conjunction.
-}
-- LOCN 50 -50 true
-- STYLE abstract
--
instance Edit Theorem where
  style thm = Text 1 (Format SansSerif 14 red True False)
--
-- LOCN 700 325 true
-- STYLE 
{-
Define a style for displaying values
of type Theorem
-}
-- LOCN 700 275 true
-- STYLE comment
