-- Vital 2.1
{-
Direct Manipulation of Sets and Binary Relations
-}
-- LOCN 50 50 true
-- STYLE title
--
import SS
--
-- LOCN 775 125 true
-- STYLE 
{-
The cardinality of the base set {A, B, ... }  (of which all sets are a subset)
-}
-- LOCN 50 125 true
-- STYLE comment
--
n = 8
--
-- LOCN 50 150 true
-- STYLE Hide
{-
Representation for a set as a list of N n
-}
-- LOCN 50 200 true
-- STYLE comment
--
data Set = Set [Int]
--
-- LOCN 50 250 true
-- STYLE 
--
egSet1 = Set [0, 3, 4, 7, 12, 14]
--
-- LOCN 375 250 true
-- STYLE Hide
{-
Display of a set. 
"gap" is the distance between adjacent elements
-}
-- LOCN 50 325 true
-- STYLE comment
--
gap = 50
--
-- LOCN 50 375 true
-- STYLE Hide
--
twoPi = 2.0 * pi
--
-- LOCN 250 375 true
-- STYLE Hide
--
location :: Double -> Int -> (Int, Int)
--
-- LOCN 50 850 true
-- STYLE 
{-
locn i  yields the location of the i-th element of the set
-}
-- LOCN 50 475 true
-- STYLE comment
--
radius = 
  let circum = fromInt (n * gap)
  in circum / twoPi
--
-- LOCN 475 375 true
-- STYLE 
--
location r i =
  let {theta = (fromInt i * twoPi / fromInt n);
       x = round (r * cos theta);
       y = round (r * sin theta)
   } in (x, y)
--
-- LOCN 50 875 true
-- STYLE Hide
--
locn 0
--
-- LOCN 525 525 true
-- STYLE 
--
locn 1
--
-- LOCN 650 525 true
-- STYLE 
--
locn (n-1)
--
-- LOCN 775 525 true
-- STYLE 
--
displayElem :: (Int, Int) -> Bool -> Bool -> Char -> Pic
--
-- LOCN 50 1150 true
-- STYLE 
{-
displayElem (x, y) visible ch
displays an selectable box labelled with ch at the point (x,y). If "visible" is false,
it displays a greyed-out (but still selectable) box. If "marked" is true, then it
marks that element.
-}
-- LOCN 50 1050 true
-- STYLE comment
--
displayElem xy visible marked ch =
  let {x = fst xy; y = snd xy;
       bc = if visible then bodyColor1  else bodyColor2;
       ec = if visible then edgeColor1  else edgeColor2;
       labelColor = if marked then labelColor0 else
                    if visible then labelColor1 else labelColor2;
       box = SelRect side side bc ec;
       box' = Trans (x - hSide) (y - hSide) box;
       fmt = Format SansSerif 16 labelColor marked False;
       text = PicText fmt [ch];
       text' = Trans (x + labelOffsetX) (y + labelOffsetY) text
    } in Super box' text'
--
-- LOCN 50 1175 true
-- STYLE Hide
--
hSide = 10
--
-- LOCN 825 1200 true
-- STYLE Hide
--
side = 2 * hSide
--
-- LOCN 825 1225 true
-- STYLE Hide
--
labelColor1 = blue
--
-- LOCN 600 1200 true
-- STYLE Hide
--
labelColor2 = gray
--
-- LOCN 600 1225 true
-- STYLE Hide
--
edgeColor1 = black
--
-- LOCN 600 1250 true
-- STYLE Hide
--
edgeColor2 = gray
--
-- LOCN 600 1275 true
-- STYLE Hide
--
bodyColor1 = yellow
--
-- LOCN 600 1300 true
-- STYLE Hide
--
bodyColor2 = trans
--
-- LOCN 600 1325 true
-- STYLE Hide
--
axes = Super
         (Trans (-10) 0 (Line 110 0 green))
         (Trans 0 (-10) (Line 0 110 green))
--
-- LOCN 825 1375 true
-- STYLE Picture
--
Super pic axes
--
-- LOCN 600 1400 true
-- STYLE Picture
--
displayElem (100, 10) False False 'M'
--
-- LOCN 50 1475 true
-- STYLE Picture
--
labelOffsetX = -5
--
-- LOCN 825 1275 true
-- STYLE Hide
--
labelOffsetY = 5
--
-- LOCN 825 1300 true
-- STYLE Hide
--
intToLetter :: Int -> Char
--
-- LOCN 50 1800 true
-- STYLE 
--
intToLetter i = chr (ord 'A' + i)
--
-- LOCN 50 1825 true
-- STYLE Hide
--
intToLetter 3
--
-- LOCN 550 1800 true
-- STYLE 
{-
Generate label character from integer position (0, 1, ...)
-}
-- LOCN 50 1750 true
-- STYLE comment
{-
Display a set.
If "maybe" contains an integer, then mark that element
-}
-- LOCN 50 2100 true
-- STYLE comment
--
displaySetElems :: Maybe Int -> Set -> Pic
--
-- LOCN 50 2150 true
-- STYLE 
--
displaySetElems maybeMarked set =
  case set of 
    Set indices ->
      super [let {xt = locn i;
                  present = elem i indices;
                  marked = case maybeMarked of {
                             Nothing -> False;
                             Just j  -> (i == j)
                           }
                  } in displayElem (locn i) (elem i indices) marked (intToLetter i)
            | i <- intLabels]
--
-- LOCN 50 2175 true
-- STYLE Hide
--
intLabels = [0 .. (n-1)]
--
-- LOCN 225 150 true
-- STYLE Hide
--
offset $ displaySetElems Nothing egSet1
--
-- LOCN 50 2550 true
-- STYLE Picture
--
super pics =
  case pics of {
    []       -> NoPic;
    pic:pics -> Super pic (super pics)
  }
--
-- LOCN 50 2425 true
-- STYLE Hide
--
super :: [Pic] -> Pic
--
-- LOCN 50 2400 true
-- STYLE 
--
pic = displayElem (0,0) True False 'M'
--
-- LOCN 50 1400 true
-- STYLE Picture
--
displaySet :: (Maybe Int) Set -> Pic
--
-- LOCN 51 3075 true
-- STYLE 
--
displaySet maybeMarked s =
  let {pic1 = displaySetElems maybeMarked s;
       pic2 = setBackground
  } in offset (pic1 `Super` pic2)
--
-- LOCN 50 3100 true
-- STYLE Hide
--
setBackground :: Pic
--
-- LOCN 50 2875 true
-- STYLE 
--
offset :: Pic -> Pic
--
-- LOCN 525 2400 true
-- STYLE 
--
offset pic =
  let {y = round radius + 20;
       x = 2 * y
  } in Trans x y pic
--
-- LOCN 525 2425 true
-- STYLE Hide
--
setBackground =
  let {r = round (1.2 * radius) + side;
       diam = 2 * r;
       box = SelRect diam diam trans black
  } in Trans (- r) (- r) box
      
--
-- LOCN 50 2900 true
-- STYLE Hide
--
instance Edit Set where {
  style s = View (displaySet Nothing) Picture;

  edit val path = 
    let {setCommands  = [("Show path", pathFn),
                         ("Show item", itemFn),
                         ("Complement set", complementFn),
                         ("Evaluate set", evaluateFn),
                         ("Copy set to clipboard", copyFn),
                         ("Intersect with clipboard", interFn),
                         ("Union with clipboard", unionFn)];
         elemCommands = [("Show path", pathFn),
                         ("Show item", itemFn),
                         ("Delete element", deleteFn),
                         ("Insert element", insertFn)]
    } in case decode path of {
           Nothing -> setCommands;
           Just _  -> elemCommands
         }
}
--
-- LOCN 50 3400 true
-- STYLE 
--
egSet1
--
-- LOCN 50 3775 true
-- STYLE 
{-
Define Edit methods for Set
-}
-- LOCN 50 3375 true
-- STYLE comment
--
pathFn val path x y lhs rhs clip =
  (Nothing, Nothing, Just (show path))
--
-- LOCN 50 4525 true
-- STYLE Hide
{-
Typical paths:
  Background        1
  A                 0 0 0
  B                 0 1 0 0
  C                 0 1 1 0 0
  D                 0 1 1 1 0 0
  F                 0 1 1 1 1 0 0
  :                 :
  O                 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 0 0
-}
-- LOCN 550 4100 true
-- STYLE commentM
{-
Function to "decode" the path
-}
-- LOCN 50 4200 true
-- STYLE comment
--
decode :: Path -> Maybe Int
--
-- LOCN 50 4225 true
-- STYLE 
--
decode path =
  if head path == 1 then 
    Nothing                    -- ie, a background click
  else
    let n = length path - 3
    in Just n                  -- ie, the n-th element

--
-- LOCN 50 4250 true
-- STYLE Hide
--
decode [1]
--
-- LOCN 625 4300 true
-- STYLE 
--
decode [0, 0, 0]
--
-- LOCN 625 4350 true
-- STYLE 
--
decode [0, 1, 1, 0, 0]
--
-- LOCN 800 4350 true
-- STYLE 
--
itemFn val path x y lhs rhs clip =
  (Nothing, Nothing, Just (describeItem path))
--
-- LOCN 50 4700 true
-- STYLE Hide
--
describeItem :: Path -> String
--
-- LOCN 675 4575 true
-- STYLE 
--
describeItem path =
  case decode path of {
    Nothing -> "Set";
    Just i  -> [intToLetter i]
  }
--
-- LOCN 675 4600 true
-- STYLE Hide
--
describeItem [1]
--
-- LOCN 675 4700 true
-- STYLE 
--
describeItem [0, 1, 1, 0, 0]
--
-- LOCN 675 4775 true
-- STYLE 
{-
Operations on sets
-}
-- LOCN 50 5450 true
-- STYLE comment
--
setComplement :: Set -> Set
--
-- LOCN 50 5475 true
-- STYLE 
--
setComplement s =
  case s of 
    Set elems -> Set [i | i <- intLabels, notElem i elems]
--
-- LOCN 50 5500 true
-- STYLE Hide
--
setComplement egSet1
--
-- LOCN 600 5475 true
-- STYLE 
--
complementFn val path x y lhs rhs clip =
  (Just ("setComplement (" ++ rhs ++ ")"), Nothing, Just "OK")
--
-- LOCN 50 4625 true
-- STYLE Hide
--
insertFn val path x y lhs rhs clip =
  let {i = decodeElem path;
       rhs' = "insert " ++ show i ++ " (" ++ rhs ++ ")"
  } in (Just rhs', Nothing, Just "OK")
--
-- LOCN 50 4775 true
-- STYLE Hide
--
evaluateFn val path x y lhs rhs clip =
  (Just (showSet val), Nothing, Just "OK")
--
-- LOCN 50 4575 true
-- STYLE Hide
{-
Show a set
-}
-- LOCN 50 1900 true
-- STYLE comment
--
showSet :: Set -> String
--
-- LOCN 50 1950 true
-- STYLE 
--
showSet s =
  case s of
    Set  indices  -> let indices' = filter (< n) indices
                        in "Set "  ++ show indices'
--
-- LOCN 50 1975 true
-- STYLE Hide
--
showSet egSet1
--
-- LOCN 550 1950 true
-- STYLE 
--
showSet egSet1
--
-- LOCN 50 4000 true
-- STYLE 
{-
Definitions of command functions
-}
-- LOCN 50 4475 true
-- STYLE comment
--
delete, insert :: Int -> Set -> Set
--
-- LOCN 50 5775 true
-- STYLE 
--
delete i s =
  case s of 
    Set ms -> let ms' = filter (/= i) ms
              in Set ms'
    
--
-- LOCN 50 5800 true
-- STYLE Hide
--
delete 0 egSet1
--
-- LOCN 600 5775 true
-- STYLE 
--
egSet1
--
-- LOCN 600 5250 true
-- STYLE 
--
insert i s =
  case s of
    Set ms -> Set (i : (filter (/= i) ms))
--
-- LOCN 75 6075 true
-- STYLE Hide
--
nub :: [a] -> [a]
--
-- LOCN 100 6450 true
-- STYLE 
--
nub xs =
  case xs of {
    []     -> [];
    x:xs   -> let xs' = filter (x /=) xs
              in x : nub xs'
  }
--
-- LOCN 100 6475 true
-- STYLE Hide
--
insert 2 egSet1
--
-- LOCN 625 6050 true
-- STYLE 
--
inter, union :: Set -> Set -> Set
--
-- LOCN 100 6325 true
-- STYLE 
--
inter s1 s2 =
  case (s1, s2) of
    (Set ms1, Set ms2) -> 
      let ms = filter (\m -> elem m ms1) ms2
      in Set ms
--
-- LOCN 100 6350 true
-- STYLE Hide
--
egSet2 = Set [0, 1, 3, 6]
--
-- LOCN 400 3775 true
-- STYLE 
--
egSet1 `inter` egSet2
--
-- LOCN 625 6350 true
-- STYLE 
--
union s1 s2 =
  case (s1, s2) of
    (Set ms1, Set ms2) ->
      let ms = filter (\m -> not (elem m ms1)) ms2
      in Set (ms ++ ms1)
--
-- LOCN 100 6650 true
-- STYLE Hide
--
egSet1 `union` egSet2
--
-- LOCN 650 6650 true
-- STYLE 
--
decodeElem :: Path -> Int
--
-- LOCN 50 4350 true
-- STYLE 
--
decodeElem path =
  case decode path of {
    Nothing  -> error "RelDemo.decodeElem: should not occur";
    Just i   -> i
  }
--
-- LOCN 50 4375 true
-- STYLE Hide
--
decodeElem [0, 1, 1, 0, 0]
--
-- LOCN 625 4425 true
-- STYLE 
--
deleteFn val path x y lhs rhs clip =
  let {i = decodeElem path;
       rhs' = "delete " ++ show i ++ " (" ++ rhs ++ ")"
  } in (Just rhs', Nothing, Just "OK")
--
-- LOCN 50 4875 true
-- STYLE Hide
--
egSet1
--
-- LOCN 650 4875 true
-- STYLE 
--
interFn val path x y lhs rhs clip =
  let rhs' = "(" ++ clip ++ ") `inter` (" ++ rhs ++ ")"
  in (Just rhs', Nothing, Just "OK")
--
-- LOCN 50 4975 true
-- STYLE Hide
--
copyFn val path x y lhs rhs clip =
  (Nothing, Just rhs, Just "OK")
--
-- LOCN 50 5075 true
-- STYLE Hide
--
showSet egSet2
--
-- LOCN 425 4000 true
-- STYLE 
--
unionFn val path x y lhs rhs clip =
  let rhs' = "(" ++ clip ++ ") `union` (" ++ rhs ++ ")"
  in (Just rhs', Nothing, Just "OK")
--
-- LOCN 50 5150 true
-- STYLE Hide
{-
RELATIONS
-}
-- LOCN 50 6950 true
-- STYLE heading1
--
data Rel = Rel [(Int, Int)] (Maybe Int)
--
-- LOCN 50 7050 true
-- STYLE 
{-
A relation is represented by a list of its edges (as ordered pairs) and
(optionally) a "marked" element
-}
-- LOCN 50 7000 true
-- STYLE comment
--
egRel = Rel edges (Just 2)
--
-- LOCN 575 7100 true
-- STYLE Hide
--
displayElem (100, 10) True True 'M'
--
-- LOCN 50 1550 true
-- STYLE Picture
--
labelColor0 = red
--
-- LOCN 600 1175 true
-- STYLE Hide
--
displayElem (100, 10) False True 'M'
--
-- LOCN 50 1625 true
-- STYLE Picture
--
offset $ displaySetElems (Just 3) egSet1
--
-- LOCN 475 2550 true
-- STYLE Picture
{-
Show a relation
-}
-- LOCN 50 7100 true
-- STYLE comment
--
showRel rel =
  case rel of {
    Rel edges Nothing ->
      "Rel " ++ show edges ++ " Nothing";
    Rel edges (Just i) ->
      "Rel " ++ show edges ++ " (Just " ++ show i ++ ")"
  }
--
-- LOCN 50 7150 true
-- STYLE Hide
--
showRel :: Rel -> String
--
-- LOCN 50 7125 true
-- STYLE 
--
showRel egRel
--
-- LOCN 575 7150 true
-- STYLE 
{-
Display a list of edges (of a relation)
-}
-- LOCN 50 7275 true
-- STYLE comment
--
displayEdges :: [(Int, Int)] -> Pic
--
-- LOCN 50 7600 true
-- STYLE 
--
displayEdges edges =
  super (map displayEdge edges)

--
-- LOCN 50 7625 true
-- STYLE Hide
--
displayEdge :: (Int, Int) -> Pic
--
-- LOCN 50 7350 true
-- STYLE 
--
edgeColor = blue
--
-- LOCN 50 8000 true
-- STYLE Hide
--
displayEdge (0,1)
--
-- LOCN 625 7425 true
-- STYLE Picture
--
edges = [(0,2), (1,2), (3, 3), (4, 1), (1, 6)]
--
-- LOCN 575 7075 true
-- STYLE Hide
--
displayEdges edges
--
-- LOCN 625 7575 true
-- STYLE Picture
{-
Display a relation
-}
-- LOCN 50 7700 true
-- STYLE comment
--
displayRel :: Rel -> Pic
--
-- LOCN 50 7750 true
-- STYLE 
--
displayRel rel =
  case rel of 
    Rel edges maybeMarked ->
      let {pic1 = displayEdges edges;
           pic2 = displaySetElems maybeMarked baseSet;
           pic3 = setBackground
      } in offset (super [pic1, pic2, pic3])
--
-- LOCN 50 7775 true
-- STYLE Hide
--
baseSet = Set intLabels
--
-- LOCN 575 7750 true
-- STYLE Hide
--
displayRel egRel
--
-- LOCN 575 7775 true
-- STYLE Picture
--
vector :: Int -> Int -> Int -> Int -> Pic
--
-- LOCN 50 8050 true
-- STYLE 
--
vector x1 y1 x2 y2 =
  let {dx = x2 - x1;
       dy = y2 - y1;
       s  = (abs dx + abs dy + abs (dx + dy) + abs (dx - dy)) `div` 2;    -- Approx length
       ux = (k * dx) `div` s;  -- u  is a short vector along the given vector
       uy = (k * dy) `div` s;
       vx = -uy `div` 2;       -- v  is a short vector perpendicular to the given vector
       vy =  ux `div` 2;
       xm = (x1 + x2) `div` 2;  ym = (y1 + y2) `div` 2;
       x4 = xm - ux + vx;  y4 = ym - uy + vy;
       x5 = xm - ux - vx;  y5 = ym - uy - vy;
       arrow1 = line xm ym x4 y4;
       arrow2 = line xm ym x5 y5;
       shaft  = line x1 y1 x2 y2
    } in super [arrow1, arrow2, shaft]
--
-- LOCN 50 8075 true
-- STYLE Hide
--
k = 15
--
-- LOCN 300 8000 true
-- STYLE Hide
--
line x1 y1 x2 y2 =
  Trans x1 y1 (Line (x2 - x1) (y2 - y1) edgeColor)
--
-- LOCN 50 7925 true
-- STYLE Hide
--
vector 0 0 100 100
--
-- LOCN 950 8150 true
-- STYLE Picture
--
vector 0 0 20 20
--
-- LOCN 950 8075 true
-- STYLE Picture
--
vector 0 0 100 0
--
-- LOCN 775 8075 true
-- STYLE Picture
--
vector 0 0 0 100
--
-- LOCN 775 8150 true
-- STYLE Picture
{-
Declaration of Edit instance for Rel
-}
-- LOCN 50 8350 true
-- STYLE comment
--
instance Edit Rel where {
  style r = View displayRel Picture;

  edit val path = 
    let {relCommands = 
           [("Show path",         relPathFn),
            ("Unmark",            relUnmarkFn),
            ("Invert",            relInverseFn),
            ("Reflexive closure", relReflexClosureFn),
            ("Symmetric closure", relSymmClosureFn),
            ("Evaluate",          relEvaluateFn)];
         relElemCommands =  
           [("Show path",         relPathFn),
            ("Mark element",      relMarkFn),
            ("Add edge",          relAddEdgeFn),
            ("Delete edge",       relDeleteEdgeFn)]
    } in case decodeRelPath path of {
           Nothing -> relCommands;
           Just _  -> relElemCommands
         }
}
--
-- LOCN 50 8400 true
-- STYLE 
--
egRel
--
-- LOCN 600 8400 true
-- STYLE 
{-
Definition of command functions for Rel
-}
-- LOCN 50 9100 true
-- STYLE comment
--
relPathFn val path x y lhs rhs clip =
  (Nothing, Nothing, Just (show path))
--
-- LOCN 50 9125 true
-- STYLE Hide
{-
Typical paths:
  Background         1 1 0
  A                  1 0 0 0
  B                  1 0 1 0 0
  C                  1 0 1 1 0 0
-}
-- LOCN 600 8650 true
-- STYLE commentM
{-
Functions to decode the path
-}
-- LOCN 50 8750 true
-- STYLE comment
--
decodeRelPath :: Path -> Maybe Int
--
-- LOCN 50 8800 true
-- STYLE 
--
decodeRelPath path =
  let path' = tail path 
  in if head path' == 1 then
       Nothing
     else
       let n = length path' - 3
       in Just n
--
-- LOCN 50 8825 true
-- STYLE Hide
--
decodeRelElemPath path = 
  case decodeRelPath path of {
    Nothing -> error "RelDemo.decodeRelElemPath: should not occur";
    Just i  -> i
  }
--
-- LOCN 50 9000 true
-- STYLE Hide
--
decodeRelElemPath :: Path -> Int
--
-- LOCN 50 8975 true
-- STYLE 
--
relMarkFn val path x y lhs rhs clip =
  let {i = decodeRelElemPath path;
       rhs' = "markRel " ++ show i ++ " (" ++ rhs ++ ")"
  } in (Just rhs', Nothing, Just "OK")
--
-- LOCN 50 9200 true
-- STYLE Hide
{-
Functions on relations
-}
-- LOCN 50 10225 true
-- STYLE heading2
--
markRel :: Int -> Rel -> Rel
--
-- LOCN 50 10275 true
-- STYLE 
--
markRel i rel =
  case rel of
    Rel edges _ -> Rel edges (Just i)
--
-- LOCN 50 10300 true
-- STYLE Hide
--
markRel 7 egRel
--
-- LOCN 625 10225 true
-- STYLE 
--
unmarkRel :: Rel -> Rel
--
-- LOCN 50 10400 true
-- STYLE 
--
unmarkRel rel =
  case rel of
    Rel edges _ -> Rel edges Nothing
--
-- LOCN 50 10425 true
-- STYLE Hide
--
relUnmarkFn val path x y lhs rhs clip =
  let rhs' = "unmarkRel (" ++ rhs ++ ")"
  in (Just rhs', Nothing, Just "OK")
--
-- LOCN 50 9300 true
-- STYLE Hide
--
addEdgeRel :: Int -> Int -> Rel -> Rel
--
-- LOCN 50 10525 true
-- STYLE 
--
addEdgeRel i j rel =
  case rel of
    Rel edges mm -> Rel ((i,j):edges) mm
--
-- LOCN 50 10550 true
-- STYLE Hide
--
deleteEdgeRel 0 2 egRel
--
-- LOCN 650 10675 true
-- STYLE 
--
isMarkedRel :: Rel -> Bool
--
-- LOCN 425 10275 true
-- STYLE 
--
isMarkedRel rel =
  case rel of {
    Rel _ Nothing -> False;
    Rel _ _       -> True
  }
--
-- LOCN 425 10300 true
-- STYLE Hide
--
relAddEdgeFn val path x y lhs rhs clip =
  case val of {
    Rel _ Nothing ->
      (Nothing, Nothing, Just "No marked element present!");
    Rel edges (Just j) ->
      let {i = decodeRelElemPath path;
           rhs' = "addEdgeRel " ++ show j ++ " " ++ show i ++ " (" ++ rhs ++ ")"
      } in (Just rhs', Nothing, Just "OK")
  }

--
-- LOCN 50 9400 true
-- STYLE Hide
--
egRel
--
-- LOCN 700 9375 true
-- STYLE 
--
relDeleteEdgeFn val path x y lhs rhs clip =
  case val of {
    Rel _ Nothing ->
      (Nothing, Nothing, Just "No marked element present!");
    Rel edges (Just j) ->
      let {i = decodeRelElemPath path;
           rhs' = "deleteEdgeRel " ++ show j ++ " " ++ show i ++ " (" ++ rhs ++ ")"
      } in (Just rhs', Nothing, Just "OK")
  }
--
-- LOCN 50 9575 true
-- STYLE Hide
--
deleteEdgeRel :: Int -> Int -> Rel -> Rel
--
-- LOCN 50 10650 true
-- STYLE 
--
deleteEdgeRel i j rel =
  case rel of 
    Rel edges mm -> 
      let edges' = [edge | edge <- edges, ((i /= fst edge) || (j /= snd edge))]
      in Rel edges' mm
--
-- LOCN 50 10675 true
-- STYLE Hide
--
edgeEq i j edge =
  (i == fst edge) && (j == snd edge)
--
-- LOCN 50 10775 true
-- STYLE Hide
--
addEdgeRel  5 7 egRel
--
-- LOCN 650 10450 true
-- STYLE 
--
inverseRel :: Rel -> Rel
--
-- LOCN 50 10850 true
-- STYLE 
--
inverseRel rel =
  case rel of
    Rel edges mm -> Rel (map swap edges) mm
--
-- LOCN 50 10875 true
-- STYLE Hide
--
symmClosureRel :: Rel -> Rel
--
-- LOCN 50 11200 true
-- STYLE 
--
symmClosureRel rel =
  case rel of
    Rel edges mm -> 
      let edges' = concat [ [e, swap e] | e <- edges]
      in Rel edges' mm
--
-- LOCN 50 11225 true
-- STYLE Hide
--
swap edge = (snd edge, fst edge)
--
-- LOCN 50 10950 true
-- STYLE Hide
--
symmClosureRel egRel
--
-- LOCN 700 11225 true
-- STYLE 
--
relInverseFn val path x y lhs rhs clip =
  let rhs' = "inverseRel (" ++ rhs ++ ")"
  in (Just rhs', Nothing, Just "OK")
--
-- LOCN 50 9750 true
-- STYLE Hide
--
relSymmClosureFn val path x y lhs rhs clip =
  let rhs' = "symmClosureRel (" ++ rhs ++ ")"
  in (Just rhs', Nothing, Just "OK")
--
-- LOCN 50 9950 true
-- STYLE Hide
--
relEvaluateFn val path x y lhs rhs clip =
  (Just (showRel val), Nothing, Just "OK")
--
-- LOCN 50 10025 true
-- STYLE Hide
{-
"locnExtra i" yields a location a distance radius' beyond the location of the i-th element of
the set. (It is used for displaying reflexive edges.)
-}
-- LOCN 50 625 true
-- STYLE comment
--
locnExtra extra i = 
  let r = radius + fromInt extra
  in location r i
--
-- LOCN 50 725 true
-- STYLE Hide
--
locn :: Int -> (Int, Int)
--
-- LOCN 50 525 true
-- STYLE 
--
locn i = location radius i
--
-- LOCN 50 550 true
-- STYLE Hide
--
locnExtra :: Int -> Int -> (Int, Int)
--
-- LOCN 50 700 true
-- STYLE 
--
displaySet (Just 2) egSet1
--
-- LOCN 475 3100 true
-- STYLE Picture
--
displayEdge edge =
  let {i = fst edge; 
       j = snd edge
  } in if (i == j) then
         let {p = locnExtra loopRadius i;
              diam = 2 * loopRadius;
              x = fst p - loopRadius; 
              y = snd p - loopRadius
         } in Trans x y (Ellipse diam diam trans edgeColor)
       else
         let {p = locn i;
              q = locn j
         } in vector (fst p) (snd p) (fst q) (snd q)
--
-- LOCN 50 7375 true
-- STYLE Hide
--
loopRadius = (2 * side) `div` 3
--
-- LOCN 50 7325 true
-- STYLE Hide
--
reflexClosureRel :: Rel -> Rel
--
-- LOCN 50 11025 true
-- STYLE 
--
reflexClosureRel rel =
  case rel of
    Rel edges mm -> 
      let edges' = [(i,i) | i <- intLabels]
      in Rel (edges' ++ edges) mm
--
-- LOCN 50 11050 true
-- STYLE Hide
--
reflexClosureRel egRel
--
-- LOCN 694 10955 true
-- STYLE 
--
relReflexClosureFn val path x y lhs rhs clip =
  let rhs' = "reflexClosureRel (" ++ rhs ++ ")"
  in (Just rhs', Nothing, Just "OK")
--
-- LOCN 50 9850 true
-- STYLE Hide
{-
Composition & transitive closure
-}
-- LOCN 50 11450 true
-- STYLE heading1
--
edgesOf rel = case rel of Rel edges _ -> edges
--
-- LOCN 50 11500 true
-- STYLE 
--
instance Edit (a -> b) where
  style f = Hide
--
-- LOCN 725 11500 true
-- STYLE 
{-
Composition: this function relies upon "compEdges", which actually does the real work
-}
-- LOCN 50 11575 true
-- STYLE comment
--
compRel :: Rel -> Rel -> Rel
--
-- LOCN 50 11625 true
-- STYLE 
--
compRel rel1 rel2 =
  let {edges1 = edgesOf rel1;
       edges2 = edgesOf rel2;
       edges  = nub2 (compEdges edges1 edges2)
  } in Rel edges Nothing
--
-- LOCN 50 11650 true
-- STYLE 
--
isLinked :: Edge -> Edge -> Bool
--
-- LOCN 50 11925 true
-- STYLE 
--
isLinked edge1 edge2 =
  snd edge1 == fst edge2
--
-- LOCN 50 11950 true
-- STYLE 
--
emptyRel = Rel [] Nothing
--
-- LOCN 50 12350 true
-- STYLE Hide
--
rel1 = markRel 0 (deleteEdgeRel 3 0 (markRel 3 (markRel 1 (deleteEdgeRel 3 1 (markRel 3 (deleteEdgeRel 3 1 (markRel 3 (addEdgeRel 1 3 (markRel 1 (addEdgeRel 0 3 (markRel 0 (emptyRel))))))))))))
--
-- LOCN 25 12450 true
-- STYLE 
--
rel2 = Rel [(1,3), (5,6), (4,5), (3,4)] (Just 1)
--
-- LOCN 350 12500 true
-- STYLE 
--
rel12 = compRel rel1 rel2
--
-- LOCN 675 12550 true
-- STYLE 
--
nEdgeEq edge1 edge2 = not (edgeEq' edge1 edge2)
--
-- LOCN 50 12100 true
-- STYLE 
--
nub2 xs =
    case xs of {
      []     -> [];
      x:xs   -> let xs' = filter (nEdgeEq x) xs
                in x : nub2 xs'
    }
--
-- LOCN 50 12150 true
-- STYLE 
--
edgeEq' edge1 edge2 = 
  (fst edge1 == fst edge2) && (snd edge1 == snd edge2)
--
-- LOCN 50 12050 true
-- STYLE 
{-
iterate f x = [x, f x, (f . f) x, ...]
-}
-- LOCN 625 13575 true
-- STYLE comment
--
transitiveComps :: Edges -> [Edges]
--
-- LOCN 25 13550 true
-- STYLE 
{-
The function "transitiveComps" forms a list of the 0th, 1st, 2nd, compositions of a relation with itself  
-}
-- LOCN 25 13500 true
-- STYLE comment
--
compEdges :: Edges -> Edges -> Edges
--
-- LOCN 50 11825 true
-- STYLE 
--
compEdges edges1 edges2 =
  concat [ [(fst edge1, snd edge2) | edge2 <- edges2, isLinked edge1 edge2]  | edge1 <- edges1]
--
-- LOCN 50 11850 true
-- STYLE 
{-
Compute the composition of these lists of edges. Note: this does NOT apply nub, so there may be duplicates
in the result
-}
-- LOCN 50 11775 true
-- STYLE comment
{-
Test of relational composition
-}
-- LOCN 50 12275 true
-- STYLE comment
{-
A variant of the edgeEq  predicate defined earlier
-}
-- LOCN 50 12025 true
-- STYLE comment
{-
Compute the transitive closure of a list of edges
-}
-- LOCN 50 12775 true
-- STYLE heading1
--
extend :: Edges -> Edges -> Edges
--
-- LOCN 50 12850 true
-- STYLE 
{-
extend R S ---> R union (R * S)
-}
-- LOCN 50 12825 true
-- STYLE comment
--
extend edges1 edges2 =
  nub2 (edges1 ++ (edges1 `compEdges` edges2))
--
-- LOCN 50 12875 true
-- STYLE 
--
extendRel :: Rel -> Rel -> Rel
--
-- LOCN 550 12850 true
-- STYLE 
--
extendRel rel1 rel2
  = let edges = extend (edgesOf rel1) (edgesOf rel2)
    in Rel edges Nothing
--
-- LOCN 550 12875 true
-- STYLE 
--
r0 = Rel [(2,3), (5,6), (4,5), (3,4)] (Just 2)
--
-- LOCN 50 12950 true
-- STYLE 
--
r1 = extendRel r0 r0
--
-- LOCN 475 12975 true
-- STYLE 
--
r2 = extendRel r0 r1
--
-- LOCN 57 13197 true
-- STYLE 
--
r3 = extendRel r0 r2
--
-- LOCN 475 13225 true
-- STYLE 
--
r3a = extendRel r1 r2
--
-- LOCN 853 12994 true
-- STYLE 
--
transitiveComps edges = iterate (extend edges) edges
--
-- LOCN 25 13575 true
-- STYLE 
--
e0 = edgesOf r0
--
-- LOCN 28 13615 true
-- STYLE 
--
tcs = transitiveComps e0
--
-- LOCN 25 13675 true
-- STYLE 
--
mkRel edges = Rel edges Nothing
--
-- LOCN 50 11525 true
-- STYLE 
--
mkRel (tcs !! 0)
--
-- LOCN 34 13745 true
-- STYLE 
--
mkRel (tcs !! 1)
--
-- LOCN 375 13750 true
-- STYLE 
--
mkRel (tcs !! 2)
--
-- LOCN 716 13746 true
-- STYLE 
--
mkRel (tcs !! 3)
--
-- LOCN 1093 13740 true
-- STYLE 
--
mkRel (tcs !! 4)
--
-- LOCN 1450 13750 true
-- STYLE 
--
take 10 (map length tcs)
--
-- LOCN 50 14000 true
-- STYLE 
--
fixPoint :: [Edges] -> Edges
--
-- LOCN 50 14250 true
-- STYLE 
--
transClosure :: Rel -> Rel
--
-- LOCN 50 14125 true
-- STYLE 
--
fixPoint edgess =
  case edgess of
    edges1 : (edges2 : edgess) -> 
      if length edges1 == length edges2 then
         edges1
      else
         fixPoint (tail edgess)
--
-- LOCN 50 14275 true
-- STYLE 
{-
The top-level function
-}
-- LOCN 50 14100 true
-- STYLE comment
--
transClosure r0
--
-- LOCN 550 14125 true
-- STYLE 
--
transClosure rel =
  let {edges = edgesOf rel;
       edges' = fixPoint (transitiveComps edges)
  } in mkRel edges'
--
-- LOCN 50 14150 true
-- STYLE 
--
rel = addEdgeRel 2 6 (addEdgeRel 2 2 (markRel 2 (addEdgeRel 7 0 (markRel 7 (addEdgeRel 6 2 (addEdgeRel 6 7 (markRel 6 (emptyRel))))))))
--
-- LOCN 100 14457 true
-- STYLE 
--
transClosure rel
--
-- LOCN 575 14500 true
-- STYLE 
{-
Back to previous document
-}
-- LOCN 69 14742 true
-- STYLE open1 ""
{-
Back to previous document
-}
-- LOCN 775 150 true
-- STYLE open1 ""
{-
The original (unretouched!) document in which the types and functions introduced in 
the RelLib library module (for supporting the presentation and manipulation of
sets and relations) were originally developed.
-}
-- LOCN 50 -75 true
-- STYLE abstract
