-- Vital 2.1
--
import SonySS
--
-- LOCN 825 75 true
-- STYLE 
{-
Visual metaphors for abstract types
-}
-- LOCN 25 75 true
-- STYLE title
{-
Aim is to be able to create Visual Metaphors for arbitrary ADTs so
that users can operate on visual representations of ADTs
-}
-- LOCN 75 275 true
-- STYLE comment2
{-
For example, the "Stack"  ADT might be implemented as:
-}
-- LOCN 75 475 true
-- STYLE comment2
{-
module Stack (Stack(), empty, push, pop, top) where

    data Stack a = Stk [a]

    push x (Stk xs) = Stk (x : xs)
    . . .
-}
-- LOCN 175 675 true
-- STYLE commentCode
{-
. . . but we might want to use a different representation to visualise stacks:
-}
-- LOCN 75 1000 true
-- STYLE comment2
--
stack = 
 PicImage 
  (imageFile 
   "food/soupPlates.jpg")
--
-- LOCN 175 1150 true
-- STYLE Hide
{-
. . . with the intention that the visualisation is a metaphor for a real stack, 
that is, that both implementation and visualisation obey the same laws:

    pop (push x s) = s
    top (push x s) = x
    . . . 
-}
-- LOCN 75 1525 true
-- STYLE comment2
--
stack
--
-- LOCN 125 1125 true
-- STYLE Picture
{-
To visualise a value of an ADT we need to:

  *  define a mapping to some viewable type

  *  define a style for that type

The "View" constructor packages these two items together
-}
-- LOCN 75 2050 true
-- STYLE comment2
{-
Visualising ADTs
-}
-- LOCN 25 1875 true
-- STYLE comment1
{-
For example, to display a Bool value (!) as a tick or a cross:
-}
-- LOCN 75 2375 true
-- STYLE comment2
--
boolRep b = 
  let name = if b then "logical/tick.gif" else "logical/cross.gif"
  in picFile name
--
-- LOCN 150 2675 true
-- STYLE Hide
--
instance Edit Bool where
  style b = boolStyle
--
-- LOCN 150 3375 true
-- STYLE 
--
2 * 2 == 4
--
-- LOCN 150 3625 true
-- STYLE 
--
2 + 2 == 5
--
-- LOCN 450 3625 true
-- STYLE 
--
boolStyle = View boolRep Picture
--
-- LOCN 150 3100 true
-- STYLE Hide
{-
And (optionally) make this style the default one for Bool:
-}
-- LOCN 150 3250 true
-- STYLE comment2
--
boolRep True
--
-- LOCN 500 2800 true
-- STYLE Picture
{-
Back to main document
-}
-- LOCN 75 3800 true
-- STYLE open1 ""
{-
First define a mapping from Bool to Pic
-}
-- LOCN 150 2550 true
-- STYLE comment2
{-
For example:
-}
-- LOCN 275 2800 true
-- STYLE comment2
{-
Then use the "View" constructor to define a style based on this:
-}
-- LOCN 150 2975 true
-- STYLE comment2
{-
Then, by default, Bool values will be shown in this style
-}
-- LOCN 150 3525 true
-- STYLE comment2
--
boolRep :: Bool -> Pic
--
-- LOCN 150 2625 true
-- STYLE 
--
title = enlarge comment1
--
-- LOCN 25 -50 true
-- STYLE Hide
