-- Vital 2.1
--
import SonySS
--
-- LOCN 825 75 true
-- STYLE 
{-
Manipulating ADTs
-}
-- LOCN 25 75 true
-- STYLE title
{-
Given that we can display ADTs pictorially, we now want to be able to
manipulate them:

  *  Selecting component parts and applying operations to them

  *  Selecting component parts and dragging them around

(As before, we want changes to be reflected in the text of the Haskell document)
-}
-- LOCN 100 200 true
-- STYLE comment2
--
rel1
--
-- LOCN 100 1750 true
-- STYLE 
{-
Example:  Metaphor for the Rel type
-}
-- LOCN 100 1475 true
-- STYLE comment1
{-
Operations on elements:  Mark, Add edge, Delete edge

Operations on the relation: Unmark, Invert, 
                            Reflexive/Symmetric closure, 
                            Union, Intersection, 
                            Evaluate
-}
-- LOCN 550 1825 true
-- STYLE comment2
--
rel1, rel2 :: Rel
--
-- LOCN 100 1550 true
-- STYLE Hide
{-
Implementation
-}
-- LOCN 100 2400 true
-- STYLE comment1
{-
The "Edit" class includes an "edit" method that defines the menu
of operations that appears when a user right-clicks on an ADT
-}
-- LOCN 100 2525 true
-- STYLE comment2
{-
class Edit a where
  style :: a -> Style

  edit :: a -> Path -> [(Name, EditFn)]
-}
-- LOCN 100 2825 true
-- STYLE commentCode
{-
When the user clicks on a click-sensitive component of the representation
of a type, the "edit"  method pops up the menu
-}
-- LOCN 100 3025 true
-- STYLE comment2
{-
When the user selects a menu entry, the corresponding function is invoked
-}
-- LOCN 100 3525 true
-- STYLE comment2
{-
editFn  val path x y lhs rhs clipboard  --->  (rhs', clipboard', message')
-}
-- LOCN 100 3650 true
-- STYLE commentCode
{-
It yields a triple of (Maybe) strings:

  - replacement Haskell text

  - info to store in the clipboard

  - a status message for the user
-}
-- LOCN 100 3825 true
-- STYLE comment2
{-
  interFn val path x y lhs rhs clip =
    let rhs' = "(" ++ clip ++ ") `inter` (" ++ rhs ++ ")"
    in (Just rhs', Nothing, Just "OK");


-}
-- LOCN 100 4325 true
-- STYLE commentCode
{-
For example, the definition of the "Intersect with clipboard" function is:
-}
-- LOCN 100 4150 true
-- STYLE comment2
{-
Back to main document
-}
-- LOCN 100 5650 true
-- STYLE open1 ""
{-
Example: a metaphor for the "Set" type
-}
-- LOCN 100 500 true
-- STYLE comment1
--
set1 = emptySet
--
-- LOCN 100 725 true
-- STYLE Hide
--
set1
--
-- LOCN 100 825 true
-- STYLE 
{-
The base set is assumed to be:
    {A, B, ..., H}
-}
-- LOCN 600 700 true
-- STYLE comment2
--
import RelLib
--
-- LOCN 100 600 true
-- STYLE 
--
set2
--
-- LOCN 100 1150 true
-- STYLE 
{-
Operations on elements:
    delete, insert

Operations on sets:
    copy to clipboard,
    complement,
    union, intersection
    evaluate
    
-}
-- LOCN 600 900 true
-- STYLE comment2
--
set2 = emptySet
--
-- LOCN 100 750 true
-- STYLE Hide
--
set1, sets2 :: Set
--
-- LOCN 100 675 true
-- STYLE 
--
rel1 = emptyRel
--
-- LOCN 103 1588 true
-- STYLE Hide
--
rel2 = emptyRel
--
-- LOCN 100 1625 true
-- STYLE Hide
--
rel2
--
-- LOCN 100 2025 true
-- STYLE 
{-
It defines a (context-sensitive) list of  name-operation pairs
-}
-- LOCN 100 2700 true
-- STYLE comment2
--
emptySet
--
-- LOCN 100 3175 true
-- STYLE 
{-
The Edit class also has an "adjust" method that allows a parameter
of an ADT to be adjusted by right-mouse drag.
-}
-- LOCN 100 4825 true
-- STYLE comment2
{-
Adjusting the parameters of an ADT
-}
-- LOCN 100 4675 true
-- STYLE comment1
{-
For example, the size of a rectangle can be controlled by
-}
-- LOCN 100 5025 true
-- STYLE comment2
--
instance Edit Pic where {
  adjust val path x y lhs rhs =
    let { x' = " (" ++ show x ++ ") ";
          y' = " (" ++ show y ++ ") ";
          text = "SelRect" ++ x' ++ y' ++ "yellow red"
    } in   (Just text, Nothing)
}
--
-- LOCN 100 5125 true
-- STYLE Hide
--
SelRect 100  100 yellow red
--
-- LOCN 100 5375 true
-- STYLE Picture
{-
A challenge for the speed of
incremental compilation!
-}
-- LOCN 625 5400 true
-- STYLE comment2
--
title = enlarge comment1
--
-- LOCN 37 -88 true
-- STYLE Hide
