-- Vital 2.1
--
import SS
--
-- LOCN 725 75 true
-- STYLE 
{-
Development of the DNA library
-}
-- LOCN 50 75 true
-- STYLE title
{-
Step-by-step interactive development of the DNA types and functions that
were used as the basis for the DNA_lib library module.
-}
-- LOCN 50 -75 true
-- STYLE abstract
{-
DNA molecules are built up from a set of 4 bases (adenine, cytosine, guanine
and thymine). These are conventionally referred to by their initial letters: A, C, G and T.
We introduce a 5th possibility, X, to allow for erroneous or undefined bases.
-}
-- LOCN 50 350 true
-- STYLE comment
--
data Base = A | C | G | T |    X
--
-- LOCN 50 450 true
-- STYLE 
{-
Bases
-}
-- LOCN 50 300 true
-- STYLE heading1
{-
Functions to show and to parse bases:
-}
-- LOCN 50 525 true
-- STYLE comment
--
showBase :: Base -> String
--
-- LOCN 50 575 true
-- STYLE 
--
showBase b =
  case b of {
    A -> "A";
    C -> "C";
    G -> "G";
    T -> "T";
    X -> "X"
  }
--
-- LOCN 50 625 true
-- STYLE 
--
parseBase :: Char -> Base
--
-- LOCN 50 1000 true
-- STYLE 
--
parseBase ch = 
  case ch of {
    'A' -> A; 
    'C' -> C;
    'G' -> G;
    'T' -> T;
     _  -> X
  }
--
-- LOCN 50 1025 true
-- STYLE 
--
showBase T
--
-- LOCN 675 625 true
-- STYLE 
--
parseBase 'G'
--
-- LOCN 675 1025 true
-- STYLE 
--
showBases :: [Base] -> String
--
-- LOCN 50 800 true
-- STYLE 
--
showBases bs = '[' : showBases' bs
--
-- LOCN 50 825 true
-- STYLE 
--
showBases' bases =
  case bases of {
    []   -> "]";
    [b]  -> showBase b ++ "]";
    b:bs -> showBase b ++ ", " ++ showBases' bs
  }
--
-- LOCN 50 875 true
-- STYLE 
--
showBases []
--
-- LOCN 675 800 true
-- STYLE 
--
showBases [A, X, C, T, G]
--
-- LOCN 675 900 true
-- STYLE 
--
instance Show Base where
  show b = showBase b
--
-- LOCN 50 1250 true
-- STYLE 
{-
DNA molecules
-}
-- LOCN 50 1350 true
-- STYLE heading1
{-
A molecule of DNA consists of a sequence of base-pairs: (A,T) or (T,A) or (C,G) or (G,C).  By convention, each pair is
identified by its first component.
-}
-- LOCN 50 1400 true
-- STYLE comment
--
data DNA = DNA [Base]
--
-- LOCN 50 1475 true
-- STYLE 
--
showDNA :: DNA -> String
--
-- LOCN 50 1750 true
-- STYLE 
{-
Functions to parse and show DNA 
-}
-- LOCN 50 1550 true
-- STYLE comment
--
showDNA dna =
  case dna of
    DNA bases -> "DNA " ++ showBases bases
--
-- LOCN 50 1775 true
-- STYLE 
--
parseDNA :: String -> DNA
--
-- LOCN 50 1600 true
-- STYLE 
--
parseDNA s = 
  let {s' = filter (/= ' ') s;         -- ignore spaces
       bases = map parseBase s'
  } in DNA bases
--
-- LOCN 50 1650 true
-- STYLE 
--
eg = parseDNA  "CGGT ACTT GCCA"
--
-- LOCN 650 1650 true
-- STYLE defaultStyle
--
showDNA eg
--
-- LOCN 650 1750 true
-- STYLE 
--
parseBase 'B'
--
-- LOCN 675 1100 true
-- STYLE 
{-
Functions to display a base pair
-}
-- LOCN 50 2050 true
-- STYLE heading2
{-
A DNA molecule is composed of a sequence of base-pairs, each of which is suspended between
a pair of backbones composed of sugar/phosphate components. The backbone components
are displayed as:
-}
-- LOCN 50 2100 true
-- STYLE comment
--
backbone, backbone' :: Pic
--
-- LOCN 50 2200 true
-- STYLE 
{-
The pictures defined in this document
use both the primitive picture-forming
constructors (defined in the Pic 
module) together with some higher-
level combinators (defined in the 
Pictures library module).
-}
-- LOCN 725 125 true
-- STYLE comment
--
import Pictures
--
-- LOCN 725 250 true
-- STYLE 
--
backbone =
  let {phosphate = Rect 6 6 white black;
       sugar = Poly [(15,7), (24,15), (20,25), (10,25), (6,15)] white black;
       upperLink = line 3 6 6 15 black;
       sideLink  = line 24 15 29 15 black;
       lowerLink = line 10 25 3 30 black
  } in super [sugar, phosphate, upperLink, sideLink, lowerLink]
--
-- LOCN 50 2225 true
-- STYLE 
--
line :: Int -> Int -> Int -> Int -> Pic
--
-- LOCN 50 2575 true
-- STYLE 
--
line x1 y1 x2 y2 c
  = let {x' = x2 - x1;
         y' = y2 - y1
    } in Trans x1 y1 (Line x' y' c)
--
-- LOCN 50 2600 true
-- STYLE 
--
super :: [Pic] -> Pic
--
-- LOCN 50 2725 true
-- STYLE 
--
super pics =
  case pics of {
    []      -> NoPic;
    pic:pics -> pic <+> super pics
  }
--
-- LOCN 50 2750 true
-- STYLE 
{-
Subsidiary functions used in the above definition
-}
-- LOCN 50 2525 true
-- STYLE comment
--
scale 5.0 backbone
--
-- LOCN 700 2350 true
-- STYLE 
--
backbone' = flipH (flipV backbone)
--
-- LOCN 50 2400 true
-- STYLE 
{-
Enlarged:
-}
-- LOCN 700 2325 true
-- STYLE comment
{-
Diagrams for the individual bases
-}
-- LOCN 50 2875 true
-- STYLE comment
--
smallBaseWidth = 14
--
-- LOCN 50 2925 true
-- STYLE hide
--
bigBaseWidth = 27
--
-- LOCN 50 2975 true
-- STYLE hide
--
bondWidth = 8
--
-- LOCN 50 2950 true
-- STYLE hide
--
baseHeight = 16
--
-- LOCN 450 2925 true
-- STYLE hide
--
totalWidth = bigBaseWidth + bondWidth + smallBaseWidth
--
-- LOCN 50 3000 true
-- STYLE hide
--
base :: Color -> Color -> Bool -> String -> Pic
--
-- LOCN 50 3050 true
-- STYLE 
--
base lc bc isBig s =
  let {w      = if isBig then bigBaseWidth else smallBaseWidth;
       offset = if isBig then 9 else 3;
       rect   = Rect w baseHeight bc black;
       fmt    = Format "SansSerif" 12 lc True False;
       text   = PicText fmt s
  } in rect <+> Trans offset 13 text
--
-- LOCN 50 3075 true
-- STYLE 
--
adenine, cytosine, guanine, thymine :: Pic
--
-- LOCN 50 3225 true
-- STYLE 
--
adenine  = base black yellow True "A"
--
-- LOCN 100 3250 true
-- STYLE 
--
cytosine = base white green False "C"
--
-- LOCN 100 3300 true
-- STYLE 
--
guanine  = base white blue True "G"
--
-- LOCN 100 3350 true
-- STYLE 
--
thymine  = base white red False "T"
--
-- LOCN 100 3400 true
-- STYLE 
--
bonds :: Bool -> Pic
--
-- LOCN 50 3500 true
-- STYLE 
--
bonds isTriple =
  let {bond = Rect bondWidth 1 red red;
       ys   = if isTriple then [3, 7, 11] else [5, 9]
  } in super [Trans 0 y bond| y <- ys]
--
-- LOCN 50 3525 true
-- STYLE 
--
bonds2, bonds3 :: Pic
--
-- LOCN 50 3625 true
-- STYLE 
--
bonds2 = bonds False
--
-- LOCN 100 3650 true
-- STYLE 
--
bonds3 = bonds True
--
-- LOCN 100 3700 true
-- STYLE 
{-
Diagrams for base pairs
-}
-- LOCN 50 3800 true
-- STYLE comment
--
cg, gc, ta, at, other :: Pic
--
-- LOCN 50 3850 true
-- STYLE 
--
cg = super [Trans smallBaseWidth 0 bonds3, cytosine, Trans (smallBaseWidth + bondWidth) 0 guanine]
--
-- LOCN 100 3875 true
-- STYLE 
--
gc = super [Trans bigBaseWidth   0 bonds3, guanine,  Trans (bigBaseWidth   + bondWidth) 0 cytosine]
--
-- LOCN 100 3950 true
-- STYLE 
--
ta = super [Trans smallBaseWidth 0 bonds2, thymine,  Trans (smallBaseWidth + bondWidth) 0 adenine]
--
-- LOCN 100 4025 true
-- STYLE 
--
at = super [Trans bigBaseWidth   0 bonds2, adenine,  Trans (bigBaseWidth   + bondWidth) 0 thymine]
--
-- LOCN 100 4100 true
-- STYLE 
--
other = Rect totalWidth baseHeight white cyan
--
-- LOCN 100 4175 true
-- STYLE 
{-
Selectable rectangle (for selection of base pairs)
-}
-- LOCN 50 4250 true
-- STYLE comment
--
selRect :: Pic
--
-- LOCN 50 4300 true
-- STYLE 
--
selRect = SelRect totalWidth baseHeight trans trans
--
-- LOCN 50 4325 true
-- STYLE 
{-
(Note that this rectangle is invisible
 unless selected.)
-}
-- LOCN 625 4325 true
-- STYLE comment
{-
Diagrams for base-pairs attached to backbone elements
-}
-- LOCN 50 4425 true
-- STYLE comment
--
attach :: Pic -> Pic
--
-- LOCN 50 4475 true
-- STYLE 
--
attach basePair = 
  super [backbone, Trans 29 8 (basePair <+> selRect), Trans (totalWidth + 29) 0 backbone']
--
-- LOCN 50 4500 true
-- STYLE 
--
cg', gc', ta', at' :: Pic
--
-- LOCN 50 4550 true
-- STYLE 
--
cg' = attach cg
--
-- LOCN 100 4600 true
-- STYLE 
--
gc' = attach gc
--
-- LOCN 100 4675 true
-- STYLE 
--
ta' = attach ta
--
-- LOCN 425 4600 true
-- STYLE 
--
at' = attach at
--
-- LOCN 425 4675 true
-- STYLE 
--
other' = attach other
--
-- LOCN 700 4600 true
-- STYLE 
--
displayBasePair :: Base -> Pic
--
-- LOCN 50 4775 true
-- STYLE 
--
displayBasePair base =
  case base of {
    A     -> at';
    C     -> cg';
    G     -> gc';
    T     -> ta';
    X     -> other'
  }
--
-- LOCN 50 4800 true
-- STYLE 
{-
Shadow effect
-}
-- LOCN 50 5000 true
-- STYLE heading2
{-
Function to create a monochrome copy of a picture
-}
-- LOCN 50 5050 true
-- STYLE comment
--
monochrome :: Color -> Pic -> Pic
--
-- LOCN 50 5100 true
-- STYLE 
--
monochrome c pic =
  case pic of {
    NoPic            -> NoPic;
    Line x y _       -> Line x y c;
    Rect w h _ _     -> Rect w h c c;
    Ellipse w h _ _  -> Ellipse w h c c;
    PicText fmt s    -> PicText (monochromeFmt c fmt) s;
    SelRect w h _ _  -> NoPic;                                          -- This kind of element is not shadowed
    Poly vs _ _      -> Poly vs c c;
    Super pic pic'   -> Super (monochrome c pic) (monochrome c pic');
    Trans x y pic    -> Trans x y (monochrome c pic);
    Animation f      -> Animation (\t -> monochrome c (f t));
    Interaction s0 f -> Interaction s0
                                    (\t x y b s ->
                                       case f t x y b s of 
                                         (s, pic) -> (s, monochrome c pic))
  }
--
-- LOCN 50 5125 true
-- STYLE 
--
monochromeFmt c fmt =
  case fmt of
    Format font ps _ b i -> Format font ps c b i
--
-- LOCN 50 5450 true
-- STYLE 
--
monochromeFmt :: Color -> Format -> Format
--
-- LOCN 50 5425 true
-- STYLE 
--
shadowed :: Int -> Int -> Color -> Pic -> Pic
--
-- LOCN 50 5550 true
-- STYLE 
--
shadowed dx dy c pic =
  let pic' = Trans dx dy (monochrome c pic)
  in pic' <+> pic
--
-- LOCN 50 5575 true
-- STYLE 
--
withShadow :: Pic -> Pic
--
-- LOCN 50 5700 true
-- STYLE 
--
withShadow = shadowed 2 3 lightGray
--
-- LOCN 50 5725 true
-- STYLE hide
{-
Function to create a light grey shadow
-}
-- LOCN 50 5675 true
-- STYLE comment
{-
Functions to display a DNA molecule with/without shadow effect
-}
-- LOCN 50 5875 true
-- STYLE comment
{-
Display of DNA molecules
-}
-- LOCN 50 5825 true
-- STYLE heading2
--
displayDNA, displayDNA' :: DNA -> Pic
--
-- LOCN 50 5925 true
-- STYLE 
--
displayDNA dna =
  case dna of 
    DNA bases -> let pics = map displayBasePair bases
                 in above pics
--
-- LOCN 50 5950 true
-- STYLE 
--
(withShadow . displayDNA) eg
--
-- LOCN 350 6050 true
-- STYLE 
--
displayDNA eg
--
-- LOCN 50 6050 true
-- STYLE 
--
scale 5.0 at'
--
-- LOCN 425 4800 true
-- STYLE 
--
scale 0.6 (displayDNA eg)
--
-- LOCN 725 6050 true
-- STYLE 
{-
Amino acids
-}
-- LOCN 50 6525 true
-- STYLE heading1
--
data Amino 
  = Ala | Arg | Asn | Asp | Cys | Gln | Glu | Gly | His | Ile
  | Leu | Lys | Met | Phe | Pro | Ser | Thr | Trp | Tyr | Val
--
-- LOCN 50 6625 true
-- STYLE 
{-
DNA, via a process known as transcription, defines the synthesis of proteins out of amino acids.
There are some 20 amino acids involved, conventionally abbreviated as:
-}
-- LOCN 50 6575 true
-- STYLE comment
{-
The full names of these amino acids are defined by:
-}
-- LOCN 51 6714 true
-- STYLE comment
--
showAmino amino =
  case amino of {
    Ala    -> "Alanine";
    Arg    -> "Arginine";
    Asn    -> "Asparagine";
    Asp    -> "Aspartic acid";
    Cys    -> "Cysteine";
    Gln    -> "Glutamine";
    Glu    -> "Glutamic acid";
    Gly    -> "Glycine";
    His    -> "Histidine";
    Ile    -> "Isoleucine";
    Leu    -> "Leucine";
    Lys    -> "Lysine";
    Met    -> "Methionine";
    Phe    -> "Phenylalanine";
    Pro    -> "Proline";
    Ser    -> "Serine";
    Thr    -> "Threonine";
    Trp    -> "Tryptophan";
    Tyr    -> "Tyrosine";
    Val    -> "Valine"
  }
--
-- LOCN 50 6775 true
-- STYLE 
--
showAmino :: Amino -> String
--
-- LOCN 50 6750 true
-- STYLE 
--
showAmino Asp
--
-- LOCN 525 6825 true
-- STYLE 
--
showAmino Trp
--
-- LOCN 525 7000 true
-- STYLE 
{-
The genetic code
-}
-- LOCN 50 7150 true
-- STYLE heading2
{-
The relation between the sequence of base-pairs on a strand of DNA and the sequence of
amino acids to which it (indirectly) gives rise is expressed by the (near universal) Genetic Code:
    - Transcription starts when the sequence (A, T, G) is encountered.
    - Thereafter, the sequence is grouped into subsequences of 3 base-pairs called codons. Each
      codon (of the 4^3 possible codons) codes either for one of the 20 amino acids or for a STOP
      of the transcription process.
-}
-- LOCN 50 7200 true
-- STYLE comment
{-
The genetic code is expressed by this function:
-}
-- LOCN 50 7425 true
-- STYLE comment
--
encode codon =
  case codon of {
    (X, _, _) -> Nothing;    -- Treat an 'X' in any position as
    (_, X, _) -> Nothing;    -- terminating the transcription
    (_, _, X) -> Nothing;    -- process.

    (T, T, T) -> Just Phe;
    (T, T, C) -> Just Phe;
    (T, T, _) -> Just Leu;
    (C, T, _) -> Just Leu;
    (A, T, G) -> Just Met;
    (A, T, _) -> Just Ile;
    (G, T, _) -> Just Val;
    
    (T, C, _) -> Just Ser;
    (C, C, _) -> Just Pro;
    (A, C, _) -> Just Thr;
    (G, C, _) -> Just Ala;

    (T, A, A) -> Nothing;
    (T, A, G) -> Nothing;
    (T, A, _) -> Just Tyr;
    (C, A, T) -> Just His;
    (C, A, C) -> Just His;
    (C, A, _) -> Just Gln;
    (A, A, T) -> Just Asn;
    (A, A, C) -> Just Asn;
    (A, A, _) -> Just Lys;
    (G, A, T) -> Just Asp;
    (G, A, C) -> Just Asp;
    (G, A, _) -> Just Glu;

    (T, G, G) -> Just Trp;
    (T, G, A) -> Nothing;
    (T, G, _) -> Just Cys;
    (C, G, _) -> Just Arg;
    (A, G, T) -> Just Ser;
    (A, G, C) -> Just Ser;
    (A, G, _) -> Just Arg;
    (G, G, _) -> Just Gly
  }
--
-- LOCN 50 7475 true
-- STYLE 
--
encode :: Codon -> Maybe Amino
--
-- LOCN 50 7450 true
-- STYLE 
--
type Codon = (Base, Base, Base)
--
-- LOCN 50 7350 true
-- STYLE 
{-
(The 'Nothing' value corresponds to the 'Stop' code;
  the 'Just' values correspond to amino acids.)
-}
-- LOCN 450 7675 true
-- STYLE comment
--
encode (G, T, A)
--
-- LOCN 450 7750 true
-- STYLE 
--
encode (T, A, G)
--
-- LOCN 450 7850 true
-- STYLE 
--
encode (T, G, X)
--
-- LOCN 450 7975 true
-- STYLE 
{-
The process of transcription
-}
-- LOCN 50 8150 true
-- STYLE heading1
{-
The sequence of bases is scanned sequentially; the subsequence (A, T, G) signals the
START of a gene; thereafter, successive codons are translated
into amino acids until a STOP codon is encountered.  The sequences of codons between the
START and STOP are termed 'genes', the remaining sequences of bases are (somewhat
inaccurately) termed 'junk'.

-}
-- LOCN 50 8200 true
-- STYLE comment
--
type Gene = [Amino]
--
-- LOCN 50 8350 true
-- STYLE 
--
type Junk = [Base]
--
-- LOCN 50 8400 true
-- STYLE 
--
data Transcript = Transcript [Either Junk Gene]
--
-- LOCN 50 8450 true
-- STYLE 
--
transcribe :: DNA -> Transcript
--
-- LOCN 50 8600 true
-- STYLE 
{-
A 'gene' is a sequence of amino acids
-}
-- LOCN 500 8350 true
-- STYLE comment
{-
A sequence of bases between genes is termed 'junk'
-}
-- LOCN 500 8400 true
-- STYLE comment
{-
A 'transcript' is (here) defined as an interleaved 
sequence of junk & genes
-}
-- LOCN 500 8450 true
-- STYLE comment
{-
The proces of transcription maps a molecule of DNA to
its transcript
-}
-- LOCN 500 8600 true
-- STYLE comment
{-
Viewed in three dimensions, DNA consists of an oriented, intertwined pair of strands (a "double helix"). 
The complementary sequences of bases on these two strands are known as 'sense' and 'anti-sense'. In
most cases, it is only the 'sense' strand that is (indirectly) translatable to protein.
-}
-- LOCN 50 1875 true
-- STYLE comment
--
sense :: DNA -> [Base]
--
-- LOCN 50 1950 true
-- STYLE 
--
sense dna = case dna of DNA bs -> bs
--
-- LOCN 50 1975 true
-- STYLE 
--
transcribe = Transcript . scanBases . sense
--
-- LOCN 50 8625 true
-- STYLE hide
{-
The 'scanBases' function used above is defined in terms of the following pair of mutually-recursive
functions: scanBases and scanCodons
-}
-- LOCN 50 9225 true
-- STYLE comment
--
scanBases :: [Base] -> [Either Junk Gene]
--
-- LOCN 50 9300 true
-- STYLE 
--
scanBases bs =
  case bs of {
    []             -> [];
    A : T : G : bs ->                                        -- look for the 'START' sequence (A, T, G)
      case scanCodons bs of
        (aminos, bs') -> Right aminos : scanBases bs';
    b : bs         -> case scanBases bs of {
                        [] -> [Left [b]];
                        Left js : es  -> Left (b:js) : es;
                        es -> Left [b] : es
                       }
  }
--
-- LOCN 50 9325 true
-- STYLE 
--
scanCodons :: [Base] -> (Gene, [Base])
--
-- LOCN 50 9550 true
-- STYLE 
--
scanCodons bs =
  case bs of {
    b1:b2:b3:bs -> 
      let codon = (b1, b2, b3)
      in case encode codon of {
           Nothing -> ([],bs);                               -- "STOP" detected
           Just amino -> case scanCodons bs of
                           (aminos, bs') -> ((amino: aminos), bs')
         };
    _  -> ([], [])
  }
--
-- LOCN 50 9575 true
-- STYLE 
{-
Display of a transcript
-}
-- LOCN 50 9850 true
-- STYLE heading2
--
displayTranscript :: Transcript -> Pic
--
-- LOCN 50 11600 true
-- STYLE 
--
displayTranscript transcript =
  case transcript of Transcript es -> displayTrans es
--
-- LOCN 50 11625 true
-- STYLE 
--
displayTrans :: [Either Junk Gene] -> Pic
--
-- LOCN 50 11750 true
-- STYLE 
--
displayTrans es =
  case es of {
    []     -> NoPic;
    Left  junk : es -> displayJunk junk <-> displayTrans es;
    Right gene : es -> displayMarker True   <-> 
                       displayGene gene <->
                       displayMarker False  <->
                       displayTrans es     
  }
--
-- LOCN 50 11775 true
-- STYLE 
{-
This is the top-level function; it yields a picture of a transcript
-}
-- LOCN 50 11550 true
-- STYLE comment
{-
It is defined in terms of this subsidiary function
-}
-- LOCN 50 11700 true
-- STYLE comment
--
displayJunk :: Junk -> Pic
--
-- LOCN 50 10250 true
-- STYLE 
--
displayJunk junk =
  let y = basePairHeight * length junk
  in Rect transcriptWidth y trans blue
--
-- LOCN 50 10300 true
-- STYLE 
--
basePairHeight =
  case bBox cg' of Just (_, _, _, h) -> h
--
-- LOCN 50 10050 true
-- STYLE 
{-
A 'junk' sequence of bases is depicted as an empty rectangle whose
height is the product of the number of bases with the height of
the depiction of an individual base-pair.
-}
-- LOCN 50 10175 true
-- STYLE comment
{-
The height of the depiction of an individual base-pair 
is obtained from the bounding box of the picture of a base-pair
(for instance, the bounding box for the cg')
-}
-- LOCN 550 10050 true
-- STYLE comment
--
displayGene :: Gene -> Pic
--
-- LOCN 50 10950 true
-- STYLE 
--
displayGene aminos =
  case aminos of {
    []   -> NoPic;
    amino : aminos 
         -> displayAmino amino <-> displayGene aminos
  }
--
-- LOCN 50 10975 true
-- STYLE 
--
displayAmino :: Amino -> Pic
--
-- LOCN 50 10725 true
-- STYLE 
--
transcriptWidth = 120
--
-- LOCN 50 10025 true
-- STYLE hide
--
displayAmino amino =
  let name = showAmino amino
  in displayCodon lightYellow name
--
-- LOCN 50 10750 true
-- STYLE 
--
displayJunk  [G, A, C, C, T]
--
-- LOCN 625 10250 true
-- STYLE 
--
displayGene [Glu, Val, Ala]
--
-- LOCN 625 10950 true
-- STYLE 
--
lightYellow = (lighten . lighten) yellow
--
-- LOCN 50 10825 true
-- STYLE hide
--
displayCodon :: Color -> String -> Pic
--
-- LOCN 50 10500 true
-- STYLE 
--
displayCodon bc s =
  let {h = 3 * basePairHeight;
       rect = Rect transcriptWidth h bc blue;
       text = PicText defaultFmt s
  } in rect <+> Trans 5 (h `div` 2) text
--
-- LOCN 50 10525 true
-- STYLE 
--
displayMarker :: Bool -> Pic
--
-- LOCN 50 11350 true
-- STYLE 
--
displayMarker flag =
  displayCodon yellow (if flag then "START" else "STOP")
--
-- LOCN 50 11375 true
-- STYLE 
--
displayMarker True
--
-- LOCN 625 11350 true
-- STYLE 
--
displayAmino Arg
--
-- LOCN 625 10725 true
-- STYLE 
--
displayCodon cyan "???"
--
-- LOCN 625 10500 true
-- STYLE 
{-
For example, here is a DNA molecule (the spaces shown in the sequence are purely to
aid legibility for the human reader; they are disregarded by the parsing function)
-}
-- LOCN 50 8700 true
-- STYLE comment
--
eg2 = parseDNA seq2
--
-- LOCN 50 8800 true
-- STYLE defaultStyle
--
seq2 = "CTCTGA ATG GCG TTT TAG CT ATG TGG TGA CCCTG"
--
-- LOCN 50 8775 true
-- STYLE hide
{-
. . . and here is its transcript
-}
-- LOCN 50 8900 true
-- STYLE comment
--
transcript2 = transcribe eg2
--
-- LOCN 50 8950 true
-- STYLE 
{-
This describes it as consisting of:
  - a junk sequence ("CTCTGA")
  - a gene consisting of two amino acids (Ala and Phe)
  - a short junk sequence ("CT")
  - a gene consisting of a single amino acid (Trp)
  - a junk sequence ("CCCTGA").
-}
-- LOCN 50 9025 true
-- STYLE comment
{-
Here is the top-level transcription function
-}
-- LOCN 50 8575 true
-- STYLE comment
{-
The following definitions are used for displaying a transcript in pictorial form (with
dimensions to match those used when displaying DNA pictorially)
-}
-- LOCN 50 9900 true
-- STYLE comment
{-
First, the dimensional constants are defined
-}
-- LOCN 50 9975 true
-- STYLE comment
{-
A codon (ie, a triplet of 3 base-pairs) is depicted as a rectangular box
-}
-- LOCN 50 10450 true
-- STYLE comment
{-
An amino acid is depicted as a labelled codon
-}
-- LOCN 50 10675 true
-- STYLE comment
{-
A gene is depicted as a sequence of amino acids
-}
-- LOCN 50 10900 true
-- STYLE comment
{-
The START and STOP codons are depicted as labelled rectangles
-}
-- LOCN 50 11300 true
-- STYLE comment
--
let pic = displayTranscript transcript2
in scale 0.5 pic
--
-- LOCN 625 11600 true
-- STYLE 
{-
Annotated DNA
-}
-- LOCN 50 12175 true
-- STYLE heading2
--
displayAnnotDNA dna =
  let {pic1 = displayDNA dna;
       transcription = transcribe dna;
       pic2 = displayTranscript transcription;
       gap = 20
  } in pic1 <|> Trans gap 0 pic2
--
-- LOCN 50 12325 true
-- STYLE 
--
displayAnnotDNA :: DNA -> Pic
--
-- LOCN 50 12300 true
-- STYLE 
{-
For some purposes, it is handy to couple a display of a DNA transcript with
the display of the DNA itself
-}
-- LOCN 50 12250 true
-- STYLE comment
{-
For example, an annotated picture of the DNA sequence eg2 described earlier is:
-}
-- LOCN 50 12475 true
-- STYLE comment
--
eg2
--
-- LOCN 50 12500 true
-- STYLE defaultStyle
--
displayAnnotDNA eg2
--
-- LOCN 50 12575 true
-- STYLE 
{-
- a junk region
-}
-- LOCN 500 12650 true
-- STYLE comment
{-
- the first gene
-}
-- LOCN 500 12950 true
-- STYLE comment
{-
- a short junk region
-}
-- LOCN 500 13175 true
-- STYLE comment
{-
- the second gene
-}
-- LOCN 499 13323 true
-- STYLE comment
{-
- a junk region
-}
-- LOCN 500 13525 true
-- STYLE comment
{-
Direct manipulation of DNA sequences
-}
-- LOCN 50 13725 true
-- STYLE heading1
{-
The aim is to use the Edit class mechanism to cause DNA values to be displayed in the "annotated
transcript" form developed above and to make the sequence of bases amenable to direct manipulation.
-}
-- LOCN 50 13800 true
-- STYLE comment
{-
The first aim is easy; it is simply a matter of defining the "style" method of the "Edit"
class to use the "View" style constructor to map values of type DNA to an annotated
pictorial view:
-}
-- LOCN 54 13876 true
-- STYLE comment
--
instance Edit DNA where
  style dna = View displayAnnotDNA Picture
--
-- LOCN 59 13971 true
-- STYLE 
--
eg3 = parseDNA "TGATGAAATAG"
--
-- LOCN 50 14125 true
-- STYLE 
{-
With this instance declaration in place, all values of type DNA are now automatically
displayed as annotated transcripts (unless some other style is explicitly set). For example:
-}
-- LOCN 50 14050 true
-- STYLE comment
{-
To allow direct manipulation of the base sequence, it is first necessary to define a set
of functions that will carry out the required operations. We assume that the following
operations are required:
  - delete a base-pair;
  - insert a gap for a new base-pair;
  - change a base-pair to one of AT, TA, CG or GC, or simply replace it with a gap
  - evaluate (ie, simplify) a DNA expression
-}
-- LOCN 50 14525 true
-- STYLE comment
{-
Here are the definitions for the required functions:
-}
-- LOCN 50 14700 true
-- STYLE comment
--
deleteBase :: Int -> DNA -> DNA
--
-- LOCN 100 14800 true
-- STYLE 
--
deleteBase i dna =
  case dna of
    DNA bases -> DNA (delete i bases)
--
-- LOCN 100 14825 true
-- STYLE 
--
delete :: Int -> [a] -> [a]
--
-- LOCN 100 14900 true
-- STYLE 
--
delete i xs =
  case xs of {
    []  -> xs;               -- invalid request!
    x:xs -> if i == 0 then
              xs
            else
              x : delete (i-1) xs
  }
--
-- LOCN 100 14925 true
-- STYLE 
--
dna = parseDNA "TGATGAAT"
--
-- LOCN 650 14800 true
-- STYLE defaultStyle
--
deleteBase 5 dna
--
-- LOCN 650 14875 true
-- STYLE defaultStyle
--
insertGap i dna =
  case dna of
    DNA bases -> DNA (insert i X bases)
--
-- LOCN 100 15150 true
-- STYLE 
{-
Delete a base-pair
-}
-- LOCN 100 14750 true
-- STYLE comment
{-
Insert a gap
-}
-- LOCN 100 15075 true
-- STYLE comment
--
insertGap :: Int -> DNA -> DNA
--
-- LOCN 100 15125 true
-- STYLE 
--
insert :: Int -> a -> [a] -> [a]
--
-- LOCN 100 15225 true
-- STYLE 
--
insert i x xs =
  case xs of {
    [] -> xs;               -- invalid request!
    y:ys -> if i == 0 then
              x : xs
            else 
              y : insert (i-1) x ys
  }
--
-- LOCN 100 15250 true
-- STYLE 
--
insertGap 1 dna
--
-- LOCN 650 15200 true
-- STYLE defaultStyle
--
dna
--
-- LOCN 650 15125 true
-- STYLE defaultStyle
{-
Set a base-pair
-}
-- LOCN 100 15425 true
-- STYLE comment
--
setBaseTo :: Int -> Base -> DNA -> DNA
--
-- LOCN 100 15450 true
-- STYLE 
--
setBaseTo i base dna =
  case dna of
    DNA bases -> DNA (setTo i base bases)
--
-- LOCN 100 15475 true
-- STYLE 
--
setTo :: Int -> a -> [a] -> [a]
--
-- LOCN 100 15550 true
-- STYLE 
--
setTo i x xs =
  case xs of {
    [] -> xs;                   -- invalid request!
    y:ys -> if i == 0 then
               x:ys
            else
               y : setTo (i-1) x ys
    }
--
-- LOCN 100 15575 true
-- STYLE 
--
dna
--
-- LOCN 650 15450 true
-- STYLE defaultStyle
--
setBaseTo 2 C dna
--
-- LOCN 650 15550 true
-- STYLE defaultStyle
{-
Next the program-synthesis functions are written (these are all of a highly-stereotyped form):
-}
-- LOCN 50 15750 true
-- STYLE comment
{-
For deleting a base pair
-}
-- LOCN 100 15800 true
-- STYLE comment
--
deleteFn dna path x y lhs rhs clip =
  let {i = decode path;
       rhs' = "deleteBase " ++ show i ++ " (" ++ rhs ++ ")";
       message' = "Deleting a base"
  } in (Just rhs', Nothing, Just message')
--
-- LOCN 100 15825 true
-- STYLE 
{-
For inserting a gap
-}
-- LOCN 100 15950 true
-- STYLE comment
--
insertFn dna path x y lhs rhs clip =
  let {i = decode path;
       rhs' = "insertGap " ++ show i ++ " (" ++ rhs ++ ")";
       message' = "Inserting a gap"
  } in (Just rhs', Nothing, Just message')
--
-- LOCN 100 15975 true
-- STYLE 
{-
For setting a base
-}
-- LOCN 100 16100 true
-- STYLE comment
--
setToFn base dna path x y lhs rhs clip =
  let {i = decode path;
       rhs' = "setBaseTo " ++ show i ++ " " ++ base ++ " (" ++ rhs ++ ")";
       message' = "Setting a base"
  } in (Just rhs', Nothing, Just message')
--
-- LOCN 100 16125 true
-- STYLE 
{-
For evaluating a DNA expression
-}
-- LOCN 100 16250 true
-- STYLE comment
--
evaluateFn dna path x y lhs rhs clip =
  let { rhs' = showDNA dna;
        message' = "Evaluating the expression"
  } in (Just rhs', Nothing, Just message')
--
-- LOCN 100 16275 true
-- STYLE 
{-
The path-decoding function (that maps the "path" to the selected DNA base to
a numerical index into the list of bases) is simply:
-}
-- LOCN 50 16375 true
-- STYLE comment
--
decode :: Path -> Int
--
-- LOCN 50 16425 true
-- STYLE 
--
decode path = length path - 5
--
-- LOCN 50 16450 true
-- STYLE 
{-
Finally, an instance of the "edit" method of the "Edit" class for the type DNA
can be defined:
-}
-- LOCN 50 16525 true
-- STYLE comment
--
instance Edit DNA where 
  edit dna path = [("Delete base",  deleteFn),
                   ("Insert a gap", insertFn),
                   ("Set to A",     setToFn "A"),
                   ("Set to C",     setToFn "C"),
                   ("Set to G",     setToFn "G"),
                   ("Set to T",     setToFn "T"),
                   ("Unset",        setToFn "X"),
                   ("Evaluate",     evaluateFn)]
--
-- LOCN 50 16575 true
-- STYLE 
--
eg2
--
-- LOCN 50 16925 true
-- STYLE 
{-
Example
-}
-- LOCN 50 16775 true
-- STYLE heading2
{-
With all the above now in place, the user is now able to manipulate DNA sequences by simple
point and click operations, and see the result immediately. For example,  "eg2" was a value
of type DNA defined earlier. It will automatically be displayed as an annotated transcript:
-}
-- LOCN 50 16825 true
-- STYLE comment
{-
To modify this sequence:
  - double-click on a base-pair to select it,
  - then right-click to drop-down a menu of allowable operations, and select
     one of these operations.
-}
-- LOCN 425 17100 true
-- STYLE comment
{-
A good example to try would be a "snip" (a single nucleotide polymorphism) of
one of the three bases that form the Alanine codon. Changing it to another 
base may either change the amino acid or may terminate the gene. Inserting or
deleting a base-pair in that codon will change the remaining amino acids
in that gene, and quite possibly will affect subsequent genes as well.
-}
-- LOCN 425 17275 true
-- STYLE comment
{-
Back to previous document
-}
-- LOCN 50 18100 true
-- STYLE open1 ""
{-
Define a "Base" instance of the "Show" class
-}
-- LOCN 50 1200 true
-- STYLE comment
{-
Enlarged:
-}
-- LOCN 425 4775 true
-- STYLE comment
--
withShadow rect
--
-- LOCN 625 5700 true
-- STYLE 
--
rect = Rect 20 20 yellow black
--
-- LOCN 625 5550 true
-- STYLE 
--
shadowed (-5) 4 cyan rect
--
-- LOCN 625 5625 true
-- STYLE 
{-
(Reduced size diagram)
-}
-- LOCN 625 11575 true
-- STYLE comment
{-
(Click this link for a 
  a detailed description
  of this mechanism.)
-}
-- LOCN 725 15800 true
-- STYLE open "RelDemo"
{-
. . . and press the "Undo" button to return to an earlier state.
-}
-- LOCN 425 17200 true
-- STYLE comment
{-
This module is a tidied-up version of the original module that was interactively
developed and from which the DNA_lib library module was subsequently abstracted.
It provides a good illustration of the incremental, "document and test as you go"
approach that this document-centered mode of programming facilitates.
-}
-- LOCN 50 150 true
-- STYLE commentB
{-
Back to previous document
-}
-- LOCN 725 275 true
-- STYLE open ""
